﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.core.graphics {
	import flash.display.DisplayObject;
	import flash.display.Graphics;
	import flash.display.Sprite;
	import flash.errors.IllegalOperationError;
	import flash.utils.getQualifiedClassName;
	import flash.utils.getQualifiedSuperclassName;
	import jp.nium.core.namespaces.nium_internal;
	import jp.nium.utils.MathUtil;
	import jp.nium.utils.ObjectUtil;
	
	use namespace nium_internal;
	
	/*======================================================================*//**
	* 
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class GraphicObject extends Sprite {
		
		/*======================================================================*//**
		* @private
		*/
		public override function get numChildren()		:int { return 0; }
		
		/*======================================================================*//**
		* @private
		*/
		public override function get tabChildren()		:Boolean { return false; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get fillColor()					:int { return _fillColor; }
		public function set fillColor( value			:int ):void {
			_fillColor = MathUtil.range( value, 0x000000, 0xFFFFFF );
			_draw();
		}
		private var _fillColor							:int = 0x000000;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get lineColor()					:int { return _lineColor; }
		public function set lineColor( value			:int ):void {
			_lineColor = MathUtil.range( value, 0x000000, 0xFFFFFF );
			_draw();
		}
		private var _lineColor							:int = 0x000000;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get lineWidth()					:int { return _lineWidth; }
		public function set lineWidth( value			:int ):void {
			_lineWidth = value;
			_draw();
		}
		private var _lineWidth							:int = -1;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function get graphics()			:Graphics { return null; }
		
		/*======================================================================*//**
		* @private
		*/
		nium_internal function get graphics()			:Graphics { return super.graphics; }
		
		
		
		
		
		/*======================================================================*//**
		* 新しい GraphicObject インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @throws	ArgumentError	GraphicObject クラスを直接インスタンス化することはできません。
		*/
		public function GraphicObject() {
			// 継承せずにオブジェクト化されたらエラーを送出する
			if ( getQualifiedSuperclassName( this ) == getQualifiedClassName( Sprite ) ) { throw new ArgumentError( "Error #2012: GraphicObject クラスを直接インスタンス化することはできません。" ); }
			
			// 初期化する
			super.tabChildren = false;
			
			// デフォルトカラーをランダムに生成する
			fillColor = Math.round( 0xFFFFFF * Math.random() );
			lineColor = fillColor;
			
			// 描画する
			_draw();
		}
		
		
		
		
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected function _draw():void {
		}
		
		/*======================================================================*//**
		* GraphicObject インスタンスに対して、複数のプロパティを一括設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	props	設定したいプロパティを含んだ Object です。
		* @return			設定対象の GraphicObject インスタンスです。
		*/
		public function setProperties( props:Object ):GraphicObject {
			ObjectUtil.setProperties( this, props );
			return this;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function addChild( child:DisplayObject ):DisplayObject {
			throw new IllegalOperationError( "メソッド addChild() は実装されていません。" );
			return null;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function addChildAt( child:DisplayObject, index:int ):DisplayObject {
			throw new IllegalOperationError( "メソッド addChildAt() は実装されていません。" );
			return null;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function removeChild( child:DisplayObject ):DisplayObject {
			throw new IllegalOperationError( "メソッド removeChild() は実装されていません。" );
			return null;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function removeChildAt( index:int ):DisplayObject {
			throw new IllegalOperationError( "メソッド removeChildAt() は実装されていません。" );
			return null;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function contains( child:DisplayObject ):Boolean {
			throw new IllegalOperationError( "メソッド contains() は実装されていません。" );
			return false;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function getChildAt( index:int ):DisplayObject {
			throw new IllegalOperationError( "メソッド getChildAt() は実装されていません。" );
			return null;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function getChildByName( name:String ):DisplayObject {
			throw new IllegalOperationError( "メソッド getChildByName() は実装されていません。" );
			return null;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function getChildIndex( child:DisplayObject ):int {
			throw new IllegalOperationError( "メソッド getChildIndex() は実装されていません。" );
			return 0;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function setChildIndex( child:DisplayObject, index:int ):void {
			throw new IllegalOperationError( "メソッド setChildIndex() は実装されていません。" );
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function swapChildren( child1:DisplayObject, child2:DisplayObject ):void {
			throw new IllegalOperationError( "メソッド swapChildren() は実装されていません。" );
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function swapChildrenAt( index1:int, index2:int ):void {
			throw new IllegalOperationError( "メソッド swapChildrenAt() は実装されていません。" );
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object GraphicObject]";
		}
	}
}





