﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.display {
	import flash.display.DisplayObject;
	import flash.display.DisplayObjectContainer;
	import flash.events.Event;
	import flash.utils.Dictionary;
	import jp.nium.core.namespaces.nium_internal;
	import jp.nium.events.IEventIntegrator;
	
	use namespace nium_internal;
	
	/*======================================================================*//**
	* ChildIndexer クラスは DisplayObjectContainer を拡張して、より高度な表示リスト操作を行うためのクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class ChildIndexer {
		
		/*======================================================================*//**
		* 登録された DisplayObjectContainer インスタンスをキーとして ChildIndexer インスタンスを格納した Dictionary インスタンスを取得します。 
		*/
		private static var _indexers		:Dictionary = new Dictionary( true );
		
		
		
		
		
		/*======================================================================*//**
		* 子ディスプレイオブジェクトの数を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get numChildren()	:int { return children.length; }
		
		/*======================================================================*//**
		* 子ディスプレイオブジェクトが格納されている配列です。
		* インデックス値が断続的に指定可能である為、getChildAt() ではなくこのプロパティを使用して子ディスプレイオブジェクト走査を行います。
		* この配列を操作することで元の配列を変更することはできません。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get children()		:Array { return _children.concat(); }
		private var _children				:Array;
		
		/*======================================================================*//**
		* インデックスを整理する対象コンテナを取得します。
		*/
		private var _container				:DisplayObjectContainer;
		
		/*======================================================================*//**
		* インデックスをキーとしてディスプレイオブジェクトを格納した Dictionary インスタンスを取得します。
		*/
		private var _indexesToChild			:Dictionary;
		
		/*======================================================================*//**
		* ディスプレイオブジェクトをキーとしてインデックスを格納した Dictionary インスタンスを取得します。
		*/
		private var _childrenToIndex		:Dictionary;
		
		/*======================================================================*//**
		* 
		*/
		private var _addChild				:Function;
		private var _addChildAt				:Function;
		private var _removeChild			:Function;
		private var _removeChildAt			:Function;
		private var _contains				:Function;
		private var _getChildAt				:Function;
		private var _getChildByName			:Function;
		private var _getChildIndex			:Function;
		private var _swapChildren			:Function;
		private var _swapChildrenAt			:Function;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい ChildIndexer インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function ChildIndexer( container:DisplayObjectContainer ) {
			// すでに登録されていればエラーを送出する
			if ( _indexers[container] ) { throw new ArgumentError( "すでに登録済みの DisplayObjectContainer を指定することはできません。" ) }
			
			// 引数を設定する
			_container = container;
			
			// 登録する
			_indexers[_container] = this;
			
			// 初期化する
			nium_internal::initialize( {
				addChild		:container.addChild,
				addChildAt		:container.addChildAt,
				removeChild		:container.removeChild,
				removeChildAt	:container.removeChildAt,
				contains		:container.contains,
				getChildAt		:container.getChildAt,
				getChildByName	:container.getChildByName,
				getChildIndex	:container.getChildIndex,
				swapChildren	:container.swapChildren,
				swapChildrenAt	:container.swapChildrenAt
			} );
		}
		
		
		
		
		
		/*======================================================================*//**
		* @private
		*/
		nium_internal function initialize( initObject:Object ):void {
			try {
				// 指定された関数を設定する
				_addChild = initObject.addChild;
				_addChildAt = initObject.addChildAt;
				_removeChild = initObject.removeChild;
				_removeChildAt = initObject.removeChildAt;
				_contains = initObject.contains;
				_getChildAt = initObject.getChildAt;
				_getChildByName = initObject.getChildByName;
				_getChildIndex = initObject.getChildIndex;
				_swapChildren = initObject.swapChildren;
				_swapChildrenAt = initObject.swapChildrenAt;
			}
			catch ( e:Error ) {
				throw new ArgumentError( "必要な引数が設定されていません。" );
			}
			
			// 初期化する
			_children = [];
			_indexesToChild = new Dictionary( true );
			_childrenToIndex = new Dictionary( true );
			
			// IEventIntegrator インターフェイスを実装していれば
			if ( _container is IEventIntegrator ) {
				try {
					// イベントを登録する
					IEventIntegrator( _container ).addExclusivelyEventListener( Event.ADDED, _added, true, int.MAX_VALUE, true );
					IEventIntegrator( _container ).addExclusivelyEventListener( Event.REMOVED, _removed, false, int.MAX_VALUE, true );
				}
				catch ( e:Error ) {
				}
			}
			else {
				// イベントを登録する
				_container.addEventListener( Event.ADDED, _added, true, int.MAX_VALUE, true );
				_container.addEventListener( Event.REMOVED, _removed, false, int.MAX_VALUE, true );
			}
			
			// 既存のディスプレイオブジェクトを子に追加する
			var l:int = _container.numChildren;
			for ( var i:int = 0; i < l; i++ ) {
				try {
					var child:DisplayObject = _getChildAt( i );
				}
				catch ( e:Error ) {
					continue;
				}
				
				// 存在すれば設定する
				_indexesToChild[i] = child;
				_childrenToIndex[child] = i;
				_children.push( child );
			}
		}
		
		/*======================================================================*//**
		* 子 DisplayObject を登録します。
		*/
		private function _registerChild( child:DisplayObject, index:int ):DisplayObject {
			// すでに配置済みのインデックス位置が指定されたらエラーを送出する
			if ( _indexesToChild[index] ) {
				throw new RangeError( "Error : 指定されたインデックス位置はすでに使用されています。" );
			}
			
			// 既存の登録があれば削除する
			if ( !isNaN( _childrenToIndex[child] ) ) {
				_unregisterChild( child );
			}
			
			// データを登録する
			_indexesToChild[index] = child;
			_childrenToIndex[child] = index;
			
			// インデックス位置を取得する
			var indexes:Array = [];
			for each ( var idx:int in _childrenToIndex ) {
				indexes.push( idx );
			}
			
			// 数値順に並び替える
			indexes.sort( Array.NUMERIC );
			
			// 子を並び替える
			_children = [];
			var l:int = indexes.length;
			for ( var i:int = 0; i < l; i++ ) {
				_children.push( _indexesToChild[indexes[i]] );
			}
			
			return child;
		}
		
		/*======================================================================*//**
		* 子 DisplayObject の登録を削除します。
		*/
		private function _unregisterChild( child:DisplayObject ):DisplayObject {
			// 子ディスプレイオブジェクトに登録があれば削除する
			if ( !isNaN( _childrenToIndex[child] ) ) {
				// 登録から削除する
				delete _indexesToChild[_childrenToIndex[child]];
				delete _childrenToIndex[child];
				
				// 子ディスプレイオブジェクトを走査する
				var l:int = _children.length;
				for ( var i:int = 0; i < l; i++ ) {
					var display:DisplayObject = DisplayObject( _children[i] );
					
					// 違っていれば次へ
					if ( display != child ) { continue; }
					
					_children.splice( i, 1 );
					break;
				}
			}
			
			return child;
		}
		
		/*======================================================================*//**
		* この DisplayObjectContainer インスタンスに子 DisplayObject インスタンスを追加します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	child	対象の DisplayObjectContainer インスタンスの子として追加する DisplayObject インスタンスです。
		* @return			child パラメータで渡す DisplayObject インスタンスです。 
		*/
		public function addChild( child:DisplayObject ):DisplayObject {
			// 親を取得する
			var parent:DisplayObjectContainer = DisplayObjectContainer( child.parent );
			
			// 親が存在すれば
			if ( parent ) {
				// ChildIndexer インスタンスを取得する
				var indexer:ChildIndexer = _indexers[parent];
				
				// 親が ChildIndexer を使用していれば
				if ( indexer ) {
					indexer.removeChild( child );
				}
				else {
					parent.removeChild( child );
				}
			}
			
			// 最高値の仮想インデックス位置を取得する
			var virtualIndex:int = numChildren ? _childrenToIndex[_getChildAt( numChildren - 1 )] + 1 : 0;
			
			// ディスプレイリストに追加して参照を返す
			return _addChild( _registerChild( child, virtualIndex ) );
		}
		
		/*======================================================================*//**
		* この DisplayObjectContainer インスタンスに子 DisplayObject インスタンスを追加します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	child		対象の DisplayObjectContainer インスタンスの子として追加する DisplayObject インスタンスです。
		* @param	index		子を追加するインデックス位置です。
		* @param	addAbove	既にオブジェクトが置かれているインデックス位置に追加しようとした場合に、その位置にあるオブジェクトの 1 つ上に追加するかどうか指定します。
		* @return				child パラメータで渡す DisplayObject インスタンスです。 
		*/
		public function addChildAt( child:DisplayObject, index:int, addAbove:Boolean = false ):DisplayObject {
			var virtualIndex:int = index;
			
			// virtualIndex が 0 未満であればエラーを送出する
			if ( virtualIndex < 0 ) {
				throw new ArgumentError( "Error #2025: 指定した DisplayObject は呼び出し元の子でなければなりません。" );
			}
			
			// 親を取得する
			var parent:DisplayObjectContainer = DisplayObjectContainer( child.parent );
			
			// 親が存在すれば、親から削除する
			if ( parent ) {
				// ChildIndexer インスタンスを取得する
				var indexer:ChildIndexer = _indexers[parent];
				
				// 親が ChildIndexer を使用していれば
				if ( indexer ) {
					indexer.removeChild( child );
				}
				else {
					parent.removeChild( child );
				}
			}
			
			var realIndex:int;
			
			// 指定された仮想インデックス位置にすでに登録されていれば
			if ( _indexesToChild[virtualIndex] ) {
				realIndex = _getChildIndex( _indexesToChild[virtualIndex] );
				
				// 対象の上に配置するのであれば
				if ( addAbove ) {
					virtualIndex++;
					realIndex++;
				}
				
				// インデックス位置を走査する
				var l:int = numChildren;
				for ( var i:int = realIndex; i < l; i++ ) {
					// 子を取得する
					var child1:DisplayObject = _getChildAt( Math.max( realIndex, i - 1 ) );
					var child2:DisplayObject = _getChildAt( i );
					
					// 一つ前のインデックス値と競合してなければ終了する
					if ( _childrenToIndex[child1] != _childrenToIndex[child2] ) { break; }
					
					// 値を 1 つずらす
					var newVirtualIndex:int = _childrenToIndex[child2] + 1;
					
					// 古い登録を削除する
					delete _indexesToChild[_childrenToIndex[child2]];
					
					// 新しく登録する
					_indexesToChild[newVirtualIndex] = child2;
					_childrenToIndex[child2] = newVirtualIndex;
				}
			}
			else {
				l = _children.length;
				for ( i = 0; i < l; i++ ) {
					// 対象の仮想インデックス位置が指定された仮想インデックスを超えていれば終了する
					if ( _childrenToIndex[_children[i]] > virtualIndex ) { break; }
				}
				realIndex = i;
			}
			
			// ディスプレイリストに追加して参照を返す
			return _addChildAt( _registerChild( child, virtualIndex ), realIndex );
		}
		
		/*======================================================================*//**
		* DisplayObjectContainer の子リストの指定されたインデックス位置から子 DisplayObject インスタンスを削除します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	child	対象の DisplayObjectContainer インスタンスの子から削除する DisplayObject インスタンスです。
		* @return			child パラメータで渡す DisplayObject インスタンスです。 
		*/
		public function removeChild( child:DisplayObject ):DisplayObject {
			return _removeChild( _unregisterChild( child ) );
		}
		
		/*======================================================================*//**
		* DisplayObjectContainer の子リストの指定されたインデックス位置から子 DisplayObject インスタンスを削除します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	index	削除する DisplayObject の子インデックスです。
		* @return			削除された DisplayObject インスタンスです。 
		*/
		public function removeChildAt( index:int ):DisplayObject {
			return removeChild( _indexesToChild[index] );
		}
		
		/*======================================================================*//**
		* DisplayObjectContainer　に追加されている全ての子 DisplayObject インスタンスを削除します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function removeAllChildren():void {
			// 現在の子リストをコピーする
			var children:Array = _children.concat();
			
			// 初期化する
			_children = [];
			_indexesToChild = new Dictionary( true );
			_childrenToIndex = new Dictionary( true );
			
			// 全ての子ディスプレイオブジェクトを削除する
			var l:int = children.length;
			for ( var i:int = 0; i < l; i++ ) {
				_removeChild( children[i] );
			}
		}
		
		/*======================================================================*//**
		* 指定された表示オブジェクトが DisplayObjectContainer インスタンスの子であるか、オブジェクト自体であるかを指定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	child	テストする子 DisplayObject インスタンスです。
		* @return			child インスタンスが DisplayObjectContainer の子であるか、コンテナ自体である場合は true となります。そうでない場合は false となります。
		*/
		public function contains( child:DisplayObject ):Boolean {
			return _contains( child );
		}
		
		/*======================================================================*//**
		* 指定のインデックス位置にある子表示オブジェクトオブジェクトを返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	index	子 DisplayObject インスタンスのインデックス位置です。
		* @return			指定されたインデックス位置にある子 DisplayObject インスタンスです。
		*/
		public function getChildAt( index:int ):DisplayObject {
			return _indexesToChild[index];
		}
		
		/*======================================================================*//**
		* 指定された名前に一致する子表示オブジェクトを返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	name	返される子 DisplayObject インスタンスの名前です。
		* @return			指定された名前を持つ子 DisplayObject インスタンスです。
		*/
		public function getChildByName( name:String ):DisplayObject {
			return _getChildByName( name );
		}
		
		/*======================================================================*//**
		* 子 DisplayObject インスタンスのインデックス位置を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	child	特定する子 DisplayObject インスタンスです。
		* @return			特定する子 DisplayObject インスタンスのインデックス位置です。
		*/
		public function getChildIndex( child:DisplayObject ):int {
			return _childrenToIndex[child];
		}
		
		/*======================================================================*//**
		* 表示オブジェクトコンテナの既存の子の位置を変更します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	child		インデックス番号を変更する子 DisplayObject インスタンスです。
		* @param	index		child インスタンスの結果のインデックス番号です。
		* @param	addAbove	既にオブジェクトが置かれているインデックス位置に移動しようとした場合に、その位置にあるオブジェクトの 1 つ上に移動するかどうか指定します。
		*/
		public function setChildIndex( child:DisplayObject, index:int, addAbove:Boolean = false ):void {
			addChildAt( removeChild( child ), index, addAbove )
		}
		
		/*======================================================================*//**
		* 指定された 2 つの子オブジェクトの z 順序（重ね順）を入れ替えます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	child1	先頭の子 DisplayObject インスタンスです。
		* @param	child2	2 番目の子 DisplayObject インスタンスです。
		*/
		public function swapChildren( child1:DisplayObject, child2:DisplayObject ):void {
			// インデックス位置を格納する
			var index1:int = _childrenToIndex[child1];
			var index2:int = _childrenToIndex[child2];
			
			if ( child1.parent != _container || child2.parent != _container ) { throw new ArgumentError( "Error #2025: 指定した DisplayObject は呼び出し元の子でなければなりません。" ); }
			
			// ディスプレイリストから削除する
			removeChild( child1 );
			removeChild( child2 );
			
			// ディスプレイリストに追加する
			addChildAt( child1, index2 );
			addChildAt( child2, index1 );
		}
		
		/*======================================================================*//**
		* 子リスト内の指定されたインデックス位置に該当する 2 つの子オブジェクトの z 順序（重ね順）を入れ替えます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	index1	最初の子 DisplayObject インスタンスのインデックス位置です。
		* @param	index2	2 番目の子 DisplayObject インスタンスのインデックス位置です。
		*/
		public function swapChildrenAt( index1:int, index2:int ):void {
			// ディスプレイオブジェクトを格納する
			var child1:DisplayObject = _indexesToChild[index1];
			var child2:DisplayObject = _indexesToChild[index2];
			
			if ( !child1 || !child2 ) { throw new ArgumentError( "Error #2025: 指定した DisplayObject は呼び出し元の子でなければなりません。" ); }
			
			// ディスプレイリストから削除する
			removeChild( child1 );
			removeChild( child2 );
			
			// ディスプレイリストに追加する
			addChildAt( child1, index2 );
			addChildAt( child2, index1 );
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public function toString():String {
			return "[object ChildIndexer]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* 表示オブジェクトが表示リストに追加されたときに送出されます。
		*/
		private function _added( e:Event ):void {
			var target:DisplayObject = DisplayObject( e.target );
			var parent:DisplayObjectContainer = target.parent;
			
			// 対象の親が管理対象の DisplayObjectContainer ではなければ終了する
			if ( parent != _container ) { return; }
			
			// インデックス値が存在していれば終了する
			if ( _childrenToIndex[target] != undefined ) { return; }
			
			// 関連付けられている ChildIndexer を取得する
			var indexer:ChildIndexer = ChildIndexer( _indexers[parent] );
			
			// target の仮想インデックス値を取得する
			var realIndex:int = indexer._getChildIndex( target );
			var virtualIndex:int = -1;
			
			// 子を走査する
			var l:int = parent.numChildren
			for ( var i:int = 0; i < l; i++ ) {
				var child:DisplayObject = indexer._getChildAt( i );
				var childRealIndex:int = indexer._getChildIndex( child );
				
				switch ( true ) {
					// 対象のインデックス値が target 未満であれば
					case realIndex > childRealIndex		: { virtualIndex = indexer.getChildIndex( child ); break; }
					// 対象と target のインデックス値が同じであれば
					case realIndex == childRealIndex	: {
						virtualIndex++;
						delete _indexesToChild[virtualIndex];
						break;
					}
					// 対象のインデックス値が target より大きければ
					default								: { _childrenToIndex[child]++; }
				}
			}
			
			// 登録する
			indexer._registerChild( target, virtualIndex );
		}
		
		/*======================================================================*//**
		* 表示オブジェクトが表示リストから削除されようとしているときに送出されます。
		*/
		private function _removed( e:Event ):void {
			var target:DisplayObject = DisplayObject( e.target );
			var parent:DisplayObjectContainer = target.parent;
			
			// 対象の親が管理対象の DisplayObjectContainer ではなければ終了する
			if ( parent != _container ) { return; }
			
			// 削除したにも関わらずインデックス値が存在していなければ終了する
			if ( _childrenToIndex[target] == undefined ) { return; }
			
			// 登録から削除する
			var indexer:ChildIndexer = ChildIndexer( _indexers[_container] );
			indexer._unregisterChild( target );
		}
	}
}






