﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.display {
	import flash.display.LoaderInfo;
	import flash.display.Stage;
	import flash.errors.IllegalOperationError;
	import flash.events.Event;
	import jp.nium.display.ExMovieClip;
	import jp.nium.events.DocumentEvent;
	import jp.nium.external.Browser;
	import jp.nium.utils.StageUtil;
	
	/*======================================================================*//**
	* SWF ファイルの読み込みが完了した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.nium.events.DocumentEvent.INIT
	*/
	[Event( name="init", type="jp.nium.events.DocumentEvent" )]
	
	/*======================================================================*//**
	* ステージのリサイズを開始した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.nium.events.DocumentEvent.RESIZE_START
	*/
	[Event( name="resizeStart", type="jp.nium.events.DocumentEvent" )]
	
	/*======================================================================*//**
	* ステージがリサイズ処理をしている場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.nium.events.DocumentEvent.RESIZE_PROGRESS
	*/
	[Event( name="resizeProgress", type="jp.nium.events.DocumentEvent" )]
	
	/*======================================================================*//**
	* ステージのリサイズを終了した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.nium.events.DocumentEvent.RESIZE_COMPLETE
	*/
	[Event( name="resizeComplete", type="jp.nium.events.DocumentEvent" )]
	
	/*======================================================================*//**
	* 
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class DocumentRoot extends ExMovieClip {
		
		/*======================================================================*//**
		* ドキュメントクラスのオブジェクトを取得します。
		*/
		private static var _instance					:DocumentRoot;
		
		/*======================================================================*//**
		* 表示オブジェクトのステージです。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get stage()				:Stage { return _instance.stage; }
		
		/*======================================================================*//**
		* ロードされた SWF ファイル内の表示オブジェクトの場合、root プロパティはその SWF ファイルが表す表示リストのツリー構造部分の一番上にある表示オブジェクトとなります。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get root()				:DocumentRoot { return _instance; }
		
		/*======================================================================*//**
		* この表示オブジェクトが属するファイルのロード情報を含む LoaderInfo インスタンスを返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get loaderInfo()			:LoaderInfo { return _instance.loaderInfo; }
		
		/*======================================================================*//**
		* ステージの現在の幅をピクセル単位で指定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get stageWidth()			:int { return stage ? stage.stageWidth : 0; }
		public static function set stageWidth( value	:int ):void { stage.stageWidth = value; }
		
		/*======================================================================*//**
		* ステージの現在の高さをピクセル単位で指定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get stageHeight()		:int { return stage ? stage.stageHeight : 0; }
		public static function set stageHeight( value	:int ):void { stage.stageHeight = value; }
		
		/*======================================================================*//**
		* ロードされたコンテンツの規格幅です。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get orgWidth()			:int { return loaderInfo ? loaderInfo.width : 0; }
		
		/*======================================================================*//**
		* ロードされたファイルの規格高さです。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get orgHeight()			:int { return loaderInfo ? loaderInfo.height : 0; }
		
		/*======================================================================*//**
		* ステージ左の X 座標を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get left()				:int { return -StageUtil.getMarginLeft( stage ); }
		
		/*======================================================================*//**
		* ステージ右の X 座標を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get right()				:int { return left + stageWidth; }
		
		/*======================================================================*//**
		* ステージ上の Y 座標を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get top()				:int { return -StageUtil.getMarginTop( stage ); }
		
		/*======================================================================*//**
		* ステージ下の Y 座標を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get bottom()				:int { return top + stageHeight; }
		
		/*======================================================================*//**
		* ステージ中央の X 座標を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get centerX()			:int { return right / 2; }
		
		/*======================================================================*//**
		* ステージ中央の Y 座標を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get centerY()			:int { return bottom / 2; }
		
		/*======================================================================*//**
		* Flash Player またはブラウザでのステージの配置を指定する StageAlign クラスの値です。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get align()				:String { return stage ? stage.align : ""; }
		public static function set align( value			:String ):void { stage.align = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get centering()			:Boolean { return _instance.centering; }
		public static function set centering( value		:Boolean ):void { _instance.centering = value; }
		
		/*======================================================================*//**
		* 使用する表示状態を指定する StageDisplayState クラスの値です。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get displayState()		:String { return stage ? stage.displayState : ""; }
		public static function set displayState( value	:String ):void { stage.displayState = value; }
		
		/*======================================================================*//**
		* ステージのフレームレートを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get frameRate()			:Number { return stage ? stage.frameRate : 0; }
		public static function set frameRate( value		:Number ):void { stage.frameRate = value; }
		
		/*======================================================================*//**
		* Flash Player が使用するレンダリング品質を指定する StageQuality クラスの値です。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get quality()			:String { return stage ? stage.quality : ""; }
		public static function set quality( value		:String ):void { stage.quality = value; }
		
		/*======================================================================*//**
		* 使用する拡大 / 縮小モードを指定する StageScaleMode クラスの値です。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get scaleMode()			:String { return stage ? stage.scaleMode : ""; }
		public static function set scaleMode( value		:String ):void { stage.scaleMode = value; }
		
		/*======================================================================*//**
		* ドキュメントクラスとなる SWF ファイルを基点とした相対パスを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get dir()				:String {
			// 一度取得してあれば、キャッシュから返す
			if ( _dir ) { return _dir; }
			
			// パスを作成する
			var path:Array = _instance.loaderInfo.url.split( "/" );
			path[path.length - 1] = "";
			
			return _dir = path.join( "/" );
		}
		private static var _dir							:String;
		
		
		
		
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get centering()					:Boolean { return _centering; }
		public function set centering( value			:Boolean ):void {
			_centering = value;
			
			// 位置を調節する
			super.x = value ? centerX : 0;
			super.y = value ? centerY : 0;
		}
		private var _centering							:Boolean = false;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function get x()				:Number { return super.x; }
		public override function set x( value			:Number ):void {
			// センタリング状態であれば終了する
			if ( _centering ) { return; }
			super.x = value;
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function get y()				:Number { return super.y; }
		public override function set y( value			:Number ):void {
			// センタリング状態であれば終了する
			if ( _centering ) { return; }
			super.y = value;
		}
		
		/*======================================================================*//**
		* ドキュメントクラスが DocumentEvent.INIT イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onInit()					:Function { return __onInit || _onInit; }
		public function set onInit( value				:Function ):void { __onInit = onInit; }
		private var __onInit							:Function;
		
		/*======================================================================*//**
		* サブクラスで onInit イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onInit プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onInit
		*/
		protected function _onInit()					:void {}
		
		/*======================================================================*//**
		* ドキュメントクラスが DocumentEvent.RESIZE_START イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onResizeStart()				:Function { return __onResizeStart || _onResizeStart; }
		public function set onResizeStart( value		:Function ):void { __onResizeStart = value; }
		private var __onResizeStart						:Function;
		
		/*======================================================================*//**
		* サブクラスで onResizeStart イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onResizeStart プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onResizeStart
		*/
		protected function _onResizeStart()				:void {}
		
		/*======================================================================*//**
		* ドキュメントクラスが DocumentEvent.RESIZE_PROGRESS イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onResizeProgress()			:Function { return __onResizeProgress || _onResizeProgress; }
		public function set onResizeProgress( value		:Function ):void { __onResizeProgress = value; }
		private var __onResizeProgress					:Function;
		
		/*======================================================================*//**
		* サブクラスで onResizeProgress イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onResizeProgress プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onResizeProgress
		*/
		protected function _onResizeProgress()			:void {}
		
		/*======================================================================*//**
		* ドキュメントクラスが DocumentEvent.RESIZE_COMPLETE イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onResizeComplete()			:Function { return __onResizeComplete || _onResizeComplete; }
		public function set onResizeComplete( value		:Function ):void { __onResizeComplete = value; }
		private var __onResizeComplete					:Function;
		
		/*======================================================================*//**
		* サブクラスで onResizeComplete イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onResizeComplete プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onResizeComplete
		*/
		protected function _onResizeComplete()			:void {}
		
		/*======================================================================*//**
		* 前回のステージサイズを取得します。
		*/
		private var _oldStageWidth						:int = 0;
		private var _oldStageHeight						:int = 0;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい DocumentRoot インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @throws	ArgumentError	DocumentRoot クラスは、ドキュメントクラス以外には使用できません。
		*/
		public function DocumentRoot() {
			// root が存在しない、または自身が root ではない場合
			if ( !( root && this == root ) ) { throw new ArgumentError( "DocumentRoot クラスは、ドキュメントクラス以外には使用できません。" ); }
			
			// 自身の参照を設定する
			_instance = this;
			
			// 読み込み完了を待つ
			addExclusivelyEventListener( Event.ENTER_FRAME, _enterFrame, false, int.MAX_VALUE, true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	url		説明文
		* @return			説明文
		*/
		public static function toDocumentBasePath( url:String ):String {
			// 絶対パスであればそのまま返す
			if ( new RegExp( "^[A-Za-z0-9]+://", "g" ).test( url ) ) { return url; }
			
			// SWF からのパスを追加して返す
			return dir + url;
		}
		
		/*======================================================================*//**
		* @copy jp.nium.events.EventIntegrator#addEventListener()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function addEventListener( type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false ):void {
			_instance.addEventListener( type, listener, useCapture, priority, useWeakReference );
		}
		
		/*======================================================================*//**
		* @copy jp.nium.events.EventIntegrator#addExclusivelyEventListener()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function addExclusivelyEventListener( type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false ):void {
			_instance.addExclusivelyEventListener( type, listener, useCapture, priority, useWeakReference );
		}
		
		/*======================================================================*//**
		* @copy jp.nium.events.EventIntegrator#removeEventListener()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function removeEventListener( type:String, listener:Function, useCapture:Boolean = false ):void {
			_instance.removeEventListener( type, listener, useCapture );
		}
		
		/*======================================================================*//**
		* @copy jp.nium.events.EventIntegrator#completelyRemoveEventListener()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function completelyRemoveEventListener( type:String, listener:Function, useCapture:Boolean = false ):void {
			_instance.completelyRemoveEventListener( type, listener, useCapture );
		}
		
		/*======================================================================*//**
		* @copy jp.nium.events.EventIntegrator#dispatchEvent()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function dispatchEvent( event:Event ):Boolean {
			return _instance.dispatchEvent( event );
		}
		
		/*======================================================================*//**
		* @copy jp.nium.events.EventIntegrator#hasEventListener()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function hasEventListener( type:String ):Boolean {
			return _instance.hasEventListener( type );
		}
		
		/*======================================================================*//**
		* @copy jp.nium.events.EventIntegrator#willTrigger()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function willTrigger( type:String ):Boolean {
			return _instance.willTrigger( type );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object DocumentRoot]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* インスタンスがディスプレイリストに追加される場合に送出されます。
		*/
		private function _enterFrame( e:Event ):void {
			// loaderInfo が存在しなければ終了する
			if ( !loaderInfo ) { return; }
			
			// 親の Loader インスタンスが存在せずローカルである場合、または読み込みが完了した場合
			if ( ( loaderInfo.loader || Browser.online ) && ( loaderInfo.bytesTotal == 0 || loaderInfo.bytesLoaded < loaderInfo.bytesTotal ) ) { return; }
			
			// リスナーを削除する
			completelyRemoveEventListener( Event.ENTER_FRAME, _enterFrame );
			
			// リスナーを追加する
			stage.addEventListener( Event.RESIZE, _resizeStart, false, int.MAX_VALUE, true );
			
			// イベントを送出する
			dispatchEvent( new DocumentEvent( DocumentEvent.INIT, false, false, this ) );
			
			// イベントハンドラメソッドを実行する
			onInit();
		}
		
		/*======================================================================*//**
		* ステージのリサイズを開始した場合に送出されます。
		*/
		private function _resizeStart( e:Event ):void {
			// リスナーを解除する
			stage.removeEventListener( Event.RESIZE, _resizeStart );
			
			// イベントを送出する
			dispatchEvent( new DocumentEvent( DocumentEvent.RESIZE_START, false, false, this ) );
			
			// イベントハンドラメソッドを実行する
			onResizeStart();
			
			// リスナーを登録する
			addExclusivelyEventListener( Event.ENTER_FRAME, _resizeProgress, false, int.MAX_VALUE, true );
		}
		
		/*======================================================================*//**
		* ステージがリサイズ処理をしている場合に送出されます。
		*/
		private function _resizeProgress( e:Event ):void {
			// 位置を調節する
			centering = centering;
			
			// イベントを送出する
			dispatchEvent( new DocumentEvent( DocumentEvent.RESIZE_PROGRESS, false, false, this ) );
			
			// イベントハンドラメソッドを実行する
			onResizeProgress();
			
			// 前回とサイズが変更されていなければ、イベントを送出する
			if ( _oldStageWidth == stageWidth && _oldStageHeight == stageHeight ) {
				// イベントを送出する
				dispatchEvent( new DocumentEvent( DocumentEvent.RESIZE_COMPLETE, false, false, this ) );
				
				// イベントハンドラメソッドを実行する
				onResizeComplete();
				
				// リスナーを解除する
				completelyRemoveEventListener( Event.ENTER_FRAME, _resizeProgress );
				
				// リスナーを登録する
				stage.addEventListener( Event.RESIZE, _resizeStart, false, int.MAX_VALUE, true );
			}
			
			// 現在のサイズを格納する
			_oldStageWidth = stageWidth;
			_oldStageHeight = stageHeight;
		}
	}
}





