﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.display {
	import flash.display.Bitmap;
	import flash.display.DisplayObject;
	import flash.events.Event;
	import jp.nium.core.collections.DisplayList;
	import jp.nium.core.namespaces.nium_internal;
	import jp.nium.display.IExDisplayObject;
	import jp.nium.events.EventIntegrator;
	import jp.nium.events.IEventIntegrator;
	import jp.nium.utils.ObjectUtil;
	
	use namespace nium_internal;
	
	/*======================================================================*//**
	* ExBitmap クラスは、Bitmap クラスの基本機能を拡張した jp.nium パッケージで使用される基本クラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class ExBitmap extends Bitmap implements IExDisplayObject, IEventIntegrator {
		
		/*======================================================================*//**
		* 
		*/
		private static var _stacks					:Array = new Array();
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get id()					:String { return DisplayList.nium_internal::getIdByInstance( this ); }
		public function set id( value				:String ):void { DisplayList.nium_internal::addInstance( this, value ); }
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get listenOnStage()			:Boolean { return _listenOnStage; }
		public function set listenOnStage( value	:Boolean ):void {
			_listenOnStage = value;
			
			if ( !value ) { return; }
			
			if ( stage ) {
				// イベントリスナーを登録する
				restoreRemovedListeners();
			}
			else {
				// イベントリスナーを解除する
				removeAllListeners();
			}
		}
		private var _listenOnStage					:Boolean = false;
		
		/*======================================================================*//**
		* リスナーを整理する対象 EventIntegrator を取得します。
		*/
		private var _integrator						:EventIntegrator;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい ExBitmap インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function ExBitmap() {
			// EventIntegrator を作成する
			_integrator = new EventIntegrator( this );
			_integrator.nium_internal::initialize( {
				addEventListener	:super.addEventListener,
				removeEventListener	:super.removeEventListener,
				hasEventListener	:super.hasEventListener,
				willTrigger			:super.willTrigger,
				dispatchEvent		:super.dispatchEvent
			} );
			
			// スタックが存在すれば実行する
			while ( _stacks.length > 0 ) {
				var o:Object = _stacks.shift();
				o.func.apply( o.target, o.args );
			}
			
			// イベントリスナーを登録する
			addExclusivelyEventListener( Event.ADDED_TO_STAGE, _addedToStage, false, int.MAX_VALUE, true );
			addExclusivelyEventListener( Event.REMOVED_FROM_STAGE, _removedFromStage, false, int.MAX_VALUE, true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function setProperties( props:Object ):DisplayObject {
			ObjectUtil.setProperties( this, props );
			return this;
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function addEventListener( type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false ):void {
			if ( !_integrator ) {
				_stacks.push( {
					target		:this,
					func		:arguments.callee,
					args		:arguments
				} );
				return;
			}
			
			_integrator.addEventListener( type, listener, useCapture, priority, useWeakReference );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function addExclusivelyEventListener( type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false ):void {
			_integrator.addExclusivelyEventListener( type, listener, useCapture, priority, useWeakReference );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function removeEventListener( type:String, listener:Function, useCapture:Boolean = false ):void {
			_integrator.removeEventListener( type, listener, useCapture );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function completelyRemoveEventListener( type:String, listener:Function, useCapture:Boolean = false ):void {
			_integrator.completelyRemoveEventListener( type, listener, useCapture );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function dispatchEvent( event:Event ):Boolean {
			return _integrator.dispatchEvent( event );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function hasEventListener( type:String ):Boolean {
			return _integrator.hasEventListener( type );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function willTrigger( type:String ):Boolean {
			return _integrator.willTrigger( type );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function removeAllListeners( completely:Boolean = false ):void {
			_integrator.removeAllListeners( completely );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function restoreRemovedListeners():void {
			_integrator.restoreRemovedListeners();
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object ExBitmap]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* 表示オブジェクトが、直接、または表示オブジェクトを含むサブツリーの追加により、ステージ上の表示リストに追加されたときに送出されます。
		*/
		private function _addedToStage( e:Event ):void {
			listenOnStage = listenOnStage;
		}
		
		/*======================================================================*//**
		* 表示オブジェクトが、直接、または表示オブジェクトを含むサブツリーの削除により、表示リストから削除されようとしているときに送出されます。
		*/
		private function _removedFromStage( e:Event ):void {
			listenOnStage = listenOnStage;
		}
	}
}





