﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.display {
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.DisplayObject;
	import flash.display.Loader;
	import flash.display.LoaderInfo;
	import flash.display.PixelSnapping;
	import flash.errors.IllegalOperationError;
	import flash.events.Event;
	import flash.events.HTTPStatusEvent;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	import flash.net.URLRequest;
	import flash.system.LoaderContext;
	import jp.nium.display.ExImageLoaderAlign;
	import jp.nium.display.ExSprite;
	import jp.nium.graphics.Square;
	
	/*======================================================================*//**
	* ロード操作が開始したときに送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType flash.events.Event.OPEN
	*/
	[Event( name="open", type="flash.events.Event" )]
	
	/*======================================================================*//**
	* ロードされた SWF ファイルのプロパティおよびメソッドにアクセスできる場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType flash.events.Event.INIT
	*/
	[Event( name="init", type="flash.events.Event" )]
	
	/*======================================================================*//**
	* データが正常にロードされたときに送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType flash.events.Event.COMPLETE
	*/
	[Event( name="complete", type="flash.events.Event" )]
	
	/*======================================================================*//**
	* ロードされたオブジェクトが Loader オブジェクトの unload() メソッドを使用して削除されるたびに、LoaderInfo オブジェクトによって送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType flash.events.Event.UNLOAD
	*/
	[Event( name="unload", type="flash.events.Event" )]
	
	/*======================================================================*//**
	* ダウンロード処理を実行中にデータを受信したときに送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType flash.events.ProgressEvent.PROGRESS
	*/
	[Event( name="progress", type="flash.events.ProgressEvent" )]
	
	/*======================================================================*//**
	* 入出力エラーが発生してロード処理が失敗したときに送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType flash.events.IOErrorEvent.IO_ERROR
	*/
	[Event( name="ioError", type="flash.events.IOErrorEvent" )]
	
	/*======================================================================*//**
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	*/
	public class ExImageLoader extends ExSprite {
		
		/*======================================================================*//**
		* @private
		*/
		public override function get numChildren()		:int { return 0; }
		
		/*======================================================================*//**
		* @private
		*/
		public override function get tabChildren()		:Boolean { return false; }
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function get width()			:Number { return _width; }
		public override function set width( value		:Number ):void {
			var old:Number = _width;
			
			// 変更されていなければ終了する
			if ( old == ( _width = value ) ) { return; }
			
			_update();
		}
		private var _width								:Number = 0;
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function get height()			:Number { return _height; }
		public override function set height( value		:Number ):void {
			var old:Number = _height;
			
			// 変更されていなければ終了する
			if ( old == ( _height = value ) ) { return; }
			
			_update();
		}
		private var _height								:Number = 0;
		
		/*======================================================================*//**
		* ロードされたコンテンツの規格幅を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get orgWidth()					:Number { return _orgWidth; }
		private var _orgWidth							:Number = 0;
		
		/*======================================================================*//**
		* ロードされたファイルの規格高さを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get orgHeight()					:Number { return _orgHeight; }
		private var _orgHeight							:Number = 0;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get align()						:String { return _align; }
		public function set align( value				:String ):void {
			switch( value ) {
				case ExImageLoaderAlign.BOTTOM			:
				case ExImageLoaderAlign.BOTTOM_LEFT		:
				case ExImageLoaderAlign.BOTTOM_RIGHT	:
				case ExImageLoaderAlign.CENTER			:
				case ExImageLoaderAlign.LEFT			:
				case ExImageLoaderAlign.RIGHT			:
				case ExImageLoaderAlign.TOP				:
				case ExImageLoaderAlign.TOP_LEFT		:
				case ExImageLoaderAlign.TOP_RIGHT		: {
					var old:String = _align;
					
					// 変更されていなければ終了する
					if ( old == ( _align = value ) ) { return; }
					
					_update();
					break;
				}
			}
		}
		private var _align								:String = ExImageLoaderAlign.TOP_LEFT;
		
		/*======================================================================*//**
		* リサイズ時に比率を維持するかどうかを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get ratio()						:Boolean { return _ratio; }
		public function set ratio( value				:Boolean ):void {
			var old:Boolean = _ratio;
			
			// 変更されていなければ終了する
			if ( old == ( _ratio = value ) ) { return; }
			
			_update();
		}
		private var _ratio								:Boolean = false;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get pixelSnapping()				:String { return _pixelSnapping; }
		public function set pixelSnapping( value		:String ):void {
			switch( value ) {
				case PixelSnapping.ALWAYS	:
				case PixelSnapping.AUTO		:
				case PixelSnapping.NEVER	: {
					var old:String = _pixelSnapping;
					
					// 変更されていなければ終了する
					if ( old == ( _pixelSnapping = value ) ) { return; }
					
					_bmp.pixelSnapping = _pixelSnapping;
					break;
				}
			}
		}
		private var _pixelSnapping						:String = PixelSnapping.ALWAYS;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get smoothing()					:Boolean { return _smoothing; }
		public function set smoothing( value			:Boolean ):void {
			var old:Boolean = _smoothing;
			
			// 変更されていなければ終了する
			if ( old == ( _smoothing = value ) ) { return; }
			
			_bmp.smoothing = _smoothing;
		}
		private var _smoothing							:Boolean = true;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get bytesLoaded()				:int { return _bytesLoaded; }
		private var _bytesLoaded						:int = 0;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get bytesTotal()				:int { return _bytesTotal; }
		private var _bytesTotal							:int = 0;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get contentType()				:String { return _contentType; }
		private var _contentType						:String;
		
		/*======================================================================*//**
		* Loader インスタンスを取得します。 
		*/
		private var _loader								:Loader;
		
		/*======================================================================*//**
		* Bitmap インスタンスを取得します。 
		*/
		private var _bmp								:Bitmap;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい ExImageLoader インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function ExImageLoader() {
			// 初期化する
			_loader = new Loader();
			_bmp = Bitmap( super.addChild( new Bitmap() ) );
			
			// マスクを作成する
			mask = Square( super.addChild( new Square().setProperties( {
				width		:0,
				height		:0
			} ) ) );
			
			// イベントリスナーを登録する
            _loader.contentLoaderInfo.addEventListener( Event.OPEN, _open, false, int.MAX_VALUE, true );
			_loader.contentLoaderInfo.addEventListener( Event.INIT, _init, false, int.MAX_VALUE, true );
			_loader.contentLoaderInfo.addEventListener( Event.COMPLETE, _complete, false, int.MAX_VALUE, true );
            _loader.contentLoaderInfo.addEventListener( Event.UNLOAD, _unload, false, int.MAX_VALUE, true );
            _loader.contentLoaderInfo.addEventListener( ProgressEvent.PROGRESS, _progress, false, int.MAX_VALUE, true );
            _loader.contentLoaderInfo.addEventListener( HTTPStatusEvent.HTTP_STATUS, _httpStatus, false, int.MAX_VALUE, true );
            _loader.contentLoaderInfo.addEventListener( IOErrorEvent.IO_ERROR, _ioError, false, int.MAX_VALUE, true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* イメージファイルを読み込みます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	request			読み込む JPEG、GIF、または PNG ファイルの絶対 URL または相対 URL です。
		* @param	context			説明文
		* @throws	ArgumentError	指定されたイメージファイルの形式には対応していません。
		*/
		public function load( request:URLRequest, context:LoaderContext = null ):void {
			// 拡張子を判別する
			var extension:String = request.url.split( "." ).pop();
			switch ( extension.toLowerCase() ) {
				case "jpg"		:
				case "jpeg"		:
				case "jpe"		:
				case "gif"		:
				case "png"		: { break; }
				default			: { throw new ArgumentError( "読み込むことができるデータ形式は jpeg, gif, png のみです。" ); }
			}
			
			// 読み込んだイメージファイルを破棄する
			unload();
			
			// 画像を読み込む
			_loader.load( request, context );
		}
		
		/*======================================================================*//**
		* 読み込んだイメージファイルを破棄します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function unload():void {
			// 初期化する
			_bmp.bitmapData = new BitmapData( Math.max( 1, _width ), Math.max( 1, _height ), true, 0xFFFFFF00 );
			_orgWidth = 0;
			_orgHeight = 0;
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		private function _update():void {
			// サイズが指定されていなければオリジナルサイズを設定する
			_width = _width || orgWidth;
			_height = _height || orgHeight;
			
			// 比率を調整する
			if ( _ratio ) {
				var scale:Number = Math.max( _width / _orgWidth, _height / _orgHeight );
				_bmp.width = Math.ceil( _orgWidth * scale );
				_bmp.height = Math.ceil( _orgHeight * scale );
			}
			else {
				_bmp.width = _width;
				_bmp.height = _height;
			}
			
			mask.width = _bmp.width;
			mask.height = _bmp.height;
			
			// X 軸の基準点を調整する
			_bmp.x = ( _width - _bmp.width ) / 2;
			mask.x = 0;
			switch( _align ) {
				case ExImageLoaderAlign.BOTTOM			:
				case ExImageLoaderAlign.CENTER			:
				case ExImageLoaderAlign.TOP				: {
					_bmp.x -= _width / 2;
					mask.x -= _width / 2;
					break;
				}
				case ExImageLoaderAlign.BOTTOM_LEFT		:
				case ExImageLoaderAlign.LEFT			:
				case ExImageLoaderAlign.TOP_LEFT		: { break; }
				case ExImageLoaderAlign.BOTTOM_RIGHT	:
				case ExImageLoaderAlign.RIGHT			:
				case ExImageLoaderAlign.TOP_RIGHT		: {
					_bmp.x -= _width;
					mask.x -= _width;
					break;
				}
			}
			
			// Y 軸の基準点を調整する
			_bmp.y = ( _height - _bmp.height ) / 2;
			mask.y = 0;
			switch( _align ) {
				case ExImageLoaderAlign.BOTTOM			:
				case ExImageLoaderAlign.BOTTOM_LEFT		:
				case ExImageLoaderAlign.BOTTOM_RIGHT	: {
					_bmp.y -= _height;
					mask.y -= _height;
					break;
				}
				case ExImageLoaderAlign.CENTER			:
				case ExImageLoaderAlign.LEFT			:
				case ExImageLoaderAlign.RIGHT			: {
					_bmp.y -= _height / 2;
					mask.y -= _height / 2;
					break;
				}
				case ExImageLoaderAlign.TOP				:
				case ExImageLoaderAlign.TOP_LEFT		:
				case ExImageLoaderAlign.TOP_RIGHT		: { break; }
			}
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function addChild( child:DisplayObject ):DisplayObject {
			throw new IllegalOperationError( "メソッド addChild() は実装されていません。" );
			return null;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function addChildAt( child:DisplayObject, index:int ):DisplayObject {
			throw new IllegalOperationError( "メソッド addChildAt() は実装されていません。" );
			return null;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function addChildAtAbove( child:DisplayObject, index:int, addAbove:Boolean = true ):DisplayObject {
			throw new IllegalOperationError( "メソッド addChildAt() は実装されていません。" );
			return null;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function removeChild( child:DisplayObject ):DisplayObject {
			throw new IllegalOperationError( "メソッド removeChild() は実装されていません。" );
			return null;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function removeChildAt( index:int ):DisplayObject {
			throw new IllegalOperationError( "メソッド removeChildAt() は実装されていません。" );
			return null;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function contains( child:DisplayObject ):Boolean {
			throw new IllegalOperationError( "メソッド contains() は実装されていません。" );
			return false;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function getChildAt( index:int ):DisplayObject {
			throw new IllegalOperationError( "メソッド getChildAt() は実装されていません。" );
			return null;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function getChildByName( name:String ):DisplayObject {
			throw new IllegalOperationError( "メソッド getChildByName() は実装されていません。" );
			return null;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function getChildIndex( child:DisplayObject ):int {
			throw new IllegalOperationError( "メソッド getChildIndex() は実装されていません。" );
			return 0;
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function setChildIndex( child:DisplayObject, index:int ):void {
			throw new IllegalOperationError( "メソッド setChildIndex() は実装されていません。" );
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function swapChildren( child1:DisplayObject, child2:DisplayObject ):void {
			throw new IllegalOperationError( "メソッド swapChildren() は実装されていません。" );
		}
		
		/*======================================================================*//**
		* @private
		*/
		public override function swapChildrenAt( index1:int, index2:int ):void {
			throw new IllegalOperationError( "メソッド swapChildrenAt() は実装されていません。" );
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object ExImageLoader]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* ロード操作が開始したときに送出されます。
		*/
		private function _open( e:Event ):void {
			// イベントを送出する
			dispatchEvent( e );
		}
		
		/*======================================================================*//**
		* ロードされた SWF ファイルのプロパティおよびメソッドにアクセスできる場合に送出されます。
		*/
		private function _init( e:Event ):void {
			// イベントを送出する
			dispatchEvent( e );
		}
		
		/*======================================================================*//**
		* データが正常にロードされたときに送出されます。
		*/
		private function _complete( e:Event ):void {
			_bytesLoaded = _loader.contentLoaderInfo.bytesLoaded;
			_bytesTotal = _loader.contentLoaderInfo.bytesTotal;
			_contentType = _loader.contentLoaderInfo.contentType;
			
			// bitmapData をコピーする
			_bmp.bitmapData = Bitmap( _loader.content ).bitmapData;
			
			// プロパティを初期化する
			pixelSnapping = pixelSnapping;
			smoothing = smoothing;
			
			// オリジナルサイズを保存する
			_orgWidth = _loader.contentLoaderInfo.width;
			_orgHeight = _loader.contentLoaderInfo.height;
			
			// 読み込んだイメージを破棄する
			_loader.unload();
			
			// 描画を更新する
			_update();
			
			// イベントを送出する
			dispatchEvent( e );
		}
		
		/*======================================================================*//**
		* ロードされたオブジェクトが Loader オブジェクトの unload() メソッドを使用して削除されるたびに、LoaderInfo オブジェクトによって送出されます。
		*/
		private function _unload( e:Event ):void {
			_bytesLoaded = 0;
			_bytesTotal = 0;
			
			// イベントを送出する
			dispatchEvent( e );
		}
		
		/*======================================================================*//**
		* ダウンロード処理を実行中にデータを受信したときに送出されます。
		*/
		private function _progress( e:ProgressEvent ):void {
			_bytesLoaded = e.bytesLoaded;
			_bytesTotal = e.bytesTotal;
			
			// イベントを送出する
			dispatchEvent( e );
		}
		
		/*======================================================================*//**
		* ネットワーク要求が HTTP を介して行われ、Flash Player が HTTP 状況コードを検出できる場合に送出されます。
		*/
		private function _httpStatus( e:HTTPStatusEvent ):void {
			// イベントを送出する
			dispatchEvent( e );
		}
		
		/*======================================================================*//**
		* 入出力エラーが発生してロード処理が失敗したときに送出されます。
		*/
		private function _ioError( e:IOErrorEvent ):void {
			// イベントを送出する
			dispatchEvent( e );
		}
	}
}





