﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.display {
	import flash.display.DisplayObject;
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.geom.Rectangle;
	import jp.nium.core.collections.DisplayList;
	import jp.nium.core.namespaces.nium_internal;
	import jp.nium.display.ChildIndexer;
	import jp.nium.display.IExDisplayObject;
	import jp.nium.display.IExDisplayObjectContainer;
	import jp.nium.events.EventIntegrator;
	import jp.nium.events.IEventIntegrator;
	import jp.nium.utils.ObjectUtil;
	
	use namespace nium_internal;
	
	/*======================================================================*//**
	* ExMovieClip クラスは、MovieClip クラスの基本機能を拡張した jp.nium パッケージで使用される基本クラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class ExMovieClip extends MovieClip implements IExDisplayObjectContainer, IEventIntegrator {
		
		/*======================================================================*//**
		* 
		*/
		private static var _stacks					:Array = new Array();
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get id()					:String { return DisplayList.nium_internal::getIdByInstance( this ); }
		public function set id( value				:String ):void { DisplayList.nium_internal::addInstance( this, value ); }
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get children()				:Array { return _indexer.children; }
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get isDragging()			:Boolean { return _isDragging; }
		private var _isDragging						:Boolean = false;
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get listenOnStage()			:Boolean { return _listenOnStage; }
		public function set listenOnStage( value	:Boolean ):void {
			_listenOnStage = value;
			
			if ( !value ) { return; }
			
			if ( stage ) {
				// イベントリスナーを登録する
				restoreRemovedListeners();
			}
			else {
				// イベントリスナーを解除・登録する
				removeAllListeners();
			}
		}
		private var _listenOnStage					:Boolean = false;
		
		/*======================================================================*//**
		* インデックスを整理する ChildIndexer を取得します。
		*/
		private var _indexer						:ChildIndexer;
		
		/*======================================================================*//**
		* リスナーを整理する対象 EventIntegrator を取得します。
		*/
		private var _integrator						:EventIntegrator;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい ExMovieClip インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function ExMovieClip() {
			// EventIntegrator を作成する
			_integrator = new EventIntegrator( this );
			_integrator.nium_internal::initialize( {
				addEventListener	:super.addEventListener,
				removeEventListener	:super.removeEventListener,
				hasEventListener	:super.hasEventListener,
				willTrigger			:super.willTrigger,
				dispatchEvent		:super.dispatchEvent
			} );
			
			// ChildIndexer を作成する
			_indexer = new ChildIndexer( this );
			_indexer.nium_internal::initialize( {
				addChild		:super.addChild,
				addChildAt		:super.addChildAt,
				removeChild		:super.removeChild,
				removeChildAt	:super.removeChildAt,
				contains		:super.contains,
				getChildAt		:super.getChildAt,
				getChildByName	:super.getChildByName,
				getChildIndex	:super.getChildIndex,
				swapChildren	:super.swapChildren,
				swapChildrenAt	:super.swapChildrenAt
			} );
			
			// スタックが存在すれば実行する
			while ( _stacks.length > 0 ) {
				var o:Object = _stacks.shift();
				o.func.apply( o.target, o.args );
			}
			
			// イベントリスナーを登録する
			addExclusivelyEventListener( Event.ADDED_TO_STAGE, _addedToStage, false, int.MAX_VALUE, true );
			addExclusivelyEventListener( Event.REMOVED_FROM_STAGE, _removedFromStage, false, int.MAX_VALUE, true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function setProperties( props:Object ):DisplayObject {
			ObjectUtil.setProperties( this, props );
			return this;
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function addChild( child:DisplayObject ):DisplayObject {
			return _indexer.addChild( child );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function addChildAt( child:DisplayObject, index:int ):DisplayObject {
			return _indexer.addChildAt( child, index );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function addChildAtAbove( child:DisplayObject, index:int, addAbove:Boolean = true ):DisplayObject {
			return _indexer.addChildAt( child, index, addAbove );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function removeChild( child:DisplayObject ):DisplayObject {
			return _indexer.removeChild( child );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function removeChildAt( index:int ):DisplayObject {
			return _indexer.removeChildAt( index );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function removeAllChildren():void {
			_indexer.removeAllChildren();
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function contains( child:DisplayObject ):Boolean {
			return _indexer.contains( child );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function getChildAt( index:int ):DisplayObject {
			if ( !_indexer ) { return null; }
			
			return _indexer.getChildAt( index );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function getChildByName( name:String ):DisplayObject {
			return _indexer.getChildByName( name );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function getChildrenByClass( className:Class ):Array {
			var list:Array = [];
			var l:int = children.length;
			for ( var i:int = 0; i < l; i++ ) {
				if ( children[i] is className ) {
					list.push( children[i] );
				}
			}
			return list;
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function getChildIndex( child:DisplayObject ):int {
			return _indexer.getChildIndex( child );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function setChildIndex( child:DisplayObject, index:int ):void {
			_indexer.setChildIndex( child, index );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function setChildIndexAbove( child:DisplayObject, index:int, addAbove:Boolean = true ):void {
			_indexer.setChildIndex( child, index, addAbove );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function swapChildren( child1:DisplayObject, child2:DisplayObject ):void {
			_indexer.swapChildren( child1, child2 );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function swapChildrenAt( index1:int, index2:int ):void {
			_indexer.swapChildrenAt( index1, index2 );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function addEventListener( type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false ):void {
			if ( !_integrator ) {
				_stacks.push( {
					target		:this,
					func		:arguments.callee,
					args		:arguments
				} );
				return;
			}
			
			_integrator.addEventListener( type, listener, useCapture, priority, useWeakReference );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function addExclusivelyEventListener( type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false ):void {
			_integrator.addExclusivelyEventListener( type, listener, useCapture, priority, useWeakReference );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function removeEventListener( type:String, listener:Function, useCapture:Boolean = false ):void {
			_integrator.removeEventListener( type, listener, useCapture );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function completelyRemoveEventListener( type:String, listener:Function, useCapture:Boolean = false ):void {
			_integrator.completelyRemoveEventListener( type, listener, useCapture );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function dispatchEvent( event:Event ):Boolean {
			return _integrator.dispatchEvent( event );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function hasEventListener( type:String ):Boolean {
			return _integrator.hasEventListener( type );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function willTrigger( type:String ):Boolean {
			return _integrator.willTrigger( type );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function removeAllListeners( completely:Boolean = false ):void {
			_integrator.removeAllListeners( completely );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function restoreRemovedListeners():void {
			_integrator.restoreRemovedListeners();
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function startDrag( lockCenter:Boolean = false, bounds:Rectangle = null ):void {
			_isDragging = true;
			super.startDrag( lockCenter, bounds );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function stopDrag():void {
			super.stopDrag();
			_isDragging = false;
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object ExMovieClip]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* 表示オブジェクトが、直接、または表示オブジェクトを含むサブツリーの追加により、ステージ上の表示リストに追加されたときに送出されます。
		*/
		private function _addedToStage( e:Event ):void {
			listenOnStage = listenOnStage;
		}
		
		/*======================================================================*//**
		* 表示オブジェクトが、直接、または表示オブジェクトを含むサブツリーの削除により、表示リストから削除されようとしているときに送出されます。
		*/
		private function _removedFromStage( e:Event ):void {
			listenOnStage = listenOnStage;
		}
	}
}





