﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.display {
	import flash.accessibility.AccessibilityProperties;
	import flash.display.DisplayObject;
	import flash.display.DisplayObjectContainer;
	import flash.display.LoaderInfo;
	import flash.display.Stage;
	import flash.events.Event;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.geom.Transform;
	import jp.nium.events.IEventIntegrator;
	import jp.nium.utils.ObjectUtil;
	
	/*======================================================================*//**
	* 
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	*/
	public interface IExDisplayObject extends IEventIntegrator {
		
		/*======================================================================*//**
		* ExDisplayObject インスタンスに対して、一意に識別する為の String インスタンスを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		function get id()								:String;
		function set id( value							:String ):void;
		
		/*======================================================================*//**
		* 設定したリスナーが、ExDisplayObject インスタンスが stage 上に配置してある場合のみ受け取るようにするかどうかを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		function get listenOnStage()					:Boolean;
		function set listenOnStage( value				:Boolean ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get accessibilityProperties()			:AccessibilityProperties;
		function set accessibilityProperties( value		:AccessibilityProperties ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get alpha()							:Number;
		function set alpha( value						:Number ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get blendMode()						:String;
		function set blendMode( value					:String ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get cacheAsBitmap()					:Boolean;
		function set cacheAsBitmap( value				:Boolean ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get filters()							:Array;
		function set filters( value						:Array ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get height()							:Number;
		function set height( value						:Number ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get loaderInfo()						:LoaderInfo;
		
		/*======================================================================*//**
		* @private
		*/
		function get mask()								:DisplayObject;
		function set mask( value						:DisplayObject ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get mouseX()							:Number;
		
		/*======================================================================*//**
		* @private
		*/
		function get mouseY()							:Number;
		
		/*======================================================================*//**
		* @private
		*/
		function get name()								:String;
		function set name( value						:String ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get opaqueBackground()					:Object;
		function set opaqueBackground( value			:Object ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get parent()							:DisplayObjectContainer;
		
		/*======================================================================*//**
		* @private
		*/
		function get root()								:DisplayObject;
		
		/*======================================================================*//**
		* @private
		*/
		function get rotation()							:Number;
		function set rotation( value					:Number ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get scale9Grid()						:Rectangle;
		function set scale9Grid( value					:Rectangle ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get scaleX()							:Number;
		function set scaleX( value						:Number ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get scaleY()							:Number;
		function set scaleY( value						:Number ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get scrollRect()						:Rectangle;
		function set scrollRect( value					:Rectangle ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get stage()							:Stage;
		
		/*======================================================================*//**
		* @private
		*/
		function get transform()						:Transform;
		function set transform( value					:Transform ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get visible()							:Boolean;
		function set visible( value						:Boolean ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get width()							:Number;
		function set width( value						:Number ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get x()								:Number;
		function set x( value							:Number ):void;
		
		/*======================================================================*//**
		* @private
		*/
		function get y()								:Number;
		function set y( value							:Number ):void;
		
		
		
		
		
		/*======================================================================*//**
		* ExDisplayObject インスタンスに対して、複数のプロパティを一括設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	props	設定したいプロパティを含んだ Object です。
		* @return			設定対象の DisplayObject インスタンスです。
		*/
		function setProperties( props:Object ):DisplayObject;
		
		/*======================================================================*//**
		* @private
		*/
		function getBounds( targetCoordinateSpace:DisplayObject ):Rectangle;
		
		/*======================================================================*//**
		* @private
		*/
		function getRect( targetCoordinateSpace:DisplayObject ):Rectangle;
		
		/*======================================================================*//**
		* @private
		*/
		function globalToLocal( point:Point ):Point;
		
		/*======================================================================*//**
		* @private
		*/
		function hitTestObject( obj:DisplayObject ):Boolean;
		
		/*======================================================================*//**
		* @private
		*/
		function hitTestPoint( x:Number, y:Number, shapeFlag:Boolean = false ):Boolean;
		
		/*======================================================================*//**
		* @private
		*/
		function localToGlobal( point:Point ):Point;
	}
}





