﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.events {
	import flash.events.Event;
	import flash.events.IEventDispatcher;
	import flash.utils.Dictionary;
	
	/*======================================================================*//**
	* 
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class EventConnector {
		
		/*======================================================================*//**
		* 登録したコネクター情報を取得します。
		*/
		private static var _connecters			:Dictionary = new Dictionary( true );
		
		/*======================================================================*//**
		* 登録したコネクター数を取得します。
		*/
		private static var _numConnecters		:int = 0;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい EventConnector インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function EventConnector() {
		}
		
		
		
		
		
		/*======================================================================*//**
		* <div class="jp">
		* 2 つの EventDispatcher インスタンスを指定されたイベントで接続します。
		* </div>
		* <div class="en">
		* </div>
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	sender				イベントが送出される IEventDispatcher インスタンスです。
		* @param	receiver			イベントを受け取る IEventDispatcher インスタンスです。
		* @param	type				イベントのタイプです。
		* @param	useCapture			リスナーが、キャプチャ段階、またはターゲットおよびバブリング段階で動作するかどうかを判断します。useCapture を true に設定すると、リスナーはキャプチャ段階のみでイベントを処理し、ターゲット段階またはバブリング段階では処理しません。useCapture を false に設定すると、リスナーはターゲット段階またはバブリング段階のみでイベントを処理します。3 つの段階すべてでイベントを受け取るには、addEventListener を 2 回呼び出します。useCapture を true に設定して 1 度呼び出し、useCapture を false に設定してもう一度呼び出します。
		* @param	priority			イベントリスナーの優先度レベルです。優先度は、符号付き 32 ビット整数で指定します。数値が大きくなるほど優先度が高くなります。優先度が n のすべてのリスナーは、優先度が n -1 のリスナーよりも前に処理されます。複数のリスナーに対して同じ優先度が設定されている場合、それらは追加された順番に処理されます。デフォルトの優先度は 0 です。
		* @param	useWeakReference	リスナーへの参照が強参照と弱参照のいずれであるかを判断します。デフォルトである強参照の場合は、リスナーのガベージコレクションが回避されます。弱参照では回避されません。
		*/
		public static function connect( sender:IEventDispatcher, receiver:IEventDispatcher, type:String, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false ):void {
			// 既存の接続があれば削除する
			disconnect( sender, receiver, type, useCapture );
			
			// リスナー関数を作成する
			var listener:Function = function( e:Event ):void {
				// イベントを送出する
				receiver.dispatchEvent( e );
			};
			
			// イベントリスナー情報を保存する
			_connecters[++_numConnecters] = {
				id					:_numConnecters,
				sender				:sender,
				receiver			:receiver,
				type				:type,
				useCapture			:useCapture,
				priority			:priority,
				useWeakReference	:useWeakReference,
				listener			:listener
			};
			
			// イベントリスナーを登録する
			sender.addEventListener( type, listener, useCapture, priority, useWeakReference );
		}
		
		/*======================================================================*//**
		* <div class="jp">
		* 2 つの EventDispatcher インスタンスの指定されたイベントでの接続を解除します。
		* </div>
		* <div class="en">
		* </div>
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	sender				イベントが送出される IEventDispatcher インスタンスです。
		* @param	receiver			イベントを受け取る IEventDispatcher インスタンスです。
		* @param	type				イベントのタイプです。
		* @param	useCapture			リスナーが、キャプチャ段階、またはターゲットおよびバブリング段階に対して登録されたかどうかを示します。リスナーがキャプチャ段階だけでなくターゲット段階とバブリング段階にも登録されている場合は、removeEventListener() を 2 回呼び出して両方のリスナーを削除する必要があります。1 回は useCapture() を true に設定し、もう 1 回は useCapture() を false に設定する必要があります。
		*/
		public static function disconnect( sender:IEventDispatcher, receiver:IEventDispatcher, type:String, useCapture:Boolean = false ):void {
			// イベントリスナー情報を走査する
			for each ( var o:Object in _connecters ) {
				// 設定値が違っていれば次へ
				if ( o.sender != sender ) { continue; }
				if ( o.receiver != receiver ) { continue; }
				if ( o.type != type ) { continue; }
				if ( o.useCapture != useCapture ) { continue; }
				
				// イベントリスナーを削除する
				sender.removeEventListener( o.type, o.listener, o.useCapture );
				
				// 登録情報を削除する
				delete _connecters[o.id];
				break;
			}
		}
	}
}









