﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.events {
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.utils.Dictionary;
	import jp.nium.core.namespaces.nium_internal;
	import jp.nium.events.IEventIntegrator;
	
	use namespace nium_internal;
	
	/*======================================================================*//**
	* EventIntegrator クラスは、IEventIntegrator インターフェイスを実装した、EventDispatcher クラスのサブクラスです。
	* EventDispatcher　クラスと比較して、イベントをより統合的に扱うことができるようになります。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class EventIntegrator extends EventDispatcher implements IEventIntegrator {
		
		/*======================================================================*//**
		* 登録したイベントリスナー情報を取得します。
		*/
		private var _listeners					:Dictionary;
		
		/*======================================================================*//**
		* 登録したイベントリスナー数を取得します。
		*/
		private var _numListeners				:int = 0;
		
		/*======================================================================*//**
		* 
		*/
		private var _addEventListener			:Function;
		private var _removeEventListener		:Function;
		private var _hasEventListener			:Function;
		private var _willTrigger				:Function;
		private var _dispatchEvent				:Function;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい EventIntegrator インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	integrator	説明文
		*/
		public function EventIntegrator( integrator:IEventIntegrator = null ) {
			super( integrator );
			
			// 初期化する
			nium_internal::initialize( {
				addEventListener	:super.addEventListener,
				removeEventListener	:super.removeEventListener,
				hasEventListener	:super.hasEventListener,
				willTrigger			:super.willTrigger,
				dispatchEvent		:super.dispatchEvent
			} );
		}
		
		
		
		
		
		/*======================================================================*//**
		* @private
		*/
		nium_internal function initialize( initObject:Object ):void {
			// 既存のイベントリスナー登録があれば削除する
			removeAllListeners( true );
			
			try {
				// 指定された関数を設定する
				_addEventListener = initObject.addEventListener;
				_removeEventListener = initObject.removeEventListener;
				_hasEventListener = initObject.hasEventListener;
				_willTrigger = initObject.willTrigger;
				_dispatchEvent = initObject.dispatchEvent;
			}
			catch ( e:Error ) {
				throw new ArgumentError( "必要な引数が設定されていません。" );
			}
		}
		
		/*======================================================================*//**
		* イベントリスナーオブジェクトを EventIntegrator オブジェクトに登録し、リスナーがイベントの通知を受け取るようにします。
		* このメソッドを使用して登録されたリスナーを removeEventListener() メソッドで削除した場合には、restoreRemovedListeners() メソッドで再登録させることができます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	type				イベントのタイプです。
		* @param	listener			イベントを処理するリスナー関数です。この関数は Event インスタンスを唯一のパラメータとして受け取り、何も返さないものである必要があります。関数は任意の名前を持つことができます。
		* @param	useCapture			リスナーが、キャプチャ段階、またはターゲットおよびバブリング段階で動作するかどうかを判断します。useCapture を true に設定すると、リスナーはキャプチャ段階のみでイベントを処理し、ターゲット段階またはバブリング段階では処理しません。useCapture を false に設定すると、リスナーはターゲット段階またはバブリング段階のみでイベントを処理します。3 つの段階すべてでイベントを受け取るには、addEventListener を 2 回呼び出します。useCapture を true に設定して 1 度呼び出し、useCapture を false に設定してもう一度呼び出します。
		* @param	priority			イベントリスナーの優先度レベルです。優先度は、符号付き 32 ビット整数で指定します。数値が大きくなるほど優先度が高くなります。優先度が n のすべてのリスナーは、優先度が n -1 のリスナーよりも前に処理されます。複数のリスナーに対して同じ優先度が設定されている場合、それらは追加された順番に処理されます。デフォルトの優先度は 0 です。
		* @param	useWeakReference	リスナーへの参照が強参照と弱参照のいずれであるかを判断します。デフォルトである強参照の場合は、リスナーのガベージコレクションが回避されます。弱参照では回避されません。
		*/
		public override function addEventListener( type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false ):void {
			// 既存のイベントリスナー登録があれば削除する
			completelyRemoveEventListener( type, listener, useCapture );
			
			// イベントリスナー情報を保存する
			_listeners[++_numListeners] = {
				id					:_numListeners,
				type				:type,
				listener			:listener,
				useCapture			:useCapture,
				priority			:priority,
				useWeakReference	:useWeakReference
			};
			
			// イベントリスナーを登録する
			_addEventListener( type, listener, useCapture, priority, useWeakReference );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function addExclusivelyEventListener( type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false ):void {
			// イベントリスナーを登録する
			_addEventListener( type, listener, useCapture, priority, useWeakReference );
		}
		
		/*======================================================================*//**
		* EventIntegrator オブジェクトからリスナーを削除します。
		* このメソッドを使用して削除されたリスナーは、restoreRemovedListeners() メソッドで再登録させることができます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	type		イベントのタイプです。
		* @param	listener	削除するリスナーオブジェクトです。
		* @param	useCapture	リスナーが、キャプチャ段階、またはターゲットおよびバブリング段階に対して登録されたかどうかを示します。リスナーがキャプチャ段階だけでなくターゲット段階とバブリング段階にも登録されている場合は、removeEventListener() を 2 回呼び出して両方のリスナーを削除する必要があります。1 回は useCapture() を true に設定し、もう 1 回は useCapture() を false に設定する必要があります。
		*/
		public override function removeEventListener( type:String, listener:Function, useCapture:Boolean = false ):void {
			// イベントリスナーを削除する
			_removeEventListener( type, listener, useCapture );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function completelyRemoveEventListener( type:String, listener:Function, useCapture:Boolean = false ):void {
			// イベントリスナー情報を走査する
			for each ( var o:Object in _listeners ) {
				// 設定値が違っていれば次へ
				if ( o.type != type ) { continue; }
				if ( o.listener != listener ) { continue; }
				if ( o.useCapture != useCapture ) { continue; }
				
				// 登録情報を削除する
				delete _listeners[o.id];
				break;
			}
			
			// イベントリスナーを削除する
			_removeEventListener( type, listener, useCapture );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function removeAllListeners( completely:Boolean = false ):void {
			// イベントリスナー情報を走査する
			for each ( var o:Object in _listeners ) {
				// イベントリスナーを解除する
				_removeEventListener( o.type, o.listener, o.useCapture );
			}
			
			// 完全に消去するなら初期化する
			if ( completely ) {
				_listeners = new Dictionary( true );
			}
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function restoreRemovedListeners():void {
			// イベントリスナー情報を走査する
			for each ( var o:Object in _listeners ) {
				// イベントリスナーを登録する
				_addEventListener( o.type, o.listener, o.useCapture, o.priority, o.useWeakReference );
			}
		}
		
		/*======================================================================*//**
		* EventIntegrator インスタンスに、特定のイベントタイプに対して登録されたリスナーがあるかどうかを確認します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	event	イベントのタイプです。
		* @return			指定したタイプのリスナーが登録されている場合は true に、それ以外の場合は false になります。
		*/
		public override function hasEventListener( type:String ):Boolean {
			return _hasEventListener( type );
		}
		
		/*======================================================================*//**
		* 指定されたイベントタイプについて、この EventIntegrator インスタンスまたはその祖先にイベントリスナーが登録されているかどうかを確認します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	event	イベントのタイプです。
		* @return			指定したタイプのリスナーがトリガされた場合は true に、それ以外の場合は false になります。
		*/
		public override function willTrigger( type:String ):Boolean {
			return _willTrigger( type );
		}
		
		/*======================================================================*//**
		* イベントをイベントフローに送出します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	event	イベントフローに送出されるイベントオブジェクトです。イベントが再度送出されると、イベントのクローンが自動的に作成されます。イベントが送出された後にそのイベントの target プロパティは変更できないため、再送出処理のためにはイベントの新しいコピーを作成する必要があります。
		* @return			値が true の場合、イベントは正常に送出されました。値が false の場合、イベントの送出に失敗したか、イベントで preventDefault() が呼び出されたことを示しています。
		*/
		public override function dispatchEvent( event:Event ):Boolean {
			return _dispatchEvent( event );
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object EventIntegrator]";
		}
	}
}









