﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.events {
	import flash.events.Event;
	import flash.events.IEventDispatcher;
	import flash.utils.Dictionary;
	import jp.nium.events.EventIntegrator;
	
	/*======================================================================*//**
	* 登録された全てのイベントが発生した際に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType flash.events.Event.COMPLETE
	*/
	[Event( name="complete", type="flash.events.Event" )]
	
	/*======================================================================*//**
	* 複数のイベント発生をまとめて処理し、全ての完了を一括で送出するクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import flash.display.Sprite;
	* import flash.events.Event;
	* import flash.events.MouseEvent;
	* 
	* import jp.nium.events.EventUnit;
	* import jp.nium.graphics.Square;
	* 
	* // EventUnit を作成する
	* var eventUnit:EventUnit = new EventUnit();
	* eventUnit.addEventListener( Event.COMPLETE, function( e:Event ):void {
	* 	trace( "全てのイベントが発生完了" );
	* 	
	* 	// リセットする
	* 	eventUnit.reset();
	* 	} );
	* 	
	* // Square を作成する
	* var square1:Square = Square( addChild( new Square() ) );
	* square1.x = 0;
	* eventUnit.addEventDispatcher( square1, MouseEvent.CLICK );
	* 
	* // Square を作成する
	* var square2:Square = Square( addChild( new Square() ) );
	* square2.x = 100;
	* eventUnit.addEventDispatcher( square2, MouseEvent.CLICK );
	* 
	* // Square を作成する
	* var square3:Square = Square( addChild( new Square() ) );
	* square3.x = 200;
	* eventUnit.addEventDispatcher( square3, MouseEvent.CLICK );
	* </listing>
	*/
	public class EventUnit extends EventIntegrator {
		
		/*======================================================================*//**
		* 登録したイベントリスナー情報を取得します。
		*/
		private var _dispatchers				:Dictionary;
		
		/*======================================================================*//**
		* 登録したイベントリスナー数を取得します。
		*/
		private var _numDispatchers				:int = 0;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい EventUnit インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function EventUnit() {
			// 初期化する
			_dispatchers = new Dictionary( true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* イベントユニットに IEventDispatcher インスタンスを登録します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	dispatcher	登録したい IEventDispatcher インスタンスです。
		* @param	type		登録したいイベントタイプです。
		*/
		public function addEventDispatcher( dispatcher:IEventDispatcher, type:String ):void {
			// 既存のイベントディスパッチャー登録があれば削除する
			removeEventDispatcher( dispatcher, type );
			
			// イベントディスパッチャー情報を保存する
			_dispatchers[++_numDispatchers] = {
				id					:_numDispatchers,
				dispatcher			:dispatcher,
				type				:type,
				dispatched			:false
			};
			
			// イベントリスナーを登録する
			dispatcher.addEventListener( type, _aggregate, false, 0, true );
		}
		
		/*======================================================================*//**
		* イベントユニットから IEventDispatcher インスタンスを削除します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	dispatcher	削除したい EventDispatcher インスタンスです。
		* @param	type		削除したいイベントタイプです。
		*/
		public function removeEventDispatcher( dispatcher:IEventDispatcher, type:String ):void {
			// イベントディスパッチャー情報を走査する
			for each ( var o:Object in _dispatchers ) {
				// 設定値が違っていれば次へ
				if ( o.dispatcher != dispatcher ) { continue; }
				if ( o.type != type ) { continue; }
				
				// 登録情報を削除する
				delete _dispatchers[o.id];
				break;
			}
			
			// イベントリスナーを削除する
			dispatcher.removeEventListener( type, _aggregate, false );
		}
		
		/*======================================================================*//**
		* イベントユニットから IEventDispatcher インスタンスを削除します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	target	削除したい EventDispatcher インスタンスです。
		* @param	type	削除したいイベントタイプです。
		*/
		public function removeAllEventDispatcher():void {
			// イベントディスパッチャー情報を走査する
			for each ( var o:Object in _dispatchers ) {
				// イベントリスナーを解除する
				o.dispatcher.removeEventListener( o.type, _aggregate, false );
			}
			
			// 初期化する
			_dispatchers = new Dictionary( true );
		}
		
		/*======================================================================*//**
		* 登録済みのイベントを全て未発生状態に設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function reset():void {
			// イベントディスパッチャー情報を走査する
			for each ( var o:Object in _dispatchers ) {
				// イベント発生を無効化する
				o.dispatched = false;
			}
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object EventUnit]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* 
		*/
		private function _aggregate( e:Event ):void {
			// イベントディスパッチャー情報を走査する
			for each ( var o:Object in _dispatchers ) {
				// 設定値が違っていれば次へ
				if ( o.dispatcher != e.target ) { continue; }
				if ( o.type != e.type ) { continue; }
				
				// イベント発生を有効化する
				o.dispatched = true;
				break;
			}
			
			// イベントディスパッチャー情報を走査する
			for each ( o in _dispatchers ) {
				// イベントが発生していなければ終了する
				if ( !o.dispatched ) { return; }
			}
			
			// イベントを送出する
			dispatchEvent( new Event( Event.COMPLETE ) );
		}
	}
}









