﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.events {
	import flash.display.FrameLabel;
	import flash.display.MovieClip;
	import flash.display.Scene;
	import flash.events.Event;
	import jp.nium.events.EventIntegrator;
	
	/*======================================================================*//**
	* 
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType flash.events.Event.CHANGE
	*/
	[Event( name="change", type="flash.events.Event" )]
	
	/*======================================================================*//**
	* 
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class OnFrameDispatcher extends EventIntegrator {
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onChange()			:Function { return __onChange || _onChange; }
		public function set onChange( value		:Function ):void { __onChange = value; }
		private var __onChange					:Function;
		
		/*======================================================================*//**
		* サブクラスで onChange イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onChange プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onChange
		*/
		protected function _onChange()			:void {}
		
		/*======================================================================*//**
		* フレームを監視する対象 MovieClip を取得します。
		*/
		private var _container					:MovieClip;
		
		/*======================================================================*//**
		* 前回のフレーム番号を取得します。
		*/
		private var _previousFrame				:int;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい OnFrameDispatcher インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	container	説明文
		*/
		public function OnFrameDispatcher( container:MovieClip ) {
			// 引数を格納する
			_container = container;
			
			// イベントリスナーを登録する
			_container.addEventListener( Event.ENTER_FRAME, _enterFrame, false, int.MAX_VALUE, true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object OnFrameDispatcher]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* 再生ヘッドが新しいフレームに入るときに送出されます。
		*/
		private function _enterFrame( e:Event ):void {
			// フレーム番号が変更されていなければ終了する
			if ( _previousFrame == _container.currentFrame ) { return; }
			
			// 現在のフレーム番号を保存する
			_previousFrame = _container.currentFrame;
			
			var evt:Event = new Event( Event.CHANGE );
			var scenes:Array = _container.scenes;
			var l:int = scenes.length;
			for ( var i:int = 0; i < l; i++ ) {
				var labels:Array = scenes[i].labels;
				var ll:int = labels.length;
				for ( var ii:int = 0; ii < ll; ii++ ) {
					// 現在のフレームではなければ次へ
					if ( labels[ii].frame != _container.currentFrame ) { continue; }
					
					// イベントを送出する
					dispatchEvent( evt );
					
					// イベントハンドラメソッドを実行する
					onChange();
				}
			}
		}
	}
}





