﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.external {
	import flash.events.Event;
	import flash.external.ExternalInterface;
	import flash.system.Security;
	import jp.nium.events.EventIntegrator;
	
	/*======================================================================*//**
	* Browser クラスを直接インスタンス化することはできません。
	* new Browser() コンストラクタを呼び出すと、ArgumentError 例外がスローされます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class Browser {
		
		/*======================================================================*//**
		* ブラウザとの JavaScript 通信が可能かどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get enabled()			:Boolean { return function():Boolean {
			if ( _integrator ) { return _enabled; }
			
			// EventIntegrator を作成する
			_integrator = new EventIntegrator();
			
			// オンラインでなければ false を返す
			if ( !online ) { return _enabled = false; }
			
			// ExternalInterface が有効でなければ false を返す
			if ( !ExternalInterface.available ) { return _enabled = false; }
			
			// ブラウザで JavaScript を実行できなければ false を返す
			if ( ExternalInterface.call( "function() { return \"true\"; }" ) != "true" ) { return _enabled = false; }
			
			return _enabled = true;
		}.apply(); };
		private static var _enabled						:Boolean = false;
		
		/*======================================================================*//**
		* SWF ファイルがオンライン上で実行されているかどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get online()				:Boolean {
			switch ( Security.sandboxType ) {
				case Security.REMOTE				: { return true; }
				case Security.LOCAL_WITH_FILE		:
				case Security.LOCAL_WITH_NETWORK	:
				case Security.LOCAL_TRUSTED			:
				default								: { return false; }
			}
		}
		
		/*======================================================================*//**
		* ブラウザのコード名を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get appCodeName()		:String { return call( "function() { return navigator.appCodeName; }" ); }
		
		/*======================================================================*//**
		* アプリケーション名を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get appName()			:String { return call( "function() { return navigator.appName; }" ); }
		
		/*======================================================================*//**
		* バージョンと機種名を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get appVersion()			:String { return call( "function() { return navigator.appVersion; }" ); }
		
		/*======================================================================*//**
		* Java アプレットが実行可能かどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get javaenabled()		:Boolean { return call( "function() { return navigator.javaenabled(); }" ) == "true"; }
		
		/*======================================================================*//**
		* プラットフォームを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get platform()			:String { return call( "function() { return navigator.platform; }" ); }
		
		/*======================================================================*//**
		* データを送信できるかどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get taintenabled()		:Boolean { return call( "function() { return navigator.taintenabled();" ) == "true"; }
		
		/*======================================================================*//**
		* エージェント名を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get userAgent()			:String { return call( "function() { return navigator.userAgent; }" ); }
		
		/*======================================================================*//**
		* 実行環境が Internet Explorer かどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get isIE()				:Boolean { return enabled ? new RegExp( "MSIE" ).test( userAgent ) : false; }
		
		/*======================================================================*//**
		* 実行環境が Gecko かどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get isGecko()			:Boolean { return enabled ? new RegExp( "Gecko" ).test( userAgent ) : false; }
		
		/*======================================================================*//**
		* 実行環境が Opera かどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get isOpera()			:Boolean { return enabled ? new RegExp( "Opera" ).test( userAgent ) : false; }
		
		/*======================================================================*//**
		* 実行環境が Safari かどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get isSafari()			:Boolean { return enabled ? new RegExp( "Safari" ).test( userAgent ) : false; }
		
		/*======================================================================*//**
		* ブラウザの URL を取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get url()				:String { return call( "function() { return window.location.href; }" ); }
		public static function set url( value			:String ):void { call( "function() { window.location.href = \"" +value + "\"; }" ); }
		
		/*======================================================================*//**
		* URL のスキームを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get scheme()				:String { return String( url.split( "#" ).shift() ); }
		public static function set scheme( value		:String ):void { url = value + "#" + fragment; }
		
		/*======================================================================*//**
		* URL のフラグメントを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get fragment()			:String {
			var path:Array = url.split( "#" );
			
			// 存在しなければ
			if ( path.length <= 1 ) { return ""; }
			
			return String( path.slice( 1 ).join( "#" ) );
		}
		public static function set fragment( value		:String ):void { url = scheme + "#" + value; }
		
		/*======================================================================*//**
		* ブラウザのドキュメントタイトルを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get title()				:String { return call( "function() { return document.title;" ) }
		public static function set title( value			:String ):void { call( "function() { document.title = \"" + value + "\"; }" ); }
		
		/*======================================================================*//**
		* ブラウザのウィンドウステータスを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get status()				:String { return call( "function() { return window.status;" ) }
		public static function set status( value		:String ):void { call( "function() { window.status = \"" + value + "\"; }" ); }
		
		/*======================================================================*//**
		* EventIntegrator インスタンスを取得します。
		*/
		private static var _integrator					:EventIntegrator;
		
		
		
		
		
		/*======================================================================*//**
		* @private
		*/
		public function Browser() {
			throw new ArgumentError( "Error #2012: Browser クラスを直接インスタンス化することはできません。" );
		}
		
		
		
		
		
		/*======================================================================*//**
		* Flash Player コンテナで公開されている関数を呼び出し、必要に応じてパラメータを渡します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	funcName	実行したい関数名です。
		* @param	args		引数に指定したい配列です。
		* @return				関数の戻り値を返します。
		*/
		public static function call( funcName:String, ... args:Array ):String {
			if ( !enabled ) { return ""; }
			args.unshift( funcName );
			var result:String = ExternalInterface.call.apply( null, args );
			return result || "";
		}
		
		/*======================================================================*//**
		* ActionScript メソッドをコンテナから呼び出し可能なものとして登録します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	funcName	コンテナが関数を呼び出すことができる名前です。
		* @param	closure		呼び出す関数閉包です。これは独立した関数にすることも、オブジェクトインスタンスのメソッドを参照するメソッド閉包とすることもできます。メソッド閉包を渡すことで、特定のオブジェクトインスタンスのメソッドでコールバックを実際にダイレクトできます。
		*/
		public static function addCallback( funcName:String, closure:Function ):void {
			if ( !enabled ) { return; }
			ExternalInterface.addCallback( funcName, closure );
		}
		
		/*======================================================================*//**
		* JavaScript を使用したアラートを表示します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	messages	出力したい String インスタンスです。
		*/
		public static function alert( ... messages:Array ):void {
			var message:String = messages.join( " " );
			
			trace( "Browser.alert() :", message );
			call( "function() { alert( \"" + message + "\" ); }" );
		}
		
		/*======================================================================*//**
		* ブラウザを再読み込みします。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function reload():void {
			call( "function() { window.location.reload(); }" );
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function historyForward():void {
			call( "function() { history.forward(); }" );
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function historyBack():void {
			call( "function() { history.back(); }" );
		}
		
		/*======================================================================*//**
		* ブラウザの印刷ダイアログを表示します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function print():void {
			call( "function() { window.print(); }" );
		}
		
		/*======================================================================*//**
		* @copy jp.nium.events.EventIntegrator#addEventListener()
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function addEventListener( type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false ):void {
			if ( !_integrator ) { return; }
			_integrator.addEventListener( type, listener, useCapture, priority, useWeakReference );
		}
		
		/*======================================================================*//**
		* @copy jp.nium.events.EventIntegrator#addExclusivelyEventListener()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function addExclusivelyEventListener( type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false ):void {
			if ( !_integrator ) { return; }
			_integrator.addExclusivelyEventListener( type, listener, useCapture, priority, useWeakReference );
		}
		
		/*======================================================================*//**
		* @copy jp.nium.events.EventIntegrator#removeEventListener()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function removeEventListener( type:String, listener:Function, useCapture:Boolean = false ):void {
			if ( !_integrator ) { return; }
			_integrator.removeEventListener( type, listener, useCapture );
		}
		
		/*======================================================================*//**
		* @copy jp.nium.events.EventIntegrator#completelyRemoveEventListener()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function completelyRemoveEventListener( type:String, listener:Function, useCapture:Boolean = false ):void {
			if ( !_integrator ) { return; }
			_integrator.completelyRemoveEventListener( type, listener, useCapture );
		}
		
		/*======================================================================*//**
		* @copy jp.nium.events.EventIntegrator#dispatchEvent()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function dispatchEvent( event:Event ):Boolean {
			if ( !_integrator ) { return false; }
			return _integrator.dispatchEvent( event );
		}
		
		/*======================================================================*//**
		* @copy jp.nium.events.EventIntegrator#hasEventListener()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function hasEventListener( type:String ):Boolean {
			if ( !_integrator ) { return false; }
			return _integrator.hasEventListener( type );
		}
		
		/*======================================================================*//**
		* @copy jp.nium.events.EventIntegrator#willTrigger()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function willTrigger( type:String ):Boolean {
			if ( !_integrator ) { return false; }
			return _integrator.willTrigger( type );
		}
		
		/*======================================================================*//**
		* @copy jp.nium.events.EventIntegrator#removeAllListeners()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function removeAllListeners( completely:Boolean = false ):void {
			if ( !_integrator ) { return; }
			_integrator.removeAllListeners( completely );
		}
		
		/*======================================================================*//**
		* @copy jp.nium.events.EventIntegrator#restoreRemovedListeners()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function restoreRemovedListeners():void {
			if ( !_integrator ) { return; }
			_integrator.restoreRemovedListeners();
		}
	}
}





