﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.external {
	import adobe.utils.MMExecute;
	import jp.nium.utils.ArrayUtil;
	import jp.nium.utils.StringUtil;
	import jp.nium.events.EventIntegrator;
	
	/*======================================================================*//**
	* JSFLInterface クラスを直接インスタンス化することはできません。
	* new JSFLInterface() コンストラクタを呼び出すと、ArgumentError 例外がスローされます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class JSFLInterface {
		
		/*======================================================================*//**
		* Flash IDE との JSFL 通信が可能かどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get enabled()		:Boolean { return function():Boolean {
			var result:String = MMExecute( "function() {}" );
			
			if ( _integrator ) { return _enabled; }
			
			// EventIntegrator を作成する
			_integrator = new EventIntegrator();
			
			// MMExecute を実行できなければ false を返す
			if ( result.length == 0 ) { return _enabled = false; }
			
			return _enabled = true;
		}.apply(); };
		private static var _enabled					:Boolean = false;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get configURI()		:String { return call( "( function() { return fl.configURI; } )" ); }
		
		/*======================================================================*//**
		* EventIntegrator インスタンスを取得します。
		*/
		private static var _integrator				:EventIntegrator;
		
		
		
		
		
		/*======================================================================*//**
		* @private
		*/
		public function JSFLInterface() {
			throw new ArgumentError( "Error #2012: JSFLInterface クラスを直接インスタンス化することはできません。" );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	funcName	説明文
		* @param	args		説明文
		* @return				説明文
		*/
		public static function call( funcName:String, ... args:Array ):String {
			if ( !enabled ) { return null; }
			
			var arg:String = ArrayUtil.toString( args );
			arg = StringUtil.collectBreak( arg );
			arg = arg.replace( new RegExp( "\n", "g" ), "\\n" );
			
			return MMExecute( funcName + ".apply( null, " + arg + " );" );
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	fileURL			説明文
		* @param	funcName		説明文
		* @param	args			説明文
		* @return					説明文
		*/
		public static function runScript( fileURL:String, funcName:String = "", ... args:Array ):String {
			var arg:String = ArrayUtil.toString( args );
			arg = arg.slice( 1, -1 );
			arg = StringUtil.collectBreak( arg );
			arg = arg.replace( new RegExp( "\n", "g" ), "\\n" );
			
			return MMExecute( "fl.runScript.apply( null, [ \"" + fileURL + "\", \"" + funcName + "\", " + arg + " ] );" );
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	messages	説明文
		* @return				説明文
		*/
		public static function alert( ... messages:Array ):void {
			var message:String = messages.join( " " );
			message = StringUtil.collectBreak( message );
			message = message.replace( new RegExp( "\n", "g" ), "\\n" );
			
			trace( "JSFLInterface.alert() :", message );
			call( "alert", message );
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	message		説明文
		* @return				説明文
		*/
		public static function confirm( ... messages:Array ):Boolean {
			var message:String = messages.join( " " );
			message = StringUtil.collectBreak( message );
			message = message.replace( new RegExp( "\n", "g" ), "\\n" );
			
			trace( "JSFLInterface.confirm() :", message );
			return !!StringUtil.toProperType( call( "confirm", message ) );
		}
	}
}














