﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.net {
	import flash.errors.IllegalOperationError;
	import flash.net.URLRequest;
	import jp.nium.net.ExURLLoader;
	import jp.nium.utils.StringUtil;
	
	/*======================================================================*//**
	* 
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class CSVLoader extends ExURLLoader {
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get delimiter()			:String { return _delimiter; }
		public function set delimiter( value	:String ):void {
			switch ( value ) {
				case "'"	:
				case "\""	: { throw new IllegalOperationError( "指定された値は設定できません。" ); }
			}
			
			_delimiter = value;
		}
		private var _delimiter					:String = ",";
		
		
		
		
		
		/*======================================================================*//**
		* 新しい CSVLoader インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	request		読み込む CSV ファイルの絶対 URL または相対 URL です。
		*/
		public function CSVLoader( request:URLRequest = null ) {
			super( request );
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @throws	ArgumentError	指定されたデータのフォーマットが csv 形式ではありません。
		*/
		public override function parse( data:* ):* {
			var csv:String = String( data );
			
			var line:int = 0;
			var result:Array = [];
			
			try {
				result[line] = [];
				
				// 改行コードを \n に統一する
				csv = StringUtil.collectBreak( csv );
				
				// 改行コードが存在しない場合には追加する
				if ( csv.indexOf( "\n" ) == -1 ) {
					csv += "\n";
				}
				
				// 改行コードの前に区切り文字を追加する
				csv = csv.replace( new RegExp( "\n", "g" ), delimiter + "\n" );
				
				while ( csv.length > 1 ) {
					// 改行コードがある場合
					if ( csv.charAt( 0 ) == "\n" ) {
						line++;
						result[line] = [];
						csv = csv.substr( 1 );
					}
					
					// 文字列の先頭をチェック
					switch ( csv.charAt( 0 ) ) {
						case "\"" : {
							// ダブルクォーテーションで括られている場合
							var n:Number = csv.indexOf( "\"" + delimiter );
							var str:String = csv.substr( 1, n - 1 );
							csv = csv.substr( n + 2 );
							break;
						}
						case "'" : {
							// シングルクォーテーションで括られている場合
							n = csv.indexOf( "'" + delimiter );
							str = csv.substr( 1, n - 1 );
							csv = csv.substr( n + 2 );
							break;
						}
						default : {
							// クォーテーションで括られていない場合
							n = csv.indexOf( delimiter );
							str = csv.substr( 0, n );
							csv = csv.substr( n + 1 );
						}
					}
					
					// 改行コード前の区切り文字を削除
					str = str.replace( delimiter + "\n", "\n" );
					
					// 任意の型に変換して代入する
					result[line].push( StringUtil.toProperType( str ) );
				}
			}
			catch ( e:Error ) {
				throw new ArgumentError( "指定されたデータのフォーマットが csv 形式ではありません。" );
			}
			
			return result;
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			var csv:String = "";
			
			var l:int = data.length;
			for ( var i:int = 0; i < l; i++ ) {
				var ll:int = data[i].length;
				for ( var ii:int = 0; ii < ll; ii++ ) {
					csv += String( data[i][ii] ) + delimiter;
				}
				
				// 最後のカンマを削除する
				csv = csv.slice( 0, -delimiter.length );
				
				// 改行を設定する
				csv += "\n";
			}
			
			return csv;
		}
	}
}










