﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.net {
	import flash.events.Event;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.utils.Dictionary;
	import jp.nium.core.namespaces.nium_internal;
	import jp.nium.events.EventIntegrator;
	import jp.nium.events.IEventIntegrator;
	
	use namespace nium_internal;
	
	/*======================================================================*//**
	* 
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class ExURLLoader extends URLLoader implements IEventIntegrator {
		
		/*======================================================================*//**
		* 
		*/
		private static var _stacks					:Array = new Array();
		
		/*======================================================================*//**
		* 
		*/
		private static var _cache					:Dictionary = new Dictionary( true );
		
		
		
		
		
		/*======================================================================*//**
		* 
		*/
		private var _url							:String;
		
		/*======================================================================*//**
		* リスナーを整理する対象 EventIntegrator を取得します。
		*/
		private var _integrator						:EventIntegrator;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい ExURLLoader インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	request		読み込むファイルの絶対 URL または相対 URL を表す URLRequest インスタンスです。
		*/
		public function ExURLLoader( request:URLRequest = null ) {
			super( request );
			
			// EventIntegrator を作成する
			_integrator = new EventIntegrator( this );
			_integrator.nium_internal::initialize( {
				addEventListener	:super.addEventListener,
				removeEventListener	:super.removeEventListener,
				hasEventListener	:super.hasEventListener,
				willTrigger			:super.willTrigger,
				dispatchEvent		:super.dispatchEvent
			} );
			
			// スタックが存在すれば実行する
			while ( _stacks.length > 0 ) {
				var o:Object = _stacks.shift();
				o.func.apply( o.target, o.args );
			}
			
			// イベントリスナーを登録する
			addExclusivelyEventListener( Event.COMPLETE, _complete, false, int.MAX_VALUE, true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function load( request:URLRequest ):void {
			_url = request.url;
			
			_cache[_url] = null;
			
			super.load( request );
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function loadAsCache( request:URLRequest ):void {
			_url = request.url;
			
			if ( _cache[_url] ) {
				data = _cache[_url];
				dispatchEvent( new Event( Event.COMPLETE ) );
				return;
			}
			
			super.load( request );
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function parse( data:* ):* {
			return data;
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function addEventListener( type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false ):void {
			if ( !_integrator ) {
				_stacks.push( {
					target		:this,
					func		:arguments.callee,
					args		:arguments
				} );
				return;
			}
			
			_integrator.addEventListener( type, listener, useCapture, priority, useWeakReference );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function addExclusivelyEventListener( type:String, listener:Function, useCapture:Boolean = false, priority:int = 0, useWeakReference:Boolean = false ):void {
			_integrator.addExclusivelyEventListener( type, listener, useCapture, priority, useWeakReference );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function removeEventListener( type:String, listener:Function, useCapture:Boolean = false ):void {
			_integrator.removeEventListener( type, listener, useCapture );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function completelyRemoveEventListener( type:String, listener:Function, useCapture:Boolean = false ):void {
			_integrator.completelyRemoveEventListener( type, listener, useCapture );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function dispatchEvent( event:Event ):Boolean {
			return _integrator.dispatchEvent( event );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function hasEventListener( type:String ):Boolean {
			return _integrator.hasEventListener( type );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function willTrigger( type:String ):Boolean {
			return _integrator.willTrigger( type );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function removeAllListeners( completely:Boolean = false ):void {
			_integrator.removeAllListeners( completely );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function restoreRemovedListeners():void {
			_integrator.restoreRemovedListeners();
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object ExURLLoader]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* データが正常にロードされたときに送出されます。
		*/
		private function _complete( e:Event ):void {
			// キャッシュする
			_cache[_url] = parse( ExURLLoader( e.target ).data );
			
			// 受信したデータをパースする
			ExURLLoader( e.target ).data = _cache[_url];
		}
	}
}













