﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.net {
	import flash.net.navigateToURL;
	import flash.net.URLRequest;
	
	/*======================================================================*//**
	* 
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class URLObject {
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @throws	SyntaxError		URL の構文が正しくありません。
		*/
		public function get url()					:String { return scheme + "://" + host + ( port ? ":" + port : "" ) + path + ( query ? "?" + query : "" ) + ( fragment ? "#" + fragment : "" ); }
		public function set url( value				:String ):void {
			if ( !validate( value ) ) { throw new SyntaxError( "URL の構文が正しくありません。" ); }
			
			_scheme = value.match( new RegExp( "^[A-Za-z][A-Za-z0-9+-.]*" ) )[0];
			
			_host = String( value.match( new RegExp( "://[A-Za-z0-9.-]+([:/]|\)" ) )[0] ).slice( 3, -1 );
			_port = 0;
			path = "";
			query = "";
			_fragment = "";
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @throws	SyntaxError		scheme の構文が正しくありません。
		*/
		public function get scheme()				:String { return _scheme; }
		public function set scheme( value			:String ):void {
			if ( !new RegExp( "^[A-Za-z][A-Za-z0-9+.-]*$", "g" ).test( value ) ) { throw new SyntaxError( "scheme の構文が正しくありません。" ); }
			
			_scheme = value;
		}
		private var _scheme							:String;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @throws	SyntaxError		host の構文が正しくありません。
		*/
		public function get host()					:String { return _host; }
		public function set host( value				:String ):void {
			if ( !new RegExp( "", "" ).test( value ) ) { throw new SyntaxError( "host の構文が正しくありません。" ); }
			
			_host = value;
		}
		private var _host							:String;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @throws	SyntaxError		ポート番号の指定が定められた範囲（0 ～ 65535）を超えています。
		*/
		public function get port()					:int { return _port; }
		public function set port( value				:int ):void {
			if ( port < 0 || 65535 < value ) { throw new SyntaxError( "ポート番号の指定が定められた範囲（0 ～ 65535）を超えています。" ); }
			
			_port = value;
		}
		private var _port							:int;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @throws	SyntaxError		path の構文が正しくありません。
		*/
		public function get path()					:String { return "/" + segments.join( "/" ) + ( filename ? "/" + filename + ( extension ? "." + extension : "" ) : "" ); }
		public function set path( value				:String ):void {
			if ( !new RegExp( "", "" ).test( value ) ) { throw new SyntaxError( "path の構文が正しくありません。" ); }
			
			_segments = [];
			_filename = "";
			_extension = "";
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @throws	SyntaxError		segments の構文が正しくありません。
		*/
		public function get segments()				:Array { return _segments; }
		public function set segments( value			:Array ):void {
			if ( !new RegExp( "", "" ).test( value.join( "/" ) ) ) { throw new SyntaxError( "segments の構文が正しくありません。" ); }
			
			_segments = value;
		}
		private var _segments						:Array;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @throws	SyntaxError		filename の構文が正しくありません。
		*/
		public function get filename()				:String { return _filename; }
		public function set filename( value			:String ):void {
			if ( !new RegExp( "", "" ).test( value ) ) { throw new SyntaxError( "filename の構文が正しくありません。" ); }
			
			_filename = value;
		}
		private var _filename						:String;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @throws	SyntaxError		extension の構文が正しくありません。
		*/
		public function get extension()				:String { return _extension; }
		public function set extension( value		:String ):void {
			if ( !new RegExp( "", "" ).test( value ) ) { throw new SyntaxError( "extension の構文が正しくありません。" ); }
			
			_extension = value;
		}
		private var _extension						:String;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @throws	SyntaxError		query の構文が正しくありません。
		*/
		public function get query()					:String { return _query; }
		public function set query( value			:String ):void {
			if ( !new RegExp( "", "" ).test( value ) ) { throw new SyntaxError( "query の構文が正しくありません。" ); }
			
			_query = value;
			
			// queryObject を作成する
			var object:Object = {};
			var list:Array = value.split( "&" );
			var l:int = list.length;
			for ( var i:int = 0; i < l; i++ ) {
				var item:Array = list[i].split( "=" );
				if ( item.length < 2 ) { continue; }
				object[item[0]] = item[1];
			}
			_queryObject = object;
		}
		private var _query							:String;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get queryObject()			:Object { return _queryObject; }
		public function set queryObject( value		:Object ):void {
			_queryObject = value;
			
			// query を作成する
			var str:String = "";
			for ( var p:String in value ) {
				str += encodeURI( p ) + "=" + encodeURI( value[p] ) + "&";
			}
			_query = str.slice( 0, -2 );
		}
		private var _queryObject					:Object;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @throws	SyntaxError		fragment の構文が正しくありません。
		*/
		public function get fragment()				:String { return _fragment; }
		public function set fragment( value			:String ):void {
			if ( !new RegExp( "", "" ).test( value ) ) { throw new SyntaxError( "fragment の構文が正しくありません。" ); }
			
			_fragment = value;
		}
		private var _fragment						:String;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい URLObject インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	url		説明文
		*/
		public function URLObject( url:String = null ) {
			// 初期化する
			_segments = [];
			_queryObject = {};
			
			// 引数を設定する
			this.url = url;
		}
		
		
		
		
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	path	説明文
		* @return			説明文
		*/
		public static function validate( path:String ):Boolean {
			return new RegExp( "", "g" ).test( path );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	path			説明文
		* @return					説明文
		* @throws	SyntaxError		path の構文が正しくありません。
		*/
		public function transfer( path:String ):String {
			if ( !new RegExp( "", "" ).test( path ) ) { throw new SyntaxError( "path の構文が正しくありません。" ); }
			
			return url;
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function cleanup():void {
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	window	説明文
		*/
		public function go( window:String = null ):void {
			navigateToURL( toURLRequest(), window );
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトの URLRequest 表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトの URLRequest 表現です。
		*/
		public function toURLRequest():URLRequest {
			return new URLRequest( url );
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public function toString():String {
			return url;
		}
	}
}





