﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.utils {
	import flash.geom.Point;
	
	/*======================================================================*//**
	* MathUtil クラスを直接インスタンス化することはできません。
	* new MathUtil() コンストラクタを呼び出すと、ArgumentError 例外がスローされます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	*/
	public class MathUtil {
		
		/*======================================================================*//**
		* @private
		*/
		public function MathUtil() {
			throw new ArgumentError( "Error #2012: MathUtil クラスを直接インスタンス化することはできません。" );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 数値を指定された周期内に収めて返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		* @param	number	周期内に収めたい数値です。
		* @param	cycle	周期となる数値です。
		* @return			変換後の数値です。
		*/
		public static function cycle( number:Number, cycle:Number ):Number {
			return ( number % cycle + cycle ) % cycle;
		}
		
		/*======================================================================*//**
		* 範囲内に適合する値を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		* @param	number	範囲内に適合させたい数値です。
		* @param	min		範囲の最小値となる数値です。
		* @param	max		範囲の最大値となる数値です。
		* @return			変換後の数値です。
		*/
		public static function range( number:Number, min:Number, max:Number ):Number {
			// min の方が max よりも大きい場合に入れ替える
			if ( min > max ) {
				var temp:Number = min;
				min = max;
				max = temp;
			}
			return Math.max( min, Math.min( number, max ) );
		}
		
		/*======================================================================*//**
		* 分母が 0 の場合に 0 となるパーセント値を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		* @param	numerator		分母となる数値です。
		* @param	denominator		分子となる数値です。
		* @return					変換後の数値です。
		*/
		public static function percent( numerator:Number, denominator:Number ):Number {
			if ( denominator == 0) { return 0; }
			return numerator / denominator * 100;
		}
		
		/*======================================================================*//**
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		* @param	rows	作成する格子の縦区切り数です。
		* @param	cols	作成する格子の横区切り数です。
		* @return			作成した格子データ配列です。
		*/
		public static function getTable( rows:int, cols:int ):Array {
			var table:Array = [];
			for ( var i:int = 0; i < rows; i++ ) {
				for ( var ii:int = 0; ii < cols; ii++ ) {
					table.push( new Point( ii, i ) );
				}
			}
			return table;
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		* @return	説明文
		*/
		public static function getCoin():int {
			return ( Math.random() < 0.5 ) ? 1 : -1; 
		}
	}
}





