﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.utils {
	import flash.display.FrameLabel;
	import flash.display.MovieClip;
	import flash.display.Scene;
	import flash.display.Sprite;
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	
	/*======================================================================*//**
	* MovieClipUtil クラスを直接インスタンス化することはできません。
	* new MovieClipUtil() コンストラクタを呼び出すと、ArgumentError 例外がスローされます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	*/
	public class MovieClipUtil {
		
		/*======================================================================*//**
		* @private
		*/
		public function MovieClipUtil() {
			throw new ArgumentError( "Error #2012: MovieClipUtil クラスを直接インスタンス化することはできません。" );
		}
		
		
		
		
		
		/*======================================================================*//**
		* MovieClip インスタンスのフレームラベルからフレーム番号を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		* @param	movie	取得したい MovieClip インスタンスです。
		* @param	label	取得したいフレームラベルです。
		* @return			フレーム番号です。
		*/
		public static function labelToFrames( movie:MovieClip, label:String ):int {
			var reg:RegExp = new RegExp( label );
			
			var scenes:Array = movie.scenes;
			
			// Scene を取得する
			var l:int = scenes.length;
			for ( var i:int = 0; i < l; i++ ) {
				// 参照を取得する
				var scene:Scene = scenes[i];
				
				// FrameLabel を取得する
				var ll:int = scene.labels.length;
				for ( var ii:int = 0; ii < ll; ii++ ) {
					// 参照を取得する
					var frameLabel:FrameLabel = scene.labels[ii];
					
					// 条件と一致すれば返す
					if ( reg.test( frameLabel.name ) ) { return frameLabel.frame; }
				}
			}
			
			return -1;
		}
		
		/*======================================================================*//**
		* MovieClip インスタンスのフレーム番号からフレームラベルを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		* @param	movie	取得したい MovieClip インスタンスです。
		* @param	frame	取得したいフレーム番号です。
		* @return			フレームラベルです。
		*/
		public static function frameToLabels( movie:MovieClip, frame:int ):String {
			var scenes:Array = movie.scenes;
			
			// Scene を取得する
			var l:int = scenes.length;
			for ( var i:int = 0; i < l; i++ ) {
				// 参照を取得する
				var scene:Scene = scenes[i];
				
				// FrameLabel を取得する
				var ll:int = scene.labels.length;
				for ( var ii:int = 0; ii < ll; ii++ ) {
					// 参照を取得する
					var frameLabel:FrameLabel = scene.labels[ii];
					
					// 条件と一致すれば追加する
					if ( frame == frameLabel.frame ) { return frameLabel.name; }
				}
			}
			
			return "";
		}
		
		/*======================================================================*//**
		* MovieClip インスタンスの再生ヘッドが指定された範囲内に存在するかどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		*/
		public static function playheadWithinFrames( movie:MovieClip, frame1:*, frame2:* ):Boolean {
			var s:int = isNaN( frame1 ) ? labelToFrames( movie, frame1 ) : frame1;
			var e:int = isNaN( frame2 ) ? labelToFrames( movie, frame2 ) : frame2;
			var c:int = movie.currentFrame;
			
			// s の方が e よりも大きい場合に入れ替える
			if ( s > e ) {
				var temp:Number = s;
				s = e;
				e = temp;
			}
			
			return s <= c && c <= e;
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		*/
		public static function doLater( target:*, callBack:Function, ... args:Array ):void {
			var timer:Timer = new Timer( 1, 1 );
			timer.addEventListener( TimerEvent.TIMER_COMPLETE, function( e:TimerEvent ):void {
				// イベントリスナーを解除する
				Timer( e.target ).removeEventListener( e.type, arguments.callee );
				
				// コールバック関数を実行する
				callBack.apply( target, args );
			} );
			timer.start();
		}
	}
}






