﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.utils {
	import flash.utils.getQualifiedClassName;
	import jp.nium.utils.ArrayUtil;
	
	/*======================================================================*//**
	* NumberUtil クラスを直接インスタンス化することはできません。
	* new NumberUtil() コンストラクタを呼び出すと、ArgumentError 例外がスローされます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	*/
	public class ObjectUtil {
		
		/*======================================================================*//**
		* @private
		*/
		public function ObjectUtil() {
			throw new ArgumentError( "Error #2012: ObjectUtil クラスを直接インスタンス化することはできません。" );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		* @param	target		説明文
		* @param	funcName	説明文
		* @param	funcArgs	説明文
		*/
		public static function applyFunctions( target:Object, funcName:String, funcArgs:Array ):void {
			// 対象を格納する配列を作成する
			var list:Array = [];
			
			// 対象が配列であれば結合する
			if ( target is Array ) {
				list = list.concat( target );
			}
			else {
				list.push( target );
			}
			
			// プロパティを設定する
			var l:int = list.length;
			for ( var i:int = 0; i < l; i++ ) {
				// プロパティが存在しなければ次へ
				if ( !( list[i][funcName] is Function ) ) { continue; }
				
				// プロパティを設定する
				list[i][funcName].apply( list[i], funcArgs );
			}
		}
		
		/*======================================================================*//**
		* 対象となる Object の複数のプロパティを一括設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		* @param	target	一括設定したい対象 Object です。
		* @param	props	設定したいプロパティを含んだ Object です。
		*/
		public static function setProperties( target:Object, props:Object ):void {
			// 対象を格納する配列を作成する
			var list:Array = [];
			
			// 対象が配列であれば結合する
			if ( target is Array ) {
				list = list.concat( target );
			}
			else {
				list.push( target );
			}
			
			// プロパティを設定する
			var l:int = list.length;
			for ( var i:int = 0; i < l; i++ ) {
				for ( var p:String in props ) {
					// プロパティが存在しなければ次へ
					if ( !( p in list[i] ) ) { continue; }
					
					// プロパティを設定する
					try {
						list[i][p] = props[p];
					}
					catch ( e:Error ) {}
				}
			}
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		* @param	target	説明文
		*/
		public static function toString( target:Object ):String {
			// Array であれば
			if ( getQualifiedClassName( target ) == "Array" ) { return ArrayUtil.toString( target as Array ); }
			
			var str:String = "{";
			
			for ( var p:String in target ) {
				str += p + ":";
				
				switch ( getQualifiedClassName( target[p] ) ) {
					case "Array"	: { str += ArrayUtil.toString( target[p] ); break; }
					case "Boolean"	:
					case "Number"	:
					case "int"		:
					case "uint"		: { str += target[p]; break; }
					case "String"	: { str += "\"" + target[p] + "\""; break; }
					case "Object"	:
					default			: { str += ObjectUtil.toString( target[p] ); }
				}
				
				str += ", ";
			}
			
			// 1 度でもループを処理していれば、最後の , を削除する
			if ( p ) {
				str = str.slice( 0, -2 );
			}
			
			str += "}";
			
			return str;
		}
	}
}















