﻿/*======================================================================*//**
* 
* jp.nium Classes
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://www.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.nium.utils {
	
	/*======================================================================*//**
	* StringUtil クラスを直接インスタンス化することはできません。
	* new StringUtil() コンストラクタを呼び出すと、ArgumentError 例外がスローされます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	*/
	public class StringUtil {
		
		/*======================================================================*//**
		* @private
		*/
		public function StringUtil() {
			throw new ArgumentError( "Error #2012: StringUtil クラスを直接インスタンス化することはできません。" );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		* @param	str			説明文
		* @param	priority	説明文
		* @return				説明文
		*/
		public static function toProperType( str:String, priority:Boolean = true ):* {
			// Number 型に変換する
			var num:Number = parseFloat( str );
			
			// モードが true なら
			if ( priority ) {
				// 数値化を優先する
				if ( !isNaN( num ) ) { return num; }
			}
			else {
				// 元データの維持を優先する
				if ( num.toString() == str ) { return num; }
			}
			
			// グローバル定数、プライマリ式キーワードで返す
			switch( str ) {
				case "true"			: { return true; }
				case "false"		: { return false; }
				case "null"			: { return null; }
				case "undefined"	: { return undefined; }
				case "Infinity"		: { return Infinity; }
				case "-Infinity"	: { return -Infinity; }
				case "NaN"			: { return NaN; }
			}
			
			return str;
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		* @param	str		説明文
		* @param	count	説明文
		* @return			説明文
		*/
		public static function repeat( str:String, count:int = 0 ):String {
			var result:String = "";
			count = Math.max( 0, count );
			for ( var i:int = 0; i < count; i++ ) {
				result += str;
			}
			return result;
		}
		
		/*======================================================================*//**
		* String の最初の文字を大文字にし、以降の文字を小文字に変換して返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		* @param	str		変換したい文字列です。
		* @return			変換後の文字列です。
		*/
		public static function toUpperCaseFirstLetter( str:String ):String {
			return str.charAt( 0 ).toUpperCase() + str.slice( 1 ).toLowerCase();
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		* @param	str		変換したい文字列です。
		* @return			変換後の文字列です。
		*/
		public static function camelize( str:String ):String {
			var words:Array = str.split( " " );
			var l:int = words.length;
			for ( var i:int = 0; i < l; i++ ) {
				words[i] = toUpperCaseFirstLetter( words[i] );
			}
			return words.join( "" );
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		* @param	str		変換したい文字列です。
		* @return			変換後の文字列です。
		*/
		public static function decamelize( str:String, separater:String = " " ):String {
			return str.replace( new RegExp( "[a-z][A-Z]", "g" ), function():String {
				return String( arguments[0] ).toLowerCase().split( "" ).join( separater );
				} );
		}
		
		/*======================================================================*//**
		* 
		* langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @example <listing version="3.0" >
		* </listing>
		* @param	str		説明文
		*/
		public static function collectBreak( str:String ):String {
			return str.replace( new RegExp( "(\n\r|\r)+", "g"), "\n" );
		}
	}
}





