﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression {
	import com.pixelbreaker.ui.osx.MacMouseWheel;
	import flash.display.Stage;
	import flash.errors.IllegalOperationError;
	import flash.system.Capabilities;
	import flash.utils.Dictionary;
	import flash.utils.getDefinitionByName;
	import jp.nium.events.EventIntegrator;
	import jp.nium.external.Browser;
	import jp.nium.text.ExTextField;
	import jp.nium.utils.ObjectUtil;
	import jp.progression.casts.ICastObject;
	import jp.progression.core.debug.Verbose;
	import jp.progression.core.display.Background;
	import jp.progression.core.display.Container;
	import jp.progression.core.managers.HistoryManager;
	import jp.progression.core.managers.KeyboardManager;
	import jp.progression.core.managers.SceneManager;
	import jp.progression.core.managers.SyncManager;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.events.ProcessEvent;
	import jp.progression.events.SceneEvent;
	import jp.progression.scenes.SceneId;
	import jp.progression.scenes.SceneObject;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* シーン移動処理が開始された場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.ProcessEvent.PROCESS_START
	*/
	[Event( name="processStart", type="jp.progression.events.ProcessEvent" )]
	
	/*======================================================================*//**
	* シーン移動処理が完了した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.ProcessEvent.PROCESS_COMPLETE
	*/
	[Event( name="processComplete", type="jp.progression.events.ProcessEvent" )]
	
	/*======================================================================*//**
	* シーン移動処理が停止された場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.ProcessEvent.PROCESS_INTERRUPT
	*/
	[Event( name="processInterrupt", type="jp.progression.events.ProcessEvent" )]
	
	/*======================================================================*//**
	* シーン移動処理中に対象シーンが変更された場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.ProcessEvent.PROCESS_SCENE
	*/
	[Event( name="processScene", type="jp.progression.events.ProcessEvent" )]
	
	/*======================================================================*//**
	* シーン移動処理中に対象シーンでイベントが発生した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.ProcessEvent.PROCESS_EVENT
	*/
	[Event( name="processEvent", type="jp.progression.events.ProcessEvent" )]
	
	/*======================================================================*//**
	* シーン移動処理中に移動先のシーンが存在しなかった場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.ProcessEvent.PROCESS_ERROR
	*/
	[Event( name="processError", type="jp.progression.events.ProcessEvent" )]
	
	/*======================================================================*//**
	* Progression クラスは、Progression Framework を実行する際に使用する基本クラスです。
	* シーン作成やシーン移動の処理は、Progression インスタンスを通じて操作を行います。
	* ただし、ブラウザ同期機能が使用できるインスタンスは 1 つのみです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.Progression;
	* import jp.progression.scenes.SceneObject;
	* 
	* // 開発者用の出力を有効化する
	* Progression.verbose.enabled = true;
	* 
	* // Progression インスタンスを作成する
	* var prog:Progression = new Progression( "index", stage );
	* 
	* // シーンを作成する
	* var page1:SceneObject = prog.root.addChild( new SceneObject( "page1" ) );
	* var page2:SceneObject = page1.addChild( new SceneObject( "page2" ) );
	* var page3:SceneObject = page2.addChild( new SceneObject( "page3" ) );
	* page3.onInit = function():void {
	* 	trace( "page3 を表示しました。" )
	* }
	* 
	* // シーンを移動する
	* prog.goto( page3.sceneId );
	* </listing>
	*/
	public class Progression extends EventIntegrator {
		
		/*======================================================================*//**
		* パッケージの名前を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const NAME							:String = "Progression Framework";
		
		/*======================================================================*//**
		* パッケージのバージョンを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const VERSION							:String = "2.0.22";
		
		/*======================================================================*//**
		* パッケージの制作者を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const AUTHOR							:String = "Copyright (c) 2008 taka:nium.jp, supported by Spark project.";
		
		/*======================================================================*//**
		* 対応している FlashPlayer のバージョンを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const PLAYER_VERSION					:String = "9,0,45,0";
		
		/*======================================================================*//**
		* Progression Framework の公式 Web サイトの URL を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		/*
		public static const OFFICIAL_WEBSITE				:String = "http://progression.jp/";
		*/
		
		/*======================================================================*//**
		* Progression Framework の技術情報を扱った公式 Web サイトの URL を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		/*
		public static const TECHNICAL_WEBSITE				:String = "http://progression.libspark.org/";
		*/
		
		/*======================================================================*//**
		* Progression Framework を使用した制作物の一般的な操作方法を記載した公式 Web サイトの URL を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		/*
		public static const USAGE_WEBSITE					:String = "http://usage.progression.jp/";
		*/
		
		
		
		
		
		/*======================================================================*//**
		* Progression Framework を起動した際にスプラッシュロゴを表示させるかどうかを指定します。
		* 表示のタイミングは、Progression インスタンスに対して、最初に goto() メソッドを使用した直後になります。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		/*
		public static function get splash()					:Boolean { return _splash; }
		public static function set splash( value			:Boolean ):void { _splash = value; }
		private static var _splash							:Boolean = false;
		*/
		
		/*======================================================================*//**
		* 開発者用に処理状態を出力する Verbose インスタンスを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get verbose()				:Verbose { return _verbose; }
		public static function set verbose( value			:* ):void { throw new IllegalOperationError( "Progression クラスの verbose プロパティは、より詳細な設定が可能な Verbose インスタンスに変更されました。\n同様の操作をする場合、今後は verbose.enable プロパティを使用してください。" ); }
		private static var _verbose							:Verbose = Verbose.progression_internal::createInstance();
		
		/*======================================================================*//**
		* Progression インスタンスリストを取得します。
		*/
		private static var _instances						:Dictionary = new Dictionary( true );
		
		
		
		
		
		/*======================================================================*//**
		* Progression 識別子を取得します。
		* この識別子は Progression インスタンスが持つルートシーン名と同じものが設定されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get id()							:String { return root.name; }
		
		/*======================================================================*//**
		* シーンオブジェクトのツリー構造部分の一番上にある SceneObject インスタンスを取得します。
		* このシーンオブジェクトの作成に使用するクラスは、インスタンス生成時に指定することができます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get root()							:SceneObject { return _sceneManager.root; }
		
		/*======================================================================*//**
		* @copy jp.progression.core.managers.SceneManager#errorScene
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get current()						:SceneObject { return _sceneManager.current; }
		
		/*======================================================================*//**
		* @copy jp.progression.core.managers.SceneManager#currentsceneId
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get currentSceneId()				:SceneId { return _sceneManager.currentSceneId; }
		
		/*======================================================================*//**
		* @copy jp.progression.core.managers.SceneManager#departedSceneId
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get departedSceneId()				:SceneId { return _sceneManager.departedSceneId; }
		
		/*======================================================================*//**
		* @copy jp.progression.core.managers.SceneManager#destinedSceneId
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get destinedSceneId()				:SceneId { return _sceneManager.destinedSceneId; }
		
		/*======================================================================*//**
		* Progression インスタンスを作成した際の状況から判断された、最初に表示するシーンに適したシーン識別子を取得します。
		* この値はオンラインコンテンツをブラウザ上で再生させた場合に、コンテンツが設置されている HTML ページの URL のフラグメント値から自動的に設定されます。
		* その他の状況で再生された場合には、常にルートシーンのシーン識別子が設定されています。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get firstSceneId()					:SceneId {
			var fragment:String = useLongPath ? _fragment : "/" + id + _fragment;
			return SceneId.validate( fragment ) ? new SceneId( fragment ) : root.sceneId;
		}
		private var _fragment								:String;
		
		/*======================================================================*//**
		* ブラウザ同期が有効化されている場合に、useLongPath プロパティの設定に対応したシーンパスを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get syncedScenePath()				:String {
			// 移動先のシーンパスを取得する
			var scenePath:String = String( currentSceneId );
			
			// ブラウザ同期が有効化されていて、ロングパスを使用していれば
			if ( sync && !useLongPath ) { return scenePath.replace( new RegExp( "^/" + id, "g" ), "" ); }
			
			return scenePath;
		}
		
		/*======================================================================*//**
		* @copy jp.progression.core.managers.SceneManager#eventType
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get eventType()						:String { return _sceneManager.eventType; }
		
		/*======================================================================*//**
		* インスタンスに関連付けられている stage インスタンスを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get stage()							:Stage { return _stage; }
		private var _stage									:Stage;
		
		/*======================================================================*//**
		* インスタンスに関連付けられている Container インスタンスを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get container()						:Container { return _container; }
		private var _container								:Container;
		
		/*======================================================================*//**
		* Background インスタンスを取得します。 
		*/
		private var _background								:Background;
		
		/*======================================================================*//**
		* 背景を透過するかどうかを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get opaqueBackground()				:Boolean { return _background.enabled; }
		public function set opaqueBackground( value			:Boolean ):void { _background.enabled = !value; }
		
		/*======================================================================*//**
		* @copy jp.progression.core.managers.SceneManager#lock
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get lock()							:Boolean { return _sceneManager.lock; }
		public function set lock( value						:Boolean ):void { _sceneManager.lock = value; }
		
		/*======================================================================*//**
		* @copy jp.progression.core.managers.SceneManager#autoLock
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get autoLock()						:Boolean { return _sceneManager.autoLock; }
		public function set autoLock( value					:Boolean ):void { _sceneManager.autoLock = value; }
		
		/*======================================================================*//**
		* @copy jp.progression.core.managers.SceneManager#running
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get running()						:Boolean { return _sceneManager.public::running; }
		
		/*======================================================================*//**
		* @copy jp.progression.core.managers.SceneManager#interrupting
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get interrupting()					:Boolean { return _sceneManager.public::interrupting; }
		
		/*======================================================================*//**
		* オンラインコンテンツをブラウザ上で実行している場合に、URL と Progression インスタンスのシーン識別子を同期させるかどうかを取得または設定します。
		* 同一コンテンツ上で有効化できる Progression インスタンスは 1 つのみであり、複数に対して有効化を試みた場合、
		* 最後に有効化された Progression インスタンス以外の sync プロパティは自動的に無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get sync()							:Boolean { return _syncManager.enabled; }
		public function set sync( value						:Boolean ):void { _syncManager.enabled = value; }
		
		/*======================================================================*//**
		* ブラウザ同期時に発行されるアドレスに root シーン名を含むかどうかを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get useLongPath()					:Boolean { return _syncManager.useLongPath; }
		public function set useLongPath( value				:Boolean ):void { _syncManager.useLongPath = value; }
		
		/*======================================================================*//**
		* キー操作に関連する設定を行う KeyboardManager インスタンスを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get keyboard()						:KeyboardManager { return _keyboard; }
		private var _keyboard								:KeyboardManager;
		
		/*======================================================================*//**
		* @private
		*/
		public function get keyboardEmulate()				:Boolean { throw new IllegalOperationError( "Progression クラスの keyboardEmulate プロパティは、より詳細な設定が可能な KeyboardManager インスタンスに変更されました。\n同様の操作をする場合、今後は keyboard.enabled プロパティを使用してください。" ); }
		public function set keyboardEmulate( value			:Boolean ):void { throw new IllegalOperationError( "Progression クラスの keyboardEmulate プロパティは、より詳細な設定が可能な KeyboardManager インスタンスに変更されました。\n同様の操作をする場合、今後は keyboard.enabled プロパティを使用してください。" ); }
		
		/*======================================================================*//**
		* SceneManager インスタンスを取得します。 
		*/
		private var _sceneManager							:SceneManager;
		
		/*======================================================================*//**
		* SyncManager インスタンスを取得します。 
		*/
		private var _syncManager							:SyncManager;
		
		/*======================================================================*//**
		* キャストオブジェクトが ProcessEvent.PROCESS_START イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onProcessStart()				:Function { return __onProcessStart || _onProcessStart; }
		public function set onProcessStart( value			:Function ):void { __onProcessStart = value; }
		private var __onProcessStart						:Function;
		
		/*======================================================================*//**
		* サブクラスで onProcessStart イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onProcessStart プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onProcessStart
		*/
		protected function _onProcessStart()				:void {}
		
		/*======================================================================*//**
		* キャストオブジェクトが ProcessEvent.PROCESS_COMPLETE イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onProcessComplete()				:Function { return __onProcessComplete || _onProcessComplete; }
		public function set onProcessComplete( value		:Function ):void { __onProcessComplete = value; }
		private var __onProcessComplete						:Function;
		
		/*======================================================================*//**
		* サブクラスで onProcessComplete イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onProcessComplete プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onProcessComplete
		*/
		protected function _onProcessComplete()				:void {}
		
		/*======================================================================*//**
		* キャストオブジェクトが ProcessEvent.PROCESS_INTERRUPT イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onProcessInterrupt()			:Function { return __onProcessInterrupt || _onProcessInterrupt; }
		public function set onProcessInterrupt( value		:Function ):void { __onProcessInterrupt = value; }
		private var __onProcessInterrupt					:Function;
		
		/*======================================================================*//**
		* サブクラスで onProcessInterrupt イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* onProcessInterrupt プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onProcessInterrupt
		*/
		protected function _onProcessInterrupt()			:void {}
		
		/*======================================================================*//**
		* キャストオブジェクトが ProcessEvent.PROCESS_SCENE イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onProcessScene()				:Function { return __onProcessScene || _onProcessScene; }
		public function set onProcessScene( value			:Function ):void { __onProcessScene = value; }
		private var __onProcessScene						:Function;
		
		/*======================================================================*//**
		* サブクラスで onProcessScene イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onProcessScene プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onProcessScene
		*/
		protected function _onProcessScene()				:void {}
		
		/*======================================================================*//**
		* キャストオブジェクトが ProcessEvent.PROCESS_EVENT イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onProcessEvent()				:Function { return __onProcessEvent || _onProcessEvent; }
		public function set onProcessEvent( value			:Function ):void { __onProcessEvent = value; }
		private var __onProcessEvent						:Function;
		
		/*======================================================================*//**
		* サブクラスで onProcessEvent イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onProcessEvent プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onProcessEvent
		*/
		protected function _onProcessEvent()				:void {}
		
		/*======================================================================*//**
		* キャストオブジェクトが SceneError.PROCESS_ERROR イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onProcessError()				:Function { return __onProcessError || _onProcessError; }
		public function set onProcessError( value			:Function ):void { __onProcessError = value; }
		private var __onProcessError						:Function;
		
		/*======================================================================*//**
		* サブクラスで onProcessError イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onProcessError プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onProcessError
		*/
		protected function _onProcessError()				:void {}
		
		
		
		
		
		/*======================================================================*//**
		* 新しい Progression インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	id						Progression 識別子です。
		* @param	stage					関連付けたい Stage インスタンスです。
		* @param	rootClass				ルートシーンに関連付けたいクラスの参照、またはパスの String 表現です。
		* @throws	IllegalOperationError	再生している FlashPlayer のバージョンには対応していません。
		* @throws	ArgumentError			指定された Progression 識別子は、すでに登録されています。
		* @throws	ArgumentError			rootClass の指定は String 型、または Class 型である必要があります。
		* @throws	ArgumentError			指定された rootClass クラスは、SceneObject クラスを継承していません。
		* @throws	ArgumentError			ルートシーンのコンストラクタで名前を変更することはできません。
		*/
		public function Progression( id:String, stage:Stage, rootClass:* = null ) {
			// パッケージ情報を出力する
			trace( "\n" + NAME + " " + VERSION + "\n" + AUTHOR + "\n" );
			
			// バージョンが対応していなければエラーを送出する
			if ( !checkVersion() ) { throw new IllegalOperationError( "Error: 再生している FlashPlayer のバージョンには対応していません。" ); }
			
			// Progression 識別子がすでに登録されていればエラーを送出する
			if ( _instances[id] ) { throw new ArgumentError( "指定された Progression 識別子は、すでに登録されています。" ); }
			
			// 起動メッセージを表示する
			Progression.verbose.log( this, null, 0, "Starting ... \"" + id + "\"", true );
			
			// インスタンスリストに登録する
			_instances[id] = this;
			
			// 引数を保存する
			_stage = stage;
			
			// URL のフラグメントを取得する
			_fragment = Browser.fragment;
			
			// Background を作成する
			_background = Background( stage.addChildAt( Background.progression_internal::createInstance(), 0 ) );
			
			// Container を作成する
			_container = Container( stage.addChild( Container.progression_internal::createInstance() ) );
			
			// SceneManager インスタンスを作成する
			_sceneManager = SceneManager.progression_internal::createInstance( this );
			_sceneManager.addExclusivelyEventListener( ProcessEvent.PROCESS_START, _processStart, false, int.MAX_VALUE, true );
			_sceneManager.addExclusivelyEventListener( ProcessEvent.PROCESS_COMPLETE, _processComplete, false, int.MAX_VALUE, true );
			_sceneManager.addExclusivelyEventListener( ProcessEvent.PROCESS_INTERRUPT, _processInterrupt, false, int.MAX_VALUE, true );
			_sceneManager.addExclusivelyEventListener( ProcessEvent.PROCESS_SCENE, _processScene, false, int.MAX_VALUE, true );
			_sceneManager.addExclusivelyEventListener( ProcessEvent.PROCESS_EVENT, _ProcessEvent, false, int.MAX_VALUE, true );
			_sceneManager.addExclusivelyEventListener( ProcessEvent.PROCESS_ERROR, _processError, false, int.MAX_VALUE, true );
			
			// KeyboardManager インスタンスを作成する
			_keyboard = KeyboardManager.progression_internal::createInstance( _stage );
			
			// SyncManager インスタンスを作成する
			_syncManager = SyncManager.progression_internal::createInstance( _sceneManager );
			
			// rootClass をクラスの参照に変換する
			switch ( true ) {
				case rootClass is String	: { var classRef:Class = Class( getDefinitionByName( rootClass ) ); break; }
				case rootClass is Class		: { classRef = Class( rootClass ); break; }
				case !rootClass				: { classRef = SceneObject; break; }
				default						: { throw new ArgumentError( "rootClass の指定は String 型、または Class 型である必要があります。" ); }
			}
			
			// classRef をクラスに変換する
			_sceneManager.root = new classRef( id );
			
			// root が SceneObject クラスを継承していなければエラーを送出する
			if ( !( root is SceneObject ) ) { throw new ArgumentError( "指定された " + rootClass + " クラスは、SceneObject クラスを継承していません。" ); }
			
			// root のシーン名が progression インスタンスの識別子と違っていればエラーを送出する
			if ( root.name != id ) { throw new ArgumentError( "ルートシーンのコンストラクタで名前を変更することはできません。" ); }
			
			// イベントを送出する
			root.dispatchEvent( new SceneEvent( SceneEvent.SCENE_ADDED, false, false, root ) );
			
			// MacMouseWheel を有効化する
			MacMouseWheel.setup( _stage );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 再生中の FlashPlayer に対応しているかどうかを返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	対応していれば true に、それ以外の場合は false になります。
		*/
		public static function checkVersion():Boolean {
			// バージョン情報を取得する
			var version:String = Capabilities.version.split( " " ).reverse()[0];
			
			// バージョンが同じであれば true を返す
			if ( version == PLAYER_VERSION ) { return true; }
			
			// 配列に分解する
			var ver1:Array = version.split( "," );
			var ver2:Array = PLAYER_VERSION.split( "," );
			
			// バージョンを比較する
			var l:int = ver1.length;
			for ( var i:int = 0; i < l; i++ ) {
				var v1:int = parseInt( ver1[i] );
				var v2:int = parseInt( ver2[i] );
				
				// バージョンが大きければ true を返す
				if ( v1 > v2 ) { return true; }
			}
			
			return false;
		}
		
		/*======================================================================*//**
		* 指定された Progression 識別子が設定されている Progression インスタンスを返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	id	条件となる Progression 識別子です。
		* @return		条件に一致した Progression インスタンスです。
		*/
		public static function getInstanceById( id:String ):Progression {
			return _instances[id];
		}
		
		/*======================================================================*//**
		* 指定されたシーン識別子の属している Progression インスタンスを返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	sceneId		条件となるシーン識別子です。
		* @return				条件に一致した Progression インスタンスです。
		*/
		public static function getInstanceBySceneId( sceneId:SceneId ):Progression {
			if ( !sceneId ) { return null; }
			
			return _instances[sceneId.getNameByIndex( 0 )];
		}
		
		/*======================================================================*//**
		* ブラウザ同期機能が有効化されている Progression インスタンスを返します。
		* 同期しているインスタンスが存在しない場合には null を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return		条件に一致した Progression インスタンスです。
		*/
		public static function getInstanceBySync():Progression {
			for each ( var instance:Progression in _instances ) {
				if ( instance.sync ) { return instance; }
			}
			
			return null;
		}
		
		/*======================================================================*//**
		* 指定されたシーン識別子またはシーンパスの指すシーンに移動します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	sceneIdOrScenePath		移動先となるシーン識別子またはシーンパスです。
		* @throws	ArgumentError			移動先に指定には SceneId インスタンスまたは String インスタンスを使用する必要があります。
		* @throws	ArgumentError			指定されたシーン識別子が属する Progression インスタンスが存在しません。
		*/
		public static function goto( sceneIdOrScenePath:* ):void {
			// 型によって振り分ける
			switch ( true ) {
				case sceneIdOrScenePath is String	: { var sceneId:SceneId = new SceneId( sceneIdOrScenePath ); break; }
				case sceneIdOrScenePath is SceneId	: { sceneId = SceneId( sceneIdOrScenePath ); break; }
				default								: { throw new ArgumentError( "移動先に指定には SceneId インスタンスまたは String インスタンスを使用する必要があります。" ); }
			}
			
			// Progression インスタンスを取得する
			var prog:Progression = getInstanceBySceneId( sceneId );
			
			// 存在しなければエラーを送出する
			if ( !prog ) { throw new ArgumentError( "指定されたシーン識別子が属する Progression インスタンスが存在しません。" ); }
			
			// 移動する
			prog.goto( sceneId );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 指定されたシーン識別子またはシーンパスの指すシーンに移動します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	sceneIdOrScenePath		移動先となるシーン識別子またはシーンパスです。
		* @throws	ArgumentError			移動先に指定には SceneId インスタンスまたは String インスタンスを使用する必要があります。
		*/
		public function goto( sceneIdOrScenePath:* ):void {
			// 型によって振り分ける
			switch ( true ) {
				case sceneIdOrScenePath is String	: { var sceneId:SceneId = new SceneId( sceneIdOrScenePath ); break; }
				case sceneIdOrScenePath is SceneId	: { sceneId = SceneId( sceneIdOrScenePath ); break; }
				default								: { throw new ArgumentError( "移動先に指定には SceneId インスタンスまたは String インスタンスを使用する必要があります。" ); }
			}
			
			_sceneManager.goto( sceneId );
		}
		
		/*======================================================================*//**
		* @copy jp.progression.core.maganers.SceneManager#interrupt()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function interrupt():void {
			_sceneManager.interrupt();
		}
		
		/*======================================================================*//**
		* Progression インスタンスに対して、複数のプロパティを一括設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	props	設定したいプロパティを含んだ Object です。
		* @return			設定対象の Progression インスタンスです。
		*/
		public function setProperties( props:Object ):Progression {
			ObjectUtil.setProperties( this, props );
			return this;
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object Progression : " + id + "]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* シーン移動処理が開始された場合に送出されます。
		*/
		private function _processStart( e:ProcessEvent ):void {
			Progression.verbose.log( this, "●", 0, "シーン移動を開始, 目的地=" + String( destinedSceneId ), true );
			
			// イベントを送出する
			dispatchEvent( e );
			
			// イベントハンドラメソッドを実行する
			onProcessStart();
		}
		
		/*======================================================================*//**
		* シーン移動処理が完了した場合に送出されます。
		*/
		private function _processComplete( e:ProcessEvent ):void {
			Progression.verbose.log( this, "■", 0, "シーン移動を完了\n" );
			
			// イベントを送出する
			dispatchEvent( e );
			
			// イベントハンドラメソッドを実行する
			onProcessComplete();
		}
		
		/*======================================================================*//**
		* シーン移動処理が停止された場合に送出されます。
		*/
		private function _processInterrupt( e:ProcessEvent ):void {
			Progression.verbose.log( this, "□", 0, "シーンの移動を停止\n" );
			
			// イベントを送出する
			dispatchEvent( e );
			
			// イベントハンドラメソッドを実行する
			onProcessInterrupt();
		}
		
		/*======================================================================*//**
		* シーン移動処理中に対象シーンが変更された場合に送出されます。
		*/
		private function _processScene( e:ProcessEvent ):void {
			Progression.verbose.log( this, "◎", 0, String( e.scene.sceneId ) + " シーンに移動" );
			
			// イベントを送出する
			dispatchEvent( e );
			
			// イベントハンドラメソッドを実行する
			onProcessScene();
		}
		
		/*======================================================================*//**
		* シーン移動処理中に対象シーンでイベントが発生した場合に送出されます。
		*/
		private function _ProcessEvent( e:ProcessEvent ):void {
			Progression.verbose.log( this, "○", 0, String( e.scene.sceneId ) + " シーンの " + e.eventType + " イベントを実行" );
			
			// イベントを送出する
			dispatchEvent( e );
			
			// イベントハンドラメソッドを実行する
			onProcessEvent();
		}
		
		/*======================================================================*//**
		* シーン移動処理中に移動先のシーンが存在しなかった場合に送出されます。
		*/
		private function _processError( e:ProcessEvent ):void {
			Progression.verbose.log( this, "□", 0, "移動先のシーンが存在しません, 目的地=" + String( destinedSceneId ) );
			
			// イベントを送出する
			dispatchEvent( e );
			
			// イベントハンドラメソッドを実行する
			onProcessError();
		}
	}
}





