﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.casts {
	import flash.display.MovieClip;
	import flash.display.Stage;
	import flash.errors.IllegalOperationError;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.net.navigateToURL;
	import flash.net.URLRequest;
	import flash.ui.ContextMenu;
	import flash.utils.getQualifiedClassName;
	import jp.nium.display.ExMovieClip;
	import jp.nium.display.ExSprite;
	import jp.nium.external.Browser;
	import jp.progression.casts.ICastObject;
	import jp.progression.commands.ParallelList;
	import jp.progression.commands.SerialList;
	import jp.progression.core.commands.Command;
	import jp.progression.core.commands.CommandList;
	import jp.progression.core.display.ToolTip;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.core.ui.CastButtonContextMenu;
	import jp.progression.events.CastEvent;
	import jp.progression.events.CastMouseEvent;
	import jp.progression.events.ProcessEvent;
	import jp.progression.Progression;
	import jp.progression.scenes.SceneId;
	import jp.progression.scenes.SceneObject;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* キャストオブジェクトが AddChild コマンド経由でディスプレイリストに追加された場合に送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CastEvent.CAST_ADDED
	*/
	[Event( name="castAdded", type="jp.progression.events.CastEvent" )]
	
	/*======================================================================*//**
	* キャストオブジェクトが RemoveChild コマンド経由でディスプレイリストから削除された場合に送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CastEvent.CAST_REMOVED
	*/
	[Event( name="castRemoved", type="jp.progression.events.CastEvent" )]
	
	/*======================================================================*//**
	* キャストオブジェクトの状態が変更された場合に送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CastEvent.CAST_STATE_CHANGE
	*/
	[Event( name="castStageChange", type="jp.progression.events.CastEvent" )]
	
	/*======================================================================*//**
	* ユーザーが CastButton をポインティングデバイスのメインボタンでクリックしたときに送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CastMouseEvent.CAST_CLICK
	*/
	[Event( name="castClick", type="jp.progression.events.CastMouseEvent" )]
	
	/*======================================================================*//**
	* インスタンスの doubleClickEnabled フラグが true に設定されている場合にユーザーが CastButton をポインティングデバイスのメインボタンで 2 回続けてすばやくクリックすると送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CastMouseEvent.CAST_DOUBLE_CLICK
	*/
	[Event( name="castDoubleClick", type="jp.progression.events.CastMouseEvent" )]
	
	/*======================================================================*//**
	* Flash Player ウィンドウの CastButton インスタンスの上でユーザーがポインティングデバイスのボタンを押すと送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CastMouseEvent.CAST_MOUSE_DOWN
	*/
	[Event( name="castMouseDown", type="jp.progression.events.CastMouseEvent" )]
	
	/*======================================================================*//**
	* ユーザーが CastButton インスタンスからポインティングデバイスを離したときに送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CastMouseEvent.CAST_MOUSE_UP
	*/
	[Event( name="castMouseUp", type="jp.progression.events.CastMouseEvent" )]
	
	/*======================================================================*//**
	* ユーザーが Flash Player ウィンドウの CastButton インスタンスにポインティングデバイスを合わせたときに送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CastMouseEvent.CAST_MOUSE_OVER
	*/
	[Event( name="castMouseOver", type="jp.progression.events.CastMouseEvent" )]
	
	/*======================================================================*//**
	* ユーザーが CastButton インスタンスからポインティングデバイスを離したときに送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CastMouseEvent.CAST_MOUSE_OUT
	*/
	[Event( name="castMouseOut", type="jp.progression.events.CastMouseEvent" )]
	
	/*======================================================================*//**
	* Flash Player ウィンドウの CastButton インスタンスの上でマウスホイールを回転させたときに送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CastMouseEvent.CAST_MOUSE_WHEEL
	*/
	[Event( name="castMouseWheel ", type="jp.progression.events.CastMouseEvent" )]
	
	/*======================================================================*//**
	* ユーザーが CastButton インスタンスにポインティングデバイスを合わせたときに送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CastMouseEvent.CAST_ROLL_OVER
	*/
	[Event( name="castRollOver", type="jp.progression.events.CastMouseEvent" )]
	
	/*======================================================================*//**
	* ユーザーが CastButton インスタンスからポインティングデバイスを離したときに送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CastMouseEvent.CAST_ROLL_OUT
	*/
	[Event( name="castRollOut", type="jp.progression.events.CastMouseEvent" )]
	
	/*======================================================================*//**
	* CastButton クラスは、CastMovieClip　クラスにより詳細なボタン機能を実装した拡張クラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class CastButton extends ExMovieClip implements ICastObject {
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get casts()								:Array { return getChildrenByClass( ICastObject ); }
		
		/*======================================================================*//**
		* @private
		*/
		public override function get contextMenu()				:ContextMenu { throw new IllegalOperationError( "CastButton クラスの contextMenu プロパティは、より詳細な設定が可能な uiContextMenu プロパティに変更されました。\n同様の操作をする場合、今後は uiContextMenu プロパティを使用してください。" ); }
		public override function set contextMenu( value			:ContextMenu ):void { throw new IllegalOperationError( "CastButton クラスの contextMenu プロパティは、より詳細な設定が可能な uiContextMenu プロパティに変更されました。\n同様の操作をする場合、今後は uiContextMenu プロパティを使用してください。" ); }
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function get contextMenu()			:ContextMenu { return super.contextMenu; }
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get uiContextMenu()						:CastButtonContextMenu { return _uiContextMenu; }
		private var _uiContextMenu								:CastButtonContextMenu;
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get parallelMode()						:Boolean { return _commandList is ParallelList; }
		public function set parallelMode( value					:Boolean ):void { _commandList = value ? new ParallelList() : new SerialList(); }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get toolTip()							:ToolTip { return _toolTip; }
		private var _toolTip									:ToolTip;
		
		/*======================================================================*//**
		* @private
		*/
		public function get toolChip()						:* { throw new IllegalOperationError( "toolChip プロパティは、スペルが間違っていたので toolTip プロパティに変更されました。\n同様の操作をする場合、今後は toolTip プロパティを使用してください。\nごめんなさいごめんなさい！" ); }
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get extra()								:Object { return _extra; }
		public function set extra( value						:Object ):void { _extra = value; }
		private var _extra										:Object;
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function get commandList()			:CommandList { return _commandList; }
		private var _commandList								:CommandList;
		
		/*======================================================================*//**
		* キャストオブジェクトが CastMouseEvent.CAST_CLICK イベントを受け取った場合に移動先となるシーン識別子を取得または設定します。
		* href プロパティと両方が設定されている場合には、href プロパティの値が優先されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get sceneId() 							:SceneId { return _sceneId; }
		public function set sceneId( value						:SceneId ):void {
			if ( _progression ) {
				_progression.completelyRemoveEventListener( ProcessEvent.PROCESS_SCENE, _processScene );
			}
			
			_sceneId = value;
			_progression = Progression.getInstanceBySceneId( value );
			
			if ( _progression ) {
				_progression.addExclusivelyEventListener( ProcessEvent.PROCESS_SCENE, _processScene, false, int.MAX_VALUE, true );
				_processScene( new ProcessEvent( ProcessEvent.PROCESS_SCENE, false, false, _progression.current, _progression.eventType ) );
			}
		}
		private var _sceneId									:SceneId;
		
		/*======================================================================*//**
		* キャストオブジェクトが CastMouseEvent.CAST_CLICK イベントを受け取った場合に移動先となる URL を取得または設定します。
		* sceneId プロパティと両方が設定されている場合には、こちらの値が優先されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get href() 								:String { return _href; }
		public function set href( value							:String ):void { _href = value; }
		private var _href										:String;
		
		/*======================================================================*//**
		* キャストオブジェクトが CastMouseEvent.CAST_CLICK イベントを受け取った場合に　href　プロパティで指定された URL に移動する際の対象ウィンドウを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get windowTarget() 						:String { return _href; }
		public function set windowTarget( value					:String ):void { _windowTarget = value; }
		private var _windowTarget								:String = "_self";
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function get currentNavigateURL() 	:String {
			// href が設定されていれば
			if ( _href ) { return _href; }
			
			// sceneId が設定されていなければ
			if ( !_sceneId ) { return ""; }
			
			// 移動先のシーンパスを取得する
			var scenePath:String = String( _sceneId );
			
			// ロングパスを使用してなければ
			if ( !_progression.useLongPath ) { scenePath = scenePath.replace( new RegExp( "^/" + _progression.id, "g" ), "" ); }
			
			return Browser.scheme + "#" + scenePath;
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @throws	ArgumentError	accessKey プロパティには a ～ z の値を指定する必要があります。
		*/
		public function get accessKey() 						:String { return _accessKey; }
		public function set accessKey( value					:String ):void {
			if ( value == "" ) { return; }
			if ( !( new RegExp( "^[A-Za-z]$", "g" ).test( value ) ) ) { throw new ArgumentError( "accessKey プロパティには a ～ z の値を指定する必要があります。" + value + " は不正な値です。" ); }
			
			_accessKey = value;
		}
		private var _accessKey									:String = "";
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get buttonEnabled()						:Boolean { return _buttonEnabled; }
		public function set buttonEnabled( value				:Boolean ):void {
			_buttonEnabled = value;
			
			buttonMode = value;
			useHandCursor = value;
			
			if ( value ) {
				// イベントリスナーを登録する
				addExclusivelyEventListener( Event.ADDED_TO_STAGE, _addedToStage, false, int.MAX_VALUE, true );
				addExclusivelyEventListener( Event.REMOVED_FROM_STAGE, _removedFromStage, false, int.MAX_VALUE, true );
				addExclusivelyEventListener( MouseEvent.CLICK, progression_internal::click, false, int.MAX_VALUE, true );
				addExclusivelyEventListener( MouseEvent.DOUBLE_CLICK, progression_internal::doubleClick, false, int.MAX_VALUE, true );
				addExclusivelyEventListener( MouseEvent.MOUSE_DOWN, progression_internal::mouseDown, false, int.MAX_VALUE, true );
				addExclusivelyEventListener( MouseEvent.MOUSE_UP, progression_internal::mouseUp, false, int.MAX_VALUE, true );
				addExclusivelyEventListener( MouseEvent.MOUSE_OVER, progression_internal::mouseOver, false, int.MAX_VALUE, true );
				addExclusivelyEventListener( MouseEvent.MOUSE_OUT, progression_internal::mouseOut, false, int.MAX_VALUE, true );
				addExclusivelyEventListener( MouseEvent.MOUSE_WHEEL, progression_internal::mouseWheel, false, int.MAX_VALUE, true );
				addExclusivelyEventListener( MouseEvent.ROLL_OVER, progression_internal::rollOver, false, int.MAX_VALUE, true );
				addExclusivelyEventListener( MouseEvent.ROLL_OUT, progression_internal::rollOut, false, int.MAX_VALUE, true );
			}
			else {
				// イベントリスナーを解除する
				completelyRemoveEventListener( Event.ADDED_TO_STAGE, _addedToStage );
				completelyRemoveEventListener( Event.REMOVED_FROM_STAGE, _removedFromStage );
				completelyRemoveEventListener( MouseEvent.CLICK, progression_internal::click );
				completelyRemoveEventListener( MouseEvent.DOUBLE_CLICK, progression_internal::doubleClick );
				completelyRemoveEventListener( MouseEvent.MOUSE_DOWN, progression_internal::mouseDown );
				completelyRemoveEventListener( MouseEvent.MOUSE_UP, progression_internal::mouseUp );
				completelyRemoveEventListener( MouseEvent.MOUSE_OVER, progression_internal::mouseOver );
				completelyRemoveEventListener( MouseEvent.MOUSE_OUT, progression_internal::mouseOut );
				completelyRemoveEventListener( MouseEvent.MOUSE_WHEEL, progression_internal::mouseWheel );
				completelyRemoveEventListener( MouseEvent.ROLL_OVER, progression_internal::rollOver );
				completelyRemoveEventListener( MouseEvent.ROLL_OUT, progression_internal::rollOut );
			}
			
			// イベントを送出する
			dispatchEvent( new CastEvent( CastEvent.CAST_STATE_CHANGE ) );
		}
		private var _buttonEnabled								:Boolean = true;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get isRollOver()						:Boolean { return _isRollOver; }
		private var _isRollOver									:Boolean = false;
		
		/*======================================================================*//**
		* このキャストオブジェクトに指定されているシーン識別子がカレントシーンであるかどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get isCurrent()							:Boolean { return _isCurrent; }
		private var _isCurrent									:Boolean = false;
		
		/*======================================================================*//**
		* このキャストオブジェクトに指定されているシーン識別子がカレントシーンオブジェクトの親シーンオブジェクトに当たるかどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get isParent()							:Boolean { return _isParent; }
		private var _isParent									:Boolean = false;
		
		/*======================================================================*//**
		* このキャストオブジェクトに指定されているシーン識別子がカレントシーンオブジェクトの子シーンオブジェクトに当たるかどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get isChild()							:Boolean { return _isChild; }
		private var _isChild									:Boolean = false;
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onCastAdded()						:Function { return __onCastAdded || _onCastAdded; }
		public function set onCastAdded( value					:Function ):void { __onCastAdded = value; }
		private var __onCastAdded								:Function;
		
		/*======================================================================*//**
		* @copy jp.progression.casts.CastSprite#_onCastAdded()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onCastAdded
		*/
		protected function _onCastAdded()						:void {}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onCastRemoved()						:Function { return __onCastRemoved || _onCastRemoved; }
		public function set onCastRemoved( value				:Function ):void { __onCastRemoved = value; }
		private var __onCastRemoved								:Function;
		
		/*======================================================================*//**
		* @copy jp.progression.casts.CastSprite#_onCastRemoved()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onCastRemoved
		*/
		protected function _onCastRemoved()						:void {}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onCastStateChange()					:Function { return __onCastStateChange || _onCastStateChange; }
		public function set onCastStateChange( value			:Function ):void { __onCastStateChange = value; }
		private var __onCastStateChange							:Function;
		
		/*======================================================================*//**
		* @copy jp.progression.casts.CastSprite#_onCastStateChange()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onCastStateChange
		*/
		protected function _onCastStateChange()					:void {}
		
		/*======================================================================*//**
		* キャストオブジェクトが CastMouseEvent.CAST_CLICK イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onCastClick()						:Function { return __onCastClick || _onCastClick; }
		public function set onCastClick( value					:Function ):void { __onCastClick = value; }
		private var __onCastClick								:Function;
		
		/*======================================================================*//**
		* サブクラスで onCastClick イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onCastClick プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onCastClick
		*/
		protected function _onCastClick()						:void {}
		
		/*======================================================================*//**
		* キャストオブジェクトが CastMouseEvent.CAST_DOUBLE_CLICK イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onCastDoubleClick()					:Function { return __onCastDoubleClick || _onCastDoubleClick; }
		public function set onCastDoubleClick( value			:Function ):void { __onCastDoubleClick = value; }
		private var __onCastDoubleClick							:Function;
		
		/*======================================================================*//**
		* サブクラスで onCastDoubleClick イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onCastDoubleClick プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onCastDoubleClick
		*/
		protected function _onCastDoubleClick()					:void {}
		
		/*======================================================================*//**
		* キャストオブジェクトが CastMouseEvent.CAST_MOUSE_DOWN イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onCastMouseDown()					:Function { return __onCastMouseDown || _onCastMouseDown; }
		public function set onCastMouseDown( value				:Function ):void { __onCastMouseDown = value; }
		private var __onCastMouseDown							:Function;
		
		/*======================================================================*//**
		* サブクラスで onCastMouseDown イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onCastMouseDown プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onCastMouseDown
		*/
		protected function _onCastMouseDown()					:void {}
		
		/*======================================================================*//**
		* キャストオブジェクトが CastMouseEvent.CAST_MOUSE_UP イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onCastMouseUp()						:Function { return __onCastMouseUp || _onCastMouseUp; }
		public function set onCastMouseUp( value				:Function ):void { __onCastMouseUp = value; }
		private var __onCastMouseUp								:Function;
		
		/*======================================================================*//**
		* サブクラスで onCastMouseUp イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onCastMouseUp プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onCastMouseUp
		*/
		protected function _onCastMouseUp()						:void {}
		
		/*======================================================================*//**
		* キャストオブジェクトが CastMouseEvent.CAST_MOUSE_OVER イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onCastMouseOver()					:Function { return __onCastMouseOver || _onCastMouseOver; }
		public function set onCastMouseOver( value				:Function ):void { __onCastMouseOver = value; }
		private var __onCastMouseOver							:Function;
		
		/*======================================================================*//**
		* サブクラスで onCastMouseOver イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onCastMouseOver プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onCastMouseOver
		*/
		protected function _onCastMouseOver()					:void {}
		
		/*======================================================================*//**
		* キャストオブジェクトが CastMouseEvent.CAST_MOUSE_OUT イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onCastMouseOut()					:Function { return __onCastMouseOut || _onCastMouseOut; }
		public function set onCastMouseOut( value				:Function ):void { __onCastMouseOut = value; }
		private var __onCastMouseOut							:Function;
		
		/*======================================================================*//**
		* サブクラスで onCastMouseOut イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onCastMouseOut プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onCastMouseOut
		*/
		protected function _onCastMouseOut()					:void {}
		
		/*======================================================================*//**
		* キャストオブジェクトが CastMouseEvent.CAST_MOUSE_WHEEL イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onCastMouseWheel()					:Function { return __onCastMouseWheel || _onCastMouseWheel; }
		public function set onCastMouseWheel( value				:Function ):void { __onCastMouseWheel = value; }
		private var __onCastMouseWheel							:Function;
		
		/*======================================================================*//**
		* サブクラスで onCastMouseWheel イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onCastMouseWheel プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onCastMouseWheel
		*/
		protected function _onCastMouseWheel()					:void {}
		
		/*======================================================================*//**
		* キャストオブジェクトが CastMouseEvent.CAST_ROLL_OVER イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onCastRollOver()					:Function { return __onCastRollOver || _onCastRollOver; }
		public function set onCastRollOver( value				:Function ):void { __onCastRollOver = value; }
		private var __onCastRollOver							:Function;
		
		/*======================================================================*//**
		* サブクラスで onCastRollOver イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onCastRollOver プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onCastRollOver
		*/
		protected function _onCastRollOver()					:void {}
		
		/*======================================================================*//**
		* キャストオブジェクトが CastMouseEvent.CAST_ROLL_OUT イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onCastRollOut()						:Function { return __onCastRollOut || _onCastRollOut; }
		public function set onCastRollOut( value				:Function ):void { __onCastRollOut = value; }
		private var __onCastRollOut								:Function;
		
		/*======================================================================*//**
		* サブクラスで onCastRollOut イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onCastRollOut プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onCastRollOut
		*/
		protected function _onCastRollOut()						:void {}
		
		/*======================================================================*//**
		* Progression インスタンスを取得します。 
		*/
		private var _progression								:Progression;
		
		/*======================================================================*//**
		* stage インスタンスを取得します。
		*/
		private var _stage										:Stage;
		
		/*======================================================================*//**
		* クラス名を取得します。
		*/
		private var _className									:String;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい CastButton インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function CastButton() {
			// 初期化する
			buttonEnabled = true;
			extra = {};
			_commandList = new SerialList();
			_toolTip = new ToolTip( this );
			_className = getQualifiedClassName( this ).split( "::" ).pop();
			
			// コンテクストメニューを初期化する
			_uiContextMenu = CastButtonContextMenu.progression_internal::createInstance( super.contextMenu = new ContextMenu() );
			
			// イベントリスナーを登録する
			addExclusivelyEventListener( CastEvent.CAST_ADDED, _castAdded, false, int.MAX_VALUE, true );
			addExclusivelyEventListener( CastEvent.CAST_REMOVED, _castRemoved, false, int.MAX_VALUE, true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function addCommand( ... commands:Array ):void {
			_commandList.addCommand.apply( null, commands );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function clearCommand():void {
			_commandList = parallelMode ? new ParallelList() : new SerialList();
		}
		
		/*======================================================================*//**
		* 指定されたシーン、または URL に移動します。
		* 引数が省略された場合には、予め CastButton インスタンスに指定されている sceneId プロパティまたは href プロパティが示す先に移動します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	location	説明文
		* @param	window		説明文
		*/
		public function navigateTo( location:* = null, window:String = null ):void {
			var request:URLRequest;
			var sceneId:SceneId;
			
			// location を型によって振り分ける
			switch ( true ) {
				case location is String		: { request = new URLRequest( location ); break; }
				case location is URLRequest	: { request = URLRequest( location ); break; }
				case location is SceneId	: { sceneId = SceneId( location ); break; }
				default						: {
					request = href ? new URLRequest( href ) : null;
					sceneId = this.sceneId ? this.sceneId.clone() : null;
				}
			}
			
			// window を振り分ける
			switch ( window ) {
				case null	: { window = _windowTarget || "_self"; break; }
			}
			
			// URL を移動する
			if ( request ) {
				navigateToURL( request, window );
				return;
			}
			
			// シーンを移動する
			if ( sceneId ) {
				// 自身を指していればそのまま移動する
				if ( window == "_self" ) {
					Progression.goto( sceneId );
					return;
				}
				
				// それ以外の場合には navigateToURL を使用する
				navigateToURL( new URLRequest( progression_internal::currentNavigateURL ), window );
			}
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object " + _className + " : " + ( id || name ) + "]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* キャストオブジェクトが AddChild コマンド経由でディスプレイリストに追加された場合に送出されます。
		*/
		private function _castAdded( e:CastEvent ):void {
			Progression.verbose.log( this, null, 1, this + " キャストの " + e.type + " イベントを実行" );
			
			// イベントハンドラメソッドを実行する
			onCastAdded();
			
			// コマンドリストを作成する
			var commandList:ParallelList = new ParallelList();
			
			// 子のイベントを送出する
			var l:int = casts.length;
			for ( var i:int = 0; i < l; i++ ) {
				var cast:ICastObject = casts[i];
				
				// 子キャストのコマンドリストを消去する
				cast.clearCommand();
				
				// ICastObject インターフェイスを実装していれば
				if ( cast is ICastObject ) {
					// 子キャストのコマンドリストを消去する
					cast.clearCommand();
				}
				else if ( "progression_internal_clearCommand" in cast ) {
					// 子キャストのコマンドリストを消去する
					MovieClip( cast ).progression_internal_clearCommand();
				}
				
				// イベントを送出する
				cast.dispatchEvent( e );
				
				var childCommandList:CommandList;
				switch ( true ) {
					case cast is CastBitmap							: { childCommandList = CastBitmap( cast ).progression_internal::commandList; break; }
					case cast is CastButton							: { childCommandList = CastButton( cast ).progression_internal::commandList; break; }
					case cast is CastLoader							: { childCommandList = CastLoader( cast ).progression_internal::commandList; break; }
					case cast is CastMovieClip						: { childCommandList = CastMovieClip( cast ).progression_internal::commandList; break; }
					case cast is CastSprite							: { childCommandList = CastSprite( cast ).progression_internal::commandList; break; }
					case cast is CastTextField						: { childCommandList = CastTextField( cast ).progression_internal::commandList; break; }
					case "progression_internal_getCommand" in cast	: { childCommandList = MovieClip( cast ).progression_internal_getCommand(); break;  }
				}
				
				// コマンドが登録されていれば追加する
				if ( childCommandList.length > 0 ) {
					commandList.addCommand( childCommandList );
				}
			}
			
			// コマンドが存在すれば
			if ( commandList.public::commands.length > 0 ) {
				// 子のコマンドを追加する
				addCommand( commandList );
			}
		}
		
		/*======================================================================*//**
		* キャストオブジェクトが RemoveChild コマンド経由でディスプレイリストから削除された場合に送出されます。
		*/
		private function _castRemoved( e:CastEvent ):void {
			Progression.verbose.log( this, null, 1, this + " キャストの " + e.type + " イベントを実行" );
			
			// イベントハンドラメソッドを実行する
			onCastRemoved();
			
			// コマンドリストを作成する
			var commandList:ParallelList = new ParallelList();
			
			// 子キャストのイベントを送出する
			var l:int = casts.length;
			for ( var i:int = 0; i < l; i++ ) {
				var cast:ICastObject = casts[i];
				
				// ICastObject インターフェイスを実装していれば
				if ( cast is ICastObject ) {
					// 子キャストのコマンドリストを消去する
					cast.clearCommand();
				}
				else if ( "progression_internal_clearCommand" in cast ) {
					// 子キャストのコマンドリストを消去する
					MovieClip( cast ).progression_internal_clearCommand();
				}
				
				// イベントを送出する
				cast.dispatchEvent( e );
				
				var childCommandList:CommandList;
				switch ( true ) {
					case cast is CastBitmap							: { childCommandList = CastBitmap( cast ).progression_internal::commandList; break; }
					case cast is CastButton							: { childCommandList = CastButton( cast ).progression_internal::commandList; break; }
					case cast is CastLoader							: { childCommandList = CastLoader( cast ).progression_internal::commandList; break; }
					case cast is CastMovieClip						: { childCommandList = CastMovieClip( cast ).progression_internal::commandList; break; }
					case cast is CastSprite							: { childCommandList = CastSprite( cast ).progression_internal::commandList; break; }
					case cast is CastTextField						: { childCommandList = CastTextField( cast ).progression_internal::commandList; break; }
					case "progression_internal_getCommand" in cast	: { childCommandList = MovieClip( cast ).progression_internal_getCommand(); break;  }
				}
				
				// コマンドが登録されていれば追加する
				if ( childCommandList.length > 0 ) {
					commandList.addCommand( childCommandList );
				}
			}
			
			// コマンドが存在すれば
			if ( commandList.public::commands.length > 0 ) {
				// 子のコマンドを追加する
				addCommand( commandList );
			}
		}
		
		/*======================================================================*//**
		* シーン移動処理中に対象シーンが変更された場合に送出されます。
		*/
		private function _processScene( e:ProcessEvent ):void {
			// カレントシーンを取得する
			var current:SceneObject = _progression.current;
			
			// カレントシーンが存在しなければ終了する
			if ( !current ) { return; }
			
			// 現在の設定を保存する
			var isCurrent:Boolean = _isCurrent;
			var isParent:Boolean = _isParent;
			var isChild:Boolean = _isChild;
			
			// カレントかどうかを判別する
			_isCurrent = ( sceneId.equals( current.sceneId ) );
			
			// 親シーンかどうかを判別する
			_isParent = ( !_isCurrent && current.sceneId.contains( sceneId ) );
			
			// 子シーンかどうかを判別する
			_isChild = ( !_isCurrent && sceneId.contains( current.sceneId ) );
			
			Progression.verbose.log( this, null, 1, this + " キャストの " + CastEvent.CAST_STATE_CHANGE + " イベントを実行" );
			
			// コマンドリストを消去する
			clearCommand();
			
			// イベントを送出する
			dispatchEvent( new CastEvent( CastEvent.CAST_STATE_CHANGE ) );
			
			// イベントハンドラメソッドを実行する
			onCastStateChange();
			
			// コマンドを実行する
			var commandList:CommandList = CommandList( progression_internal::commandList );
			if ( commandList && commandList.length > 0 ) {
				commandList.execute( {} );
			}
		}
		
		/*======================================================================*//**
		* 表示オブジェクトが、直接、または表示オブジェクトを含むサブツリーの追加により、ステージ上の表示リストに追加されたときに送出されます。
		*/
		private function _addedToStage( e:Event ):void {
			// プロパティを更新する
			sceneId = sceneId;
			_stage = stage;
			
			// イベントリスナーを登録する
			_stage.addEventListener( KeyboardEvent.KEY_UP, _keyUp, false, int.MAX_VALUE, true );
		}
		
		/*======================================================================*//**
		* 表示オブジェクトが、直接、または表示オブジェクトを含むサブツリーの削除により、表示リストから削除されようとしているときに送出されます。
		*/
		private function _removedFromStage( e:Event ):void {
			// イベントリスナーを解除する
			_stage.removeEventListener( KeyboardEvent.KEY_UP, _keyUp );
			
			// プロパティを更新する
			_stage = null;
		}
		
		/*======================================================================*//**
		* @private
		* ユーザーが InteractiveObject をポインティングデバイスのメインボタンでクリックしたときに送出されます。
		*/
		progression_internal function click( e:MouseEvent ):void {
			Progression.verbose.log( this, null, 1, this + " キャストの " + CastMouseEvent.CAST_CLICK + " イベントを実行" );
			
			// コマンドリストを消去する
			clearCommand();
			
			// イベントを送出する
			dispatchEvent( new CastMouseEvent( CastMouseEvent.CAST_CLICK, e.bubbles, e.cancelable, e.localX, e.localY, e.relatedObject, e.ctrlKey, e.altKey, e.shiftKey, e.buttonDown, e.delta ) );
			
			// イベントハンドラメソッドを実行する
			onCastClick();
			
			// コマンドを実行する
			var commandList:CommandList = CommandList( progression_internal::commandList );
			if ( commandList && commandList.length > 0 ) {
				commandList.execute( {} );
			}
			
			// 移動する
			navigateTo();
		}
		
		/*======================================================================*//**
		* @private
		* インスタンスの doubleClickEnabled フラグが true に設定されている場合にユーザーが InteractiveObject をポインティングデバイスのメインボタンで 2 回続けてすばやくクリックすると送出されます。
		*/
		progression_internal function doubleClick( e:MouseEvent ):void {
			Progression.verbose.log( this, null, 1, this + " キャストの " + CastMouseEvent.CAST_DOUBLE_CLICK + " イベントを実行" );
			
			// コマンドリストを消去する
			clearCommand();
			
			// イベントを送出する
			dispatchEvent( new CastMouseEvent( CastMouseEvent.CAST_DOUBLE_CLICK, e.bubbles, e.cancelable, e.localX, e.localY, e.relatedObject, e.ctrlKey, e.altKey, e.shiftKey, e.buttonDown, e.delta ) );
			
			// イベントハンドラメソッドを実行する
			onCastDoubleClick();
			
			// コマンドを実行する
			var commandList:CommandList = CommandList( progression_internal::commandList );
			if ( commandList && commandList.length > 0 ) {
				commandList.execute( {} );
			}
		}
		
		/*======================================================================*//**
		* @private
		* Flash Player ウィンドウの InteractiveObject インスタンスの上でユーザーがポインティングデバイスのボタンを押すと送出されます。
		*/
		progression_internal function mouseDown( e:MouseEvent ):void {
			Progression.verbose.log( this, null, 1, this + " キャストの " + CastMouseEvent.CAST_MOUSE_DOWN + " イベントを実行" );
			
			// コマンドリストを消去する
			clearCommand();
			
			// イベントを送出する
			dispatchEvent( new CastMouseEvent( CastMouseEvent.CAST_MOUSE_DOWN, e.bubbles, e.cancelable, e.localX, e.localY, e.relatedObject, e.ctrlKey, e.altKey, e.shiftKey, e.buttonDown, e.delta ) );
			
			// イベントハンドラメソッドを実行する
			onCastMouseDown();
			
			// コマンドを実行する
			var commandList:CommandList = CommandList( progression_internal::commandList );
			if ( commandList && commandList.length > 0 ) {
				commandList.execute( {} );
			}
		}
		
		/*======================================================================*//**
		* @private
		* ユーザーが Flash Player ウィンドウの InteractiveObject インスタンスの上でポインティングデバイスのボタンを解放したときに送出されます。
		*/
		progression_internal function mouseUp( e:MouseEvent ):void {
			Progression.verbose.log( this, null, 1, this + " キャストの " + CastMouseEvent.CAST_MOUSE_UP + " イベントを実行" );
			
			// コマンドリストを消去する
			clearCommand();
			
			// イベントを送出する
			dispatchEvent( new CastMouseEvent( CastMouseEvent.CAST_MOUSE_UP, e.bubbles, e.cancelable, e.localX, e.localY, e.relatedObject, e.ctrlKey, e.altKey, e.shiftKey, e.buttonDown, e.delta ) );
			
			// イベントハンドラメソッドを実行する
			onCastMouseUp();
			
			// コマンドを実行する
			var commandList:CommandList = CommandList( progression_internal::commandList );
			if ( commandList && commandList.length > 0 ) {
				commandList.execute( {} );
			}
		}
		
		/*======================================================================*//**
		* @private
		* ユーザーが Flash Player ウィンドウの InteractiveObject インスタンスにポインティングデバイスを合わせたときに送出されます。
		*/
		progression_internal function mouseOver( e:MouseEvent ):void {
			Progression.verbose.log( this, null, 1, this + " キャストの " + CastMouseEvent.CAST_MOUSE_OVER + " イベントを実行" );
			
			// コマンドリストを消去する
			clearCommand();
			
			// イベントを送出する
			dispatchEvent( new CastMouseEvent( CastMouseEvent.CAST_MOUSE_OVER, e.bubbles, e.cancelable, e.localX, e.localY, e.relatedObject, e.ctrlKey, e.altKey, e.shiftKey, e.buttonDown, e.delta ) );
			
			// イベントハンドラメソッドを実行する
			onCastMouseOver();
			
			// コマンドを実行する
			var commandList:CommandList = CommandList( progression_internal::commandList );
			if ( commandList && commandList.length > 0 ) {
				commandList.execute( {} );
			}
		}
		
		/*======================================================================*//**
		* @private
		* ユーザーが InteractiveObject インスタンスからポインティングデバイスを離したときに送出されます。
		*/
		progression_internal function mouseOut( e:MouseEvent ):void {
			Progression.verbose.log( this, null, 1, this + " キャストの " + CastMouseEvent.CAST_MOUSE_OUT + " イベントを実行" );
			
			// コマンドリストを消去する
			clearCommand();
			
			// イベントを送出する
			dispatchEvent( new CastMouseEvent( CastMouseEvent.CAST_MOUSE_OUT, e.bubbles, e.cancelable, e.localX, e.localY, e.relatedObject, e.ctrlKey, e.altKey, e.shiftKey, e.buttonDown, e.delta ) );
			
			// イベントハンドラメソッドを実行する
			onCastMouseOut();
			
			// コマンドを実行する
			var commandList:CommandList = CommandList( progression_internal::commandList );
			if ( commandList && commandList.length > 0 ) {
				commandList.execute( {} );
			}
		}
		
		/*======================================================================*//**
		* @private
		* Flash Player ウィンドウの InteractiveObject インスタンスの上でマウスホイールを回転させたときに送出されます。
		*/
		progression_internal function mouseWheel( e:MouseEvent ):void {
			Progression.verbose.log( this, null, 1, this + " キャストの " + CastMouseEvent.CAST_MOUSE_WHEEL + " イベントを実行" );
			
			// コマンドリストを消去する
			clearCommand();
			
			// イベントを送出する
			dispatchEvent( new CastMouseEvent( CastMouseEvent.CAST_MOUSE_WHEEL, e.bubbles, e.cancelable, e.localX, e.localY, e.relatedObject, e.ctrlKey, e.altKey, e.shiftKey, e.buttonDown, e.delta ) );
			
			// イベントハンドラメソッドを実行する
			onCastMouseWheel();
			
			// コマンドを実行する
			var commandList:CommandList = CommandList( progression_internal::commandList );
			if ( commandList && commandList.length > 0 ) {
				commandList.execute( {} );
			}
		}
		
		/*======================================================================*//**
		* @private
		* ユーザーが InteractiveObject インスタンスにポインティングデバイスを合わせたときに送出されます。
		*/
		progression_internal function rollOver( e:MouseEvent ):void {
			Progression.verbose.log( this, null, 1, this + " キャストの " + CastMouseEvent.CAST_ROLL_OVER + " イベントを実行" );
			
			// ロールオーバー状態に設定する
			_isRollOver = true;
			
			// ブラウザのステータスバーに移動先の URL を表示する
			Browser.status = progression_internal::currentNavigateURL;
			
			// コマンドリストを消去する
			clearCommand();
			
			// イベントを送出する
			dispatchEvent( new CastMouseEvent( CastMouseEvent.CAST_ROLL_OVER, e.bubbles, e.cancelable, e.localX, e.localY, e.relatedObject, e.ctrlKey, e.altKey, e.shiftKey, e.buttonDown, e.delta ) );
			
			// イベントハンドラメソッドを実行する
			onCastRollOver();
			
			// コマンドを実行する
			var commandList:CommandList = CommandList( progression_internal::commandList );
			if ( commandList && commandList.length > 0 ) {
				commandList.execute( {} );
			}
		}
		
		/*======================================================================*//**
		* @private
		* ユーザーが InteractiveObject インスタンスからポインティングデバイスを離したときに送出されます。
		*/
		progression_internal function rollOut( e:MouseEvent ):void {
			Progression.verbose.log( this, null, 1, this + " キャストの " + CastMouseEvent.CAST_ROLL_OUT + " イベントを実行" );
			
			// 非ロールオーバー状態に設定する
			_isRollOver = false;
			
			// ブラウザのステータスバーの表示を削除する
			Browser.status = "";
			
			// コマンドリストを消去する
			clearCommand();
			
			// イベントを送出する
			dispatchEvent( new CastMouseEvent( CastMouseEvent.CAST_ROLL_OUT, e.bubbles, e.cancelable, e.localX, e.localY, e.relatedObject, e.ctrlKey, e.altKey, e.shiftKey, e.buttonDown, e.delta ) );
			
			// イベントハンドラメソッドを実行する
			onCastRollOut();
			
			// コマンドを実行する
			var commandList:CommandList = CommandList( progression_internal::commandList );
			if ( commandList && commandList.length > 0 ) {
				commandList.execute( {} );
			}
		}
		
		/*======================================================================*//**
		* ユーザーがキーを離したときに送出されます。
		*/
		private function _keyUp( e:KeyboardEvent ):void {
			if ( e.charCode != accessKey.toLowerCase().charCodeAt() ) { return; }
			
			// シーンを移動する
			Progression.goto( sceneId );
		}
	}
}





