﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.casts {
	import flash.display.MovieClip;
	import flash.errors.IllegalOperationError;
	import flash.ui.ContextMenu;
	import flash.utils.getQualifiedClassName;
	import jp.nium.display.ExMovieClip;
	import jp.progression.casts.ICastObject;
	import jp.progression.commands.ParallelList;
	import jp.progression.commands.SerialList;
	import jp.progression.core.commands.Command;
	import jp.progression.core.commands.CommandList;
	import jp.progression.core.display.ToolTip;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.core.ui.CastObjectContextMenu;
	import jp.progression.events.CastEvent;
	import jp.progression.Progression;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* キャストオブジェクトが AddChild コマンド経由でディスプレイリストに追加された場合に送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CastEvent.CAST_ADDED
	*/
	[Event( name="castAdded", type="jp.progression.events.CastEvent" )]
	
	/*======================================================================*//**
	* キャストオブジェクトが RemoveChild コマンド経由でディスプレイリストから削除された場合に送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CastEvent.CAST_REMOVED
	*/
	[Event( name="castRemoved", type="jp.progression.events.CastEvent" )]
	
	/*======================================================================*//**
	* キャストオブジェクトの状態が変更された場合に送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CastEvent.CAST_STATE_CHANGE
	*/
	[Event( name="castStageChange", type="jp.progression.events.CastEvent" )]
	
	/*======================================================================*//**
	* CastMovieClip クラスは、Progression Framework 上でシーン機能およびコマンド機能と連動させる機能を実装した基本キャストクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.nium.graphics.Square;
	* 
	* import jp.progression.Progression;
	* import jp.progression.casts.CastMovieClip;
	* import jp.progression.commands.AddChild;
	* import jp.progression.commands.RemoveChild;
	* import jp.progression.scenes.SceneObject;
	* 
	* // CastMovieClip を作成する
	* var cast:CastMovieClip = new CastMovieClip();
	* cast.addChild( new Square() );
	* cast.onCastAdded = function():void { 
	* 	cast.addCommand(
	* 		new DoTween( this, {
	* 			x		:100,
	* 	 		y		:100
	* 		}, Elastic.easeInOut, 500 )
	* 	);
	* }
	* cast.onCastRemoved = function():void {
	* 	cast.addCommand(
	* 		new DoTween( this, {
	* 			x		:0,
	* 			y		:0
	* 		}, Elastic.easeInOut, 500 )
	* 	);
	* }
	* 
	* // Progression インスタンスを作成する
	* var prog:Progression = new Progression( "index", stage );
	* 
	* // page1 シーンを作成する
	* var page1:SceneObject = prog.root.addChild( new SceneObject( "page1" ) );
	* page1.onInit = function():void {
	* 	page1.addCommand( new AddChild( this, cast ) );
	* }
	* page1.onGoto = function():void {
	* 	page1.addCommand( new RemoveChild( this, cast );
	* }
	* 
	* // シーンを移動する
	* prog.goto( page1.sceneId );
	* </listing>
	*/
	public class CastMovieClip extends ExMovieClip implements ICastObject {
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get casts()							:Array { return getChildrenByClass( ICastObject ); }
		
		/*======================================================================*//**
		* @private
		*/
		public override function get contextMenu()			:ContextMenu { throw new IllegalOperationError( "CastMovieClip クラスの contextMenu プロパティは、より詳細な設定が可能な uiContextMenu プロパティに変更されました。\n同様の操作をする場合、今後は uiContextMenu プロパティを使用してください。" ); }
		public override function set contextMenu( value		:ContextMenu ):void { throw new IllegalOperationError( "CastMovieClip クラスの contextMenu プロパティは、より詳細な設定が可能な uiContextMenu プロパティに変更されました。\n同様の操作をする場合、今後は uiContextMenu プロパティを使用してください。" ); }
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function get contextMenu()		:ContextMenu { return super.contextMenu; }
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get uiContextMenu()					:CastObjectContextMenu { return _uiContextMenu; }
		private var _uiContextMenu							:CastObjectContextMenu;
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get parallelMode()					:Boolean { return _commandList is ParallelList; }
		public function set parallelMode( value				:Boolean ):void { _commandList = value ? new ParallelList() : new SerialList(); }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get toolTip()						:ToolTip { return _toolTip; }
		private var _toolTip								:ToolTip;
		
		/*======================================================================*//**
		* @private
		*/
		public function get toolChip()						:* { throw new IllegalOperationError( "toolChip プロパティは、スペルが間違っていたので toolTip プロパティに変更されました。\n同様の操作をする場合、今後は toolTip プロパティを使用してください。\nごめんなさいごめんなさい！" ); }
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get extra()							:Object { return _extra; }
		public function set extra( value					:Object ):void { _extra = value; }
		private var _extra									:Object;
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function get commandList()		:CommandList { return _commandList; }
		private var _commandList							:CommandList;
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onCastAdded()					:Function { return __onCastAdded || _onCastAdded; }
		public function set onCastAdded( value				:Function ):void { __onCastAdded = value; }
		private var __onCastAdded							:Function;
		
		/*======================================================================*//**
		* @copy jp.progression.casts.CastMovieClip#_onCastAdded()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onCastAdded
		*/
		protected function _onCastAdded()					:void {}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onCastRemoved()					:Function { return __onCastRemoved || _onCastRemoved; }
		public function set onCastRemoved( value			:Function ):void { __onCastRemoved = value; }
		private var __onCastRemoved							:Function;
		
		/*======================================================================*//**
		* @copy jp.progression.casts.CastMovieClip#_onCastRemoved()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onCastRemoved
		*/
		protected function _onCastRemoved()					:void {}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onCastStateChange()				:Function { return __onCastStateChange || _onCastStateChange; }
		public function set onCastStateChange( value		:Function ):void { __onCastStateChange = value; }
		private var __onCastStateChange						:Function;
		
		/*======================================================================*//**
		* @copy jp.progression.casts.CastMovieClip#_onCastStateChange()
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onCastStateChange
		*/
		protected function _onCastStateChange()				:void {}
		
		/*======================================================================*//**
		* クラス名を取得します。
		*/
		private var _className								:String;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい CastMovieClip インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0 
		*/
		public function CastMovieClip() {
			// 初期化する
			extra = {};
			_commandList = new SerialList();
			_toolTip = new ToolTip( this );
			_className = getQualifiedClassName( this ).split( "::" ).pop();
			
			// コンテクストメニューを初期化する
			_uiContextMenu = CastObjectContextMenu.progression_internal::createInstance( super.contextMenu = new ContextMenu() );
			
			// イベントリスナーを登録する
			addExclusivelyEventListener( CastEvent.CAST_ADDED, _castAdded, false, 0, true );
			addExclusivelyEventListener( CastEvent.CAST_REMOVED, _castRemoved, false, 0, true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function addCommand( ... commands:Array ):void {
			_commandList.addCommand.apply( null, commands );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function clearCommand():void {
			_commandList = parallelMode ? new ParallelList() : new SerialList();
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object " + _className + " : " + ( id || name ) + "]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* キャストオブジェクトが AddChild コマンド経由でディスプレイリストに追加された場合に送出されます。
		*/
		private function _castAdded( e:CastEvent ):void {
			Progression.verbose.log( this, null, 1, this + " キャストの " + e.type + " イベントを実行" );
			
			// イベントハンドラメソッドを実行する
			onCastAdded();
			
			// コマンドリストを作成する
			var commandList:ParallelList = new ParallelList();
			
			// 子のイベントを送出する
			var l:int = casts.length;
			for ( var i:int = 0; i < l; i++ ) {
				var cast:ICastObject = casts[i];
				
				// 子キャストのコマンドリストを消去する
				cast.clearCommand();
				
				// ICastObject インターフェイスを実装していれば
				if ( cast is ICastObject ) {
					// 子キャストのコマンドリストを消去する
					cast.clearCommand();
				}
				else if ( "progression_internal_clearCommand" in cast ) {
					// 子キャストのコマンドリストを消去する
					MovieClip( cast ).progression_internal_clearCommand();
				}
				
				// イベントを送出する
				cast.dispatchEvent( e );
				
				var childCommandList:CommandList;
				switch ( true ) {
					case cast is CastBitmap							: { childCommandList = CastBitmap( cast ).progression_internal::commandList; break; }
					case cast is CastButton							: { childCommandList = CastButton( cast ).progression_internal::commandList; break; }
					case cast is CastLoader							: { childCommandList = CastLoader( cast ).progression_internal::commandList; break; }
					case cast is CastMovieClip						: { childCommandList = CastMovieClip( cast ).progression_internal::commandList; break; }
					case cast is CastSprite							: { childCommandList = CastSprite( cast ).progression_internal::commandList; break; }
					case cast is CastTextField						: { childCommandList = CastTextField( cast ).progression_internal::commandList; break; }
					case "progression_internal_getCommand" in cast	: { childCommandList = MovieClip( cast ).progression_internal_getCommand(); break;  }
				}
				
				// コマンドが登録されていれば追加する
				if ( childCommandList.length > 0 ) {
					commandList.addCommand( childCommandList );
				}
			}
			
			// コマンドが存在すれば
			if ( commandList.public::commands.length > 0 ) {
				// 子のコマンドを追加する
				addCommand( commandList );
			}
		}
		
		/*======================================================================*//**
		* キャストオブジェクトが RemoveChild コマンド経由でディスプレイリストから削除された場合に送出されます。
		*/
		private function _castRemoved( e:CastEvent ):void {
			Progression.verbose.log( this, null, 1, this + " キャストの " + e.type + " イベントを実行" );
			
			// イベントハンドラメソッドを実行する
			onCastRemoved();
			
			// コマンドリストを作成する
			var commandList:ParallelList = new ParallelList();
			
			// 子キャストのイベントを送出する
			var l:int = casts.length;
			for ( var i:int = 0; i < l; i++ ) {
				var cast:ICastObject = casts[i];
				
				// ICastObject インターフェイスを実装していれば
				if ( cast is ICastObject ) {
					// 子キャストのコマンドリストを消去する
					cast.clearCommand();
				}
				else if ( "progression_internal_clearCommand" in cast ) {
					// 子キャストのコマンドリストを消去する
					MovieClip( cast ).progression_internal_clearCommand();
				}
				
				// イベントを送出する
				cast.dispatchEvent( e );
				
				var childCommandList:CommandList;
				switch ( true ) {
					case cast is CastBitmap							: { childCommandList = CastBitmap( cast ).progression_internal::commandList; break; }
					case cast is CastButton							: { childCommandList = CastButton( cast ).progression_internal::commandList; break; }
					case cast is CastLoader							: { childCommandList = CastLoader( cast ).progression_internal::commandList; break; }
					case cast is CastMovieClip						: { childCommandList = CastMovieClip( cast ).progression_internal::commandList; break; }
					case cast is CastSprite							: { childCommandList = CastSprite( cast ).progression_internal::commandList; break; }
					case cast is CastTextField						: { childCommandList = CastTextField( cast ).progression_internal::commandList; break; }
					case "progression_internal_getCommand" in cast	: { childCommandList = MovieClip( cast ).progression_internal_getCommand(); break;  }
				}
				
				// コマンドが登録されていれば追加する
				if ( childCommandList.length > 0 ) {
					commandList.addCommand( childCommandList );
				}
			}
			
			// コマンドが存在すれば
			if ( commandList.public::commands.length > 0 ) {
				// 子のコマンドを追加する
				addCommand( commandList );
			}
		}
	}
}





