﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.casts.presets {
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.events.HTTPStatusEvent;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	import flash.net.URLRequest;
	import flash.system.LoaderContext;
	import jp.nium.display.ExImageLoader;
	import jp.progression.casts.CastMovieClip;
	import jp.progression.events.CastEvent;
	
	/*======================================================================*//**
	* 
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class ImageBox extends CastMovieClip {
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get source() 					:String { return _source; }
		public function set source( value				:String ):void {
			_source = value;
			
			if ( _source ) {
				_loader.load( new URLRequest( _source ) );
			}
		}
		private var _source								:String = "";
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get loadStateFrames()			:Array { return _loadStateFrames; }
		public function set loadStateFrames( value		:Array ):void { _loadStateFrames = value; }
		private var _loadStateFrames					:Array;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get errorStateFrame()			:* { return _errorStateFrame; }
		public function set errorStateFrame( value		:* ):void { _errorStateFrame = value; }
		private var _errorStateFrame					:*;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get ratio()						:Boolean { return _loader.ratio; }
		public function set ratio( value				:Boolean ):void { _loader.ratio = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get pixelSnapping()				:String { return _loader.pixelSnapping; }
		public function set pixelSnapping( value		:String ):void { _loader.pixelSnapping = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get smoothing()					:Boolean { return _loader.smoothing; }
		public function set smoothing( value			:Boolean ):void { _loader.smoothing = value; }
		
		/*======================================================================*//**
		* @private
		*/
		protected var _loader							:ExImageLoader;
		
		/*======================================================================*//**
		* @private
		*/
		protected var _target							:MovieClip;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい ImageBox インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function ImageBox() {
			// 初期化する
			_loadStateFrames = [];
			_target = this;
			
			// ExImageLoader を作成する
			_loader = ExImageLoader( addChildAt( new ExImageLoader(), 0 ) );
            _loader.addEventListener( Event.OPEN, _open, false, int.MAX_VALUE, true );
			_loader.addEventListener( Event.COMPLETE, _complete, false, int.MAX_VALUE, true );
            _loader.addEventListener( ProgressEvent.PROGRESS, _progress, false, int.MAX_VALUE, true );
            _loader.addEventListener( HTTPStatusEvent.HTTP_STATUS, _httpStatus, false, int.MAX_VALUE, true );
            _loader.addEventListener( IOErrorEvent.IO_ERROR, _ioError, false, int.MAX_VALUE, true );
			
			// 再生を停止する
			stop();
			
			// イベントリスナーを登録する
			addExclusivelyEventListener( CastEvent.CAST_ADDED, _castAdded, false, int.MAX_VALUE, true );
			addExclusivelyEventListener( CastEvent.CAST_REMOVED, _castRemoved, false, int.MAX_VALUE, true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function load( request:URLRequest, context:LoaderContext = null ):void {
			_loader.load( request, context );
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function unload():void {
			_loader.unload();
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object ImageBox : " + ( id || name ) + "]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* キャストオブジェクトが AddChild コマンド経由でディスプレイリストに追加された場合に送出されます。
		*/
		private function _castAdded( e:CastEvent ):void {
		}
		
		/*======================================================================*//**
		* キャストオブジェクトが RemoveChild コマンド経由でディスプレイリストから削除された場合に送出されます。
		*/
		private function _castRemoved( e:CastEvent ):void {
		}
		
		/*======================================================================*//**
		* ロード操作が開始したときに送出されます。
		*/
		private function _open( e:Event ):void {
		}
		
		/*======================================================================*//**
		* ダウンロード処理を実行中にデータを受信したときに送出されます。
		*/
		private function _progress( e:ProgressEvent ):void {
			var distance:int = 0;
			var percent:Number = e.bytesLoaded / e.bytesTotal;
			var position:int = Math.floor( distance * percent );
			
			// 読み込みアニメーション位置に移動する
			gotoAndStop( position );
		}
		
		/*======================================================================*//**
		* データが正常にロードされたときに送出されます。
		*/
		private function _complete( e:Event ):void {
		}
		
		/*======================================================================*//**
		* ネットワーク要求が HTTP を介して行われ、Flash Player が HTTP 状況コードを検出できる場合に送出されます。
		*/
		private function _httpStatus( e:HTTPStatusEvent ):void {
		}
		
		/*======================================================================*//**
		* 入出力エラーが発生してロード処理が失敗したときに送出されます。
		*/
		private function _ioError( e:IOErrorEvent ):void {
		}
	}
}





