﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.casts.presets {
	import flash.display.MovieClip;
	import jp.progression.casts.CastMovieClip;
	import jp.progression.commands.DoTweenFrame;
	import jp.progression.events.CastEvent;
	
	/*======================================================================*//**
	* InOutMovie クラスは、コマンドクラスを経由してのディスプレイリスト操作処理に応じて、タイムラインアニメーションを再生させるプリセットクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class InOutMovie extends CastMovieClip {
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get inStateFrames()			:Array { return _inStateFrames; }
		public function set inStateFrames( value	:Array ):void { _inStateFrames = value; }
		private var _inStateFrames					:Array;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get outStateFrames()		:Array { return _outStateFrames; }
		public function set outStateFrames( value	:Array ):void { _outStateFrames = value; }
		private var _outStateFrames					:Array;
		
		/*======================================================================*//**
		* @private
		*/
		protected var _target						:MovieClip;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい InOutMovie インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function InOutMovie() {
			// 初期化する
			_inStateFrames = [];
			_outStateFrames = [];
			_target = this;
			
			// 再生を停止する
			stop();
			
			// イベントリスナーを登録する
			addExclusivelyEventListener( CastEvent.CAST_ADDED, _castAdded, false, int.MAX_VALUE, true );
			addExclusivelyEventListener( CastEvent.CAST_REMOVED, _castRemoved, false, int.MAX_VALUE, true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object InOutMovie : " + ( id || name ) + "]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* キャストオブジェクトが AddChild コマンド経由でディスプレイリストに追加された場合に送出されます。
		*/
		private function _castAdded( e:CastEvent ):void {
			// 停止する
			_target.gotoAndStop( inStateFrames[0] );
			
			// アニメーションを追加する
			var l:int = inStateFrames.length;
			for ( var i:int = 0; i < l; i++ ) {
				// 開始点、終了点を取得する
				var ps:* = inStateFrames[i];
				var pe:* = inStateFrames[i + 1];
				
				// pe が存在しなければ終了する
				if ( !pe ) { break; }
				
				// コマンドを追加する
				addCommand( new DoTweenFrame( _target, ps, pe ) );
			}
		}
		
		/*======================================================================*//**
		* キャストオブジェクトが RemoveChild コマンド経由でディスプレイリストから削除された場合に送出されます。
		*/
		private function _castRemoved( e:CastEvent ):void {
			// 停止する
			_target.gotoAndStop( outStateFrames[0] );
			
			// アニメーションを追加する
			var l:int = outStateFrames.length;
			for ( var i:int = 0; i < l; i++ ) {
				// 開始点、終了点を取得する
				var ps:* = outStateFrames[i];
				var pe:* = outStateFrames[i + 1];
				
				// pe が存在しなければ終了する
				if ( !pe ) { break; }
				
				// コマンドを追加する
				addCommand( new DoTweenFrame( _target, ps, pe ) );
			}
		}
	}
}





