﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.casts.presets {
	import jp.progression.casts.presets.RollOverButton;
	import jp.progression.events.ProcessEvent;
	import jp.progression.Progression;
	
	/*======================================================================*//**
	* NextButton クラスは、カレントシーンの次シーンへ移動処理を実行するプリセットクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class NextButton extends RollOverButton {
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get progressionId() 				:String { return _progressionId; }
		public function set progressionId( value			:String ):void {
			_progressionId = value;
			
			// 値を設定する
			navigatePath = "/" + _progressionId;
			navigateType = RollOverButtonNavigateType.SCENE;
		}
		private var _progressionId							:String;
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function get navigatePath() 				:String { return super.navigatePath; }
		public override function set navigatePath( value			:String ):void {
			// 設定が存在していれば
			if ( _progression ) {
				_progression.completelyRemoveEventListener( ProcessEvent.PROCESS_SCENE, _processScene );
			}
			
			// 値を設定する
			super.navigatePath = value;
			_progression = Progression.getInstanceBySceneId( sceneId );
			
			// 設定が存在していれば
			if ( _progression ) {
				_progression.addExclusivelyEventListener( ProcessEvent.PROCESS_SCENE, _processScene, false, int.MAX_VALUE, true );
				_processScene( new ProcessEvent( ProcessEvent.PROCESS_SCENE, false, false, _progression.current, _progression.eventType ) );
			}
		}
		
		/*======================================================================*//**
		* Progression インスタンスを取得します。 
		*/
		private var _progression							:Progression;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい NextButton インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function NextButton() {
		}
		
		
		
		
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object NextButton : " + ( id || name ) + "]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* シーン移動処理中に対象シーンが変更された場合に送出されます。
		*/
		private function _processScene( e:ProcessEvent ):void {
			// シーンが存在しなければ終了する
			if ( !e.scene ) { return; }
			if ( !e.scene.next ) { return; }
			
			// カレントシーンの次シーンへの移動先を指定します。
			sceneId = e.scene.next.sceneId.clone();
		}
	}
}





