﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.casts.presets {
	import caurina.transitions.Tweener;
	import flash.display.MovieClip;
	import flash.errors.IllegalOperationError;
	import flash.events.Event;
	import jp.nium.utils.MovieClipUtil;
	import jp.progression.casts.CastButton;
	import jp.progression.casts.presets.RollOverButtonNavigateType;
	import jp.progression.commands.DoTweenFrame;
	import jp.progression.events.CastEvent;
	import jp.progression.events.CastMouseEvent;
	import jp.progression.Progression;
	import jp.progression.scenes.SceneId;
	
	/*======================================================================*//**
	* RollOverButton クラスは、ポインティングデバイスの状態に応じたタイムラインアニメーションを再生させるプリセットクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class RollOverButton extends CastButton {
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get navigatePath() 				:String { return _navigatePath; }
		public function set navigatePath( value			:String ):void {
			// イベントリスナーを解除する
			completelyRemoveEventListener( Event.ENTER_FRAME, _enterFrame );
			
			// null の場合は終了する
			if ( !value ) { return; }
			
			// 値を設定する
			_navigatePath = value;
			
			// タイプによって振り分ける
			switch ( _navigateType ) {
				case RollOverButtonNavigateType.HTTP	: {
					href = value;
					sceneId = null;
					break;
				}
				case RollOverButtonNavigateType.MAILTO	: {
					href = "mailto:" + value;
					sceneId = null;
					break;
				}
				case RollOverButtonNavigateType.SCENE	: {
					if ( !SceneId.validate( value ) ) { new IllegalOperationError( "navigatePath に設定されているシーンパスの書式が正しくありません。" ); }
					
					href = null;
					sceneId = new SceneId( value );
					
					// イベントリスナーを登録する
					addExclusivelyEventListener( Event.ENTER_FRAME, _enterFrame, false, int.MAX_VALUE, true );
					break;
				}
			}
		}
		private var _navigatePath						:String;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get navigateType() 				:String { return _navigateType; }
		public function set navigateType( value			:String ):void {
			// タイプによって振り分ける
			switch ( value ) {
				case RollOverButtonNavigateType.HTTP	:
				case RollOverButtonNavigateType.MAILTO	:
				case RollOverButtonNavigateType.SCENE	: { _navigateType = value; break; }
				default									: { new IllegalOperationError( "navigateType に設定されているシーンタイプが正しくありません。" ); }
			}
			
			// navigatePath を再設定する
			navigatePath = navigatePath;
		}
		private var _navigateType						:String = RollOverButtonNavigateType.SCENE;
		
		/*======================================================================*//**
		* @private
		*/
		public function get scenePath() 				:String { throw new IllegalOperationError( "RollOverButton クラスの scenePath プロパティは使用方法の変更に伴って削除されました。\n同様の操作をする場合、今後は navigatePath プロパティを使用してください。" ); }
		public function set scenePath( value			:String ):void { throw new IllegalOperationError( "RollOverButton クラスの scenePath プロパティは使用方法の変更に伴って削除されました。\n同様の操作をする場合、今後は navigatePath プロパティを使用してください。" ); }
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="対象ウィンドウ", defaultValue="_self" )]
		public override function get windowTarget() 			:String { return super.windowTarget; }
		public override function set windowTarget( value		:String ):void { super.windowTarget = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get upStateFrames()				:Array { return _upStateFrames; }
		public function set upStateFrames( value		:Array ):void { _upStateFrames = value; }
		private var _upStateFrames						:Array;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get overStateFrames()			:Array { return _overStateFrames; }
		public function set overStateFrames( value		:Array ):void { _overStateFrames = value; }
		private var _overStateFrames					:Array;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get downStateFrames()			:Array { return _downStateFrames; }
		public function set downStateFrames( value		:Array ):void { _downStateFrames = value; }
		private var _downStateFrames					:Array;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get currentStateFrame()			:* { return _currentStateFrame; }
		public function set currentStateFrame( value	:* ):void { _currentStateFrame = value; }
		private var _currentStateFrame					:*;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get disableStateFrame()			:* { return _disableStateFrame; }
		public function set disableStateFrame( value	:* ):void { _disableStateFrame = value; }
		private var _disableStateFrame					:*;
		
		/*======================================================================*//**
		* @private
		*/
		protected var _target							:MovieClip;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい RollOverButton インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function RollOverButton() {
			// 初期化する
			navigatePath = navigatePath;
			navigateType = navigateType;
			_upStateFrames = [ "over", "up" ];
			_overStateFrames = [ "up", "over" ];
			_downStateFrames = [ "down" ];
			_currentStateFrame = "current";
			_disableStateFrame = "disable";
			_target = this;
			
			// 再生を停止する
			stop();
			
			// イベントリスナーを登録する
			addExclusivelyEventListener( CastEvent.CAST_STATE_CHANGE, _castStateChange, false, int.MAX_VALUE, true );
			addExclusivelyEventListener( CastMouseEvent.CAST_ROLL_OVER, _castRollOver, false, int.MAX_VALUE, true );
			addExclusivelyEventListener( CastMouseEvent.CAST_ROLL_OUT, _castRollOut, false, int.MAX_VALUE, true );
			addExclusivelyEventListener( CastMouseEvent.CAST_MOUSE_DOWN, _castMouseDown, false, int.MAX_VALUE, true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object RollOverButton : " + ( id || name ) + "]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* 再生ヘッドが新しいフレームに入るときに送出されます。
		*/
		private function _enterFrame( e:Event ):void {
			// navigatePath を再登録する
			navigatePath = navigatePath;
			
			// 登録が完了したら
			if ( Progression.getInstanceBySceneId( sceneId ) ) {
				// イベントリスナーを解除する
				completelyRemoveEventListener( Event.ENTER_FRAME, _enterFrame );
			}
		}
		
		/*======================================================================*//**
		* キャストオブジェクトの状態が変更された場合に送出されます。
		*/
		private function _castStateChange( e:CastEvent ):void {
			// 無効化されていたら無効化表示にする
			if ( !buttonEnabled ) {
				_target.gotoAndStop( disableStateFrame );
				return;
			}
			
			// カレント状態であればカレント表示にする
			if ( isCurrent ) {
				_target.gotoAndStop( currentStateFrame );
				return;
			}
			
			// ロールアウトする
			_castRollOut( new CastMouseEvent( CastMouseEvent.CAST_ROLL_OUT ) );
		}
		
		/*======================================================================*//**
		* ユーザーが CastButton インスタンスにポインティングデバイスを合わせたときに送出されます。
		*/
		private function _castRollOver( e:CastMouseEvent ):void {
			if ( isCurrent || !buttonEnabled ) { return; }
			
			var start:int = 0;
			var current:int = 0;
			
			// 停止する
			try {
				var l:int = overStateFrames.length;
				for ( var i:int = 0; i < l; i++ ) {
					// 開始点、終了点を取得する
					var ps:* = overStateFrames[i];
					var pe:* = overStateFrames[i + 1];
					
					// pe が存在しなければ終了する
					if ( !pe ) { break; }
					
					if ( MovieClipUtil.playheadWithinFrames( _target, ps, pe ) ) {
						start = i;
						current = _target.currentFrame;
						_target.gotoAndStop( current );
						break;
					}
				}
				
				_target.gotoAndStop( ( i == l ) ? overStateFrames[0] : current );
			}
			catch ( e:Error ) {
				return;
			}
			
			// アニメーションを追加する
			var ll:int = overStateFrames.length;
			for ( var ii:int = start; ii < ll; ii++ ) {
				// 開始点、終了点を取得する
				ps = ( ii == start ) ? current : overStateFrames[ii];
				pe = overStateFrames[ii + 1];
				
				// pe が存在しなければ終了する
				if ( !pe ) { break; }
				
				// コマンドを追加する
				addCommand( new DoTweenFrame( _target, ps, pe ) );
			}
		}
		
		/*======================================================================*//**
		* ユーザーが CastButton インスタンスからポインティングデバイスを離したときに送出されます。
		*/
		private function _castRollOut( e:CastMouseEvent ):void {
			if ( isCurrent || !buttonEnabled ) { return; }
			
			var start:int = 0;
			var current:int = 0;
			
			// 停止する
			try {
				var l:int = upStateFrames.length;
				for ( var i:int = 0; i < l; i++ ) {
					// 開始点、終了点を取得する
					var ps:* = upStateFrames[i];
					var pe:* = upStateFrames[i + 1];
					
					// pe が存在しなければ終了する
					if ( !pe ) { break; }
					
					if ( MovieClipUtil.playheadWithinFrames( _target, ps, pe ) ) {
						start = i;
						current = _target.currentFrame;
						break;
					}
				}
				
				_target.gotoAndStop( ( i == l ) ? upStateFrames[0] : current );
			}
			catch ( e:Error ) {
				return;
			}
			
			// アニメーションを追加する
			var ll:int = upStateFrames.length;
			for ( var ii:int = start; ii < ll; ii++ ) {
				// 開始点、終了点を取得する
				ps = ( ii == start ) ? current : upStateFrames[ii];
				pe = upStateFrames[ii + 1];
				
				// pe が存在しなければ終了する
				if ( !pe ) { break; }
				
				// コマンドを追加する
				addCommand( new DoTweenFrame( _target, ps, pe ) );
			}
		}
		
		/*======================================================================*//**
		* Flash Player ウィンドウの CastButton インスタンスの上でユーザーがポインティングデバイスのボタンを押すと送出されます。
		*/
		private function _castMouseDown( e:CastMouseEvent ):void {
			if ( isCurrent || !buttonEnabled ) { return; }
			
			var start:int = 0;
			var current:int = 0;
			
			// 停止する
			try {
				var l:int = downStateFrames.length;
				for ( var i:int = 0; i < l; i++ ) {
					// 開始点、終了点を取得する
					var ps:* = downStateFrames[i];
					var pe:* = downStateFrames[i + 1];
					
					// pe が存在しなければ終了する
					if ( !pe ) { break; }
					
					if ( MovieClipUtil.playheadWithinFrames( _target, ps, pe ) ) {
						start = i;
						current = _target.currentFrame;
						break;
					}
				}
				
				_target.gotoAndStop( ( i == l ) ? downStateFrames[0] : current );
			}
			catch ( e:Error ) {
				return;
			}
			
			// アニメーションを追加する
			var ll:int = downStateFrames.length;
			for ( var ii:int = start; ii < ll; ii++ ) {
				// 開始点、終了点を取得する
				ps = ( ii == start ) ? current : downStateFrames[ii];
				pe = downStateFrames[ii + 1];
				
				// pe が存在しなければ終了する
				if ( !pe ) { break; }
				
				// コマンドを追加する
				addCommand( new DoTweenFrame( _target, ps, pe ) );
			}
		}
	}
}





