﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.commands {
	import fl.transitions.easing.Regular;
	import flash.display.DisplayObject;
	import flash.display.DisplayObjectContainer;
	import flash.display.MovieClip;
	import jp.nium.display.IExDisplayObjectContainer;
	import jp.progression.casts.CastBitmap;
	import jp.progression.casts.CastButton;
	import jp.progression.casts.CastLoader;
	import jp.progression.casts.CastMovieClip;
	import jp.progression.casts.CastSprite;
	import jp.progression.casts.CastTextField;
	import jp.progression.casts.ICastObject;
	import jp.progression.commands.DoTween;
	import jp.progression.commands.SerialList;
	import jp.progression.core.commands.Command;
	import jp.progression.core.commands.CommandList;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.events.CastEvent;
	import jp.progression.events.CommandCatchEvent;
	import jp.progression.events.CommandEvent;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* コマンドの実行中に例外エラーが発生した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CommandCatchEvent.EXCEPTION
	*/
	[Event( name="exception", type="jp.progression.events.CommandCatchEvent" )]
	
	/*======================================================================*//**
	* AddChild クラスは対象の DisplayObjectContainer インスタンスのディスプレイリストに ICastObject インスタンスを追加するコマンドクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.casts.CastSprite;
	* import jp.progression.commands.AddChild;
	* import jp.progression.commands.RemoveChild;
	* 
	* // CastSprite を作成する
	* var child:CastSprite = new CastSprite( "cast" );
	* 
	* // AddChild コマンドを実行する
	* new AddChild( this, child )
	* 	.before( null, trace, [ "キャストをディスプレイリストに追加します。" ] )
	* 	.after( null, trace, [ "キャストをディスプレイリストに追加しました。" ] )
	* 	.execute();
	* 
	* // RemoveChild コマンドを実行する
	* new RemoveChild( this, child )
	* 	.before( null, trace, [ "キャストをディスプレイリストから削除します。" ] )
	* 	.after( null, trace, [ "キャストをディスプレイリストから削除しました。" ] )
	* 	.execute();
	* </listing>
	*/
	public class AddChild extends Command {
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function get enabled()		:Boolean { return ( _container is DisplayObjectContainer && _child is DisplayObject ) ? super.enabled : false; }
		
		/*======================================================================*//**
		* ICastObject インスタンスを追加する対象のディスプレイリストを含む DisplayObjectContainer インスタンスを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get container()				:DisplayObjectContainer { return _container; }
		public function set container( value		:DisplayObjectContainer ):void { _container = value; }
		private var _container						:DisplayObjectContainer;
		
		/*======================================================================*//**
		* ディスプレイリストに追加したい DisplayObject インスタンスを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get child()					:DisplayObject { return _child; }
		public function set child( value			:DisplayObject ):void { _child = value; }
		private var _child							:DisplayObject;
		
		/*======================================================================*//**
		* キャストを削除する際に、alpha プロパティを使用したフェード効果を使用するかどうかを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get autoAlpha()			:Boolean { return _autoAlpha; }
		public function set autoAlpha( value	:Boolean ):void { _autoAlpha = value; }
		private var _autoAlpha					:Boolean = false;
		
		/*======================================================================*//**
		* CommandList インスタンスを取得します。 
		*/
		private var _commandList					:CommandList;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい AddChild インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	container	指定された DisplayObject インスタンスを表示リストを追加する対象の DisplayObjectContainer インスタンスです。
		* @param	child		表示リストに追加したい DisplayObject インスタンスです。
		* @param	autoAlpha	キャストを削除する際に、alpha プロパティを使用したフェード効果を使用するかどうかです。
		* @param	delay		処理開始まで遅延時間をミリ秒で指定します。
		*/
		public function AddChild( container:DisplayObjectContainer = null, child:DisplayObject = null, autoAlpha:Boolean = false, delay:int = 0 ) {
			super( "AddChild", delay );
			
			// 引数を設定する
			this.container = container;
			this.child = child;
			this.autoAlpha = autoAlpha;
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _executeProgress():void {
			// すでに画面に配置されていれば終了する
			if ( _container.contains( _child ) ) {
				// 処理を終了する
				_executeComplete();
				return;
			}
			
			// ICastObject インターフェイスを実装していれば
			if ( _child is ICastObject ) {
				// キャストのコマンドリストを消去する
				ICastObject( _child ).clearCommand();
			}
			else if ( "progression_internal_clearCommand" in _child ) {
				// キャストのコマンドリストを消去する
				MovieClip( _child ).progression_internal_clearCommand();
			}
			
			// ディスプレイリストに追加する
			_container.addChild( _child );
			
			// イベントを送出する
			_child.dispatchEvent( new CastEvent( CastEvent.CAST_ADDED, false, false ) );
			
			var commandList:CommandList;
			switch ( true ) {
				case child is CastBitmap						: { commandList = CastBitmap( child ).progression_internal::commandList; break; }
				case child is CastButton						: { commandList = CastButton( child ).progression_internal::commandList; break; }
				case child is CastLoader						: { commandList = CastLoader( child ).progression_internal::commandList; break; }
				case child is CastMovieClip						: { commandList = CastMovieClip( child ).progression_internal::commandList; break; }
				case child is CastSprite						: { commandList = CastSprite( child ).progression_internal::commandList; break; }
				case child is CastTextField						: { commandList = CastTextField( child ).progression_internal::commandList; break; }
				case "progression_internal_getCommand" in child	: { commandList = MovieClip( child ).progression_internal_getCommand(); break;  }
			}
			
			// autoAlpha プロパティが有効であれば
			if ( autoAlpha ) {
				_child.alpha = 0;
				
				commandList = commandList || new SerialList();
				
				commandList.addCommand( new DoTween( child, {
					alpha	:1
					}, Regular.easeInOut, 500 ) );
			}
			
			// コマンドが存在すれば実行する
			if ( commandList && commandList.length > 0 ) {
				_commandList = CommandList( commandList.clone() );
				_commandList.addExclusivelyEventListener( CommandEvent.COMMAND_COMPLETE, _commandComplete, false, int.MAX_VALUE, true );
				_commandList.addExclusivelyEventListener( CommandCatchEvent.EXCEPTION, _catch, false, int.MAX_VALUE, true );
				_commandList.addExclusivelyEventListener( CommandCatchEvent.ERROR, _catch, false, int.MAX_VALUE, true );
				_commandList.execute( extra );
				return;
			}
			
			// コマンドを空にする
			_commandList = null;
			
			// 処理を終了する
			_executeComplete();
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _interruptProgress():void {
			// 実行中のコマンドがあれば
			if ( _commandList ) {
				// イベントリスナーを解除する
				_commandList.removeAllListeners( true );
				
				// 停止する
				_commandList.addEventListener( CommandEvent.COMMAND_INTERRUPT, _commandInterrupt, false, int.MAX_VALUE, true );
				_commandList.interrupt( extra );
				return;
			}
			
			// ディスプレイリストに追加する
			_container.addChild( _child );
			
			// 停止処理を終了する
			_interruptComplete();
		}
		
		/*======================================================================*//**
		* AddChild インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい AddChild インスタンスです。
		*/
		public override function clone():Command {
			var command:Command = new AddChild( _container, _child, _autoAlpha, delay );
			command.enabled = enabled;
			command.before( _beforeScorp, _beforeFunc, _beforeArgs );
			command.after( _afterScorp, _afterFunc, _afterArgs );
			return command;
		}
		
		
		
		
		
		/*======================================================================*//**
		* コマンドの処理が完了した場合に送出されます。
		*/
		private function _commandComplete( e:CommandEvent ):void {
			Command( e.target ).removeAllListeners( true );
			
			// 処理を終了する
			_executeComplete();
		}
		
		/*======================================================================*//**
		* コマンドの処理を停止した場合に送出されます。
		*/
		private function _commandInterrupt( e:CommandEvent ):void {
			Command( e.target ).removeAllListeners( true );
			
			// コマンドを空にする
			_commandList = null;
			
			// ディスプレイリストに追加する
			_container.addChild( _child );
			
			// 停止処理を終了する
			_interruptComplete();
		}
		
		/*======================================================================*//**
		* コマンド処理中にエラーが発生した場合に送出されます。
		*/
		private function _catch( e:CommandCatchEvent ):void {
			Command( e.target ).removeAllListeners( true );
			
			// 処理を終了する
			_executeComplete();
		}
	}
}





