﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.commands {
	import fl.transitions.Tween;
	import fl.transitions.TweenEvent;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.events.Event;
	import jp.nium.events.EventUnit;
	import jp.progression.core.commands.Command;
	import jp.progression.events.CommandCatchEvent;
	import jp.progression.events.CommandEvent;
	
	/*======================================================================*//**
	* コマンドの実行中に例外エラーが発生した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CommandCatchEvent.EXCEPTION
	*/
	[Event( name="exception", type="jp.progression.events.CommandCatchEvent" )]
	
	/*======================================================================*//**
	* DoTweener クラスは fl.transitions.Tween パッケージのイージング機能を実行するコマンドクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import fl.transitions.easing.Elastic;
	* import jp.nium.graphics.Square;
	* import jp.progression.commands.DoTween;
	* 
	* // Square を作成する
	* var square:Square = new Square();
	* addChild( square );
	* 
	* // DoTween コマンドを実行する
	* new DoTween( this, {
	* 	x			:100,
	* 	y			:100
	* 	}, Elastic.easeInOut, 1000 )
	* 	.before( null, trace, [ "トゥイーンを実行します。" ] )
	* 	.after( null, trace, [ "トゥイーンを実行しました。" ] )
	* 	.execute();
	* </listing>
	*/
	public class DoTween extends Command {
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function get enabled()		:Boolean { return ( _target is Object && _props is Object && _easing is Function ) ? super.enabled : false; }
		
		/*======================================================================*//**
		* イージング処理を行いたい対象のオブジェクトを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get target()				:Object { return _target; }
		public function set target( value			:Object ):void { _target = value; }
		private var _target							:Object;
		
		/*======================================================================*//**
		* イージング処理を行いたいプロパティを含んだオブジェクトを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get props()					:Object { return _props; }
		public function set props( value			:Object ):void { _props = value; }
		private var _props							:Object;
		
		/*======================================================================*//**
		* イージング処理を行う関数を取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get easing()				:Function { return _easing; }
		public function set easing( value			:Function ):void { _easing = value; }
		private var _easing							:Function;
		
		/*======================================================================*//**
		* イージング処理の継続時間です。負の数、または省略されている場合、infinity に設定されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get duration()				:int { return _duration; }
		public function set duration( value			:int ):void { _duration = value; }
		private var _duration						:int;
		
		/*======================================================================*//**
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		/*
		public function get useBitmap()				:Boolean { return _useBitmap; }
		public function set useBitmap( value		:Boolean ):void { _useBitmap = value; }
		private var _useBitmap						:Boolean = false;
		*/
		
		/*======================================================================*//**
		* Bitmap インスタンスを取得します。 
		*/
		private var _bitmap							:Bitmap;
		
		/*======================================================================*//**
		*/
		private var _tweens							:Array;
		
		/*======================================================================*//**
		* EventUnit インスタンスを取得します。 
		*/
		private var _eventUnit						:EventUnit;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい DoTween インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	target		Tween のターゲットになるオブジェクトです。
		* @param	props		影響を受ける (target パラメータ値) のプロパティの名前と値を格納した Object インスタンスです。 
		* @param	easing		使用するイージング関数の名前です。
		* @param	duration	モーションの継続時間です。負の数、または省略されている場合、infinity に設定されます。
		* @param	delay		処理開始まで遅延時間をミリ秒で指定します。
		*/
		public function DoTween( target:Object = null, props:Object = null, easing:Function = null, duration:int = 0, delay:int = 0 ) {//, useBitmap:Boolean = false ) {
			super( "DoTween", delay );
			
			// 初期化する
			_tweens = [];
			_eventUnit = new EventUnit();
			
			// 引数を設定する
			this.target = target;
			this.props = props;
			this.easing = easing;
			this.duration = duration;
			
			/*
			this.useBitmap = useBitmap;
			*/
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _executeProgress():void {
			// 初期化する
			_tweens = [];
			_eventUnit = new EventUnit();
			_eventUnit.addExclusivelyEventListener( Event.COMPLETE, _complete, false, int.MAX_VALUE, true );
			
			// Tween を作成する
			for ( var p:String in _props ) {
				// プロパティが存在していなければ次へ
				if ( !( p in _target ) ) { continue; }
				
				// Tween を作成する
				var tween:Tween = new Tween( _target, p, _easing, _target[p], _props[p], _duration / 1000, true );
				_eventUnit.addEventDispatcher( tween, TweenEvent.MOTION_FINISH );
				_tweens.push( tween );
			}
			
			// Tween を実行する
			var l:int = _tweens.length;
			for ( var i:int = 0; i < l; i++ ) {
				_tweens[i].start();
			}
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _interruptProgress():void {
			// イベントリスナーを解除する
			_eventUnit.removeAllListeners( true );
			
			// Tween を解除する
			var l:int = _tweens.length;
			for ( var i:int = 0; i < l; i++ ) {
				Tween( _tweens[i] ).stop();
			}
			
			// 停止処理を終了する
			_interruptComplete();
		}
		
		/*======================================================================*//**
		* DoTween インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい DoTween インスタンスです。
		*/
		public override function clone():Command {
			var command:Command = new DoTween( _target, _props, _easing, _duration, delay );// , useBitmap );
			command.enabled = enabled;
			command.before( _beforeScorp, _beforeFunc, _beforeArgs );
			command.after( _afterScorp, _afterFunc, _afterArgs );
			return command;
		}
		
		
		
		
		
		/*======================================================================*//**
		* 登録された全てのイベントが発生した際に送出されます。
		*/
		private function _complete( e:Event ):void {
			// イベントリスナーを解除する
			EventUnit( e.target ).removeAllListeners( true );
			
			// 処理を終了する
			_executeComplete();
		}
	}
}





