﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.commands {
	import flash.display.MovieClip;
	import flash.events.Event;
	import jp.nium.utils.MovieClipUtil;
	import jp.progression.core.commands.Command;
	import jp.progression.events.CommandEvent;
	
	/*======================================================================*//**
	* DoTweenFrame クラスは、指定されたフレーム間のアニメーション処理を実行するコマンドクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.casts.CastMovieClip;
	* import jp.progression.commands.DoTweenFrame;
	* 
	* // CastMovieClip を作成する
	* var cast:CastMovieClip = new CastMovieClip();
	* addChild( cast );
	* 
	* // DoTweenFrame コマンドを実行する
	* new DoTweenFrame( cast, 1, 10 )
	* 	.before( null, trace, [ "フレーム　1 からトゥイーンを実行します。" ] )
	* 	.after( null, trace, [ "フレーム　10 までトゥイーンを実行しました。" ] )
	* 	.execute();
	* 
	* // DoTweenFrame コマンドを実行する
	* new DoTweenFrame( cast, "start", "end" )
	* 	.before( null, trace, [ "start ラベルからトゥイーンを実行します。" ] )
	* 	.after( null, trace, [ "end ラベルまでトゥイーンを実行しました。" ] )
	* 	.execute();
	* </listing>
	*/
	public class DoTweenFrame extends Command {
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function get enabled()		:Boolean { return ( _target is MovieClip ) ? super.enabled : false; }
		
		/*======================================================================*//**
		* アニメーション処理を行いたい対象の MovieClip インスタンスを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get target()				:MovieClip { return _target; }
		public function set target( value			:MovieClip ):void { _target = value; }
		private var _target							:MovieClip;
		
		/*======================================================================*//**
		* アニメーションを開始するフレーム番号もしくはラベル名を取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get startFrame()			:* { return _startFrame; }
		public function set startFrame( value		:* ):void { _startFrame = value; }
		private var _startFrame						:* = 1;
		
		/*======================================================================*//**
		* アニメーションを開始するフレーム番号を取得します。
		*/
		private var _startFrameNum					:int = 1;
		
		/*======================================================================*//**
		* アニメーションを終了するフレーム番号もしくはラベル名を取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get endFrame()				:* { return _endFrame; }
		public function set endFrame( value			:* ):void { _endFrame = value; }
		private var _endFrame						:* = 1;
		
		/*======================================================================*//**
		* アニメーションを終了するフレーム番号を取得します。
		*/
		private var _endFrameNum					:int = 1;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい DoTweenFrame インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	target		アニメーション処理を行いたい対象の MovieClip インスタンスです。
		* @param	startFrame	アニメーションを開始するフレーム番号もしくはラベル名です。
		* @param	endFrame	アニメーションを終了するフレーム番号もしくはラベル名です。
		* @param	delay		処理開始まで遅延時間をミリ秒で指定します。
		*/
		public function DoTweenFrame( target:MovieClip = null, startFrame:* = 1, endFrame:* = 1, delay:int = 0 ) {
			super( "DoTweenFrame", delay );
			
			// 引数を設定する
			this.target = target;
			this.startFrame = startFrame;
			this.endFrame = endFrame;
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _executeProgress():void {
			// 移動先のフレーム番号を取得する
			_startFrameNum = isNaN( _startFrame ) ? MovieClipUtil.labelToFrames( _target, _startFrame ) : _startFrame;
			_endFrameNum = isNaN( _endFrame ) ? MovieClipUtil.labelToFrames( _target, _endFrame ) : _endFrame;
			
			// フレームが存在しなければ
			if ( _startFrameNum < 1 || _endFrameNum < 1 ) {
				// 処理を終了する
				_executeComplete();
				return;
			}
			
			// 現在位置が範囲内に存在しなければ
			if ( !MovieClipUtil.playheadWithinFrames( _target, _startFrameNum, _endFrameNum ) ) {
				// 開始フレームに移動する
				_target.gotoAndStop( _startFrameNum );
			}
			
			// イベントリスナーを登録する
			target.addEventListener( Event.ENTER_FRAME, _enterFrame, false, int.MAX_VALUE, true );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _interruptProgress():void {
			// イベントリスナーを解除する
			_target.removeEventListener( Event.ENTER_FRAME, _enterFrame );
			
			// 停止処理を終了する
			_interruptComplete();
		}
		
		/*======================================================================*//**
		* DoTweenFrame インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい DoTweenFrame インスタンスです。
		*/
		public override function clone():Command {
			var command:Command = new DoTweenFrame( _target, _startFrame, _endFrame, delay );
			command.enabled = enabled;
			command.before( _beforeScorp, _beforeFunc, _beforeArgs );
			command.after( _afterScorp, _afterFunc, _afterArgs );
			return command;
		}
		
		
		
		
		
		/*======================================================================*//**
		*/
		private function _enterFrame( e:Event ):void {
			if ( _target.currentFrame == _endFrameNum ) {
				// イベントリスナーを解除する
				_target.removeEventListener( Event.ENTER_FRAME, _enterFrame );
				
				// 処理を終了する
				_executeComplete();
			}
			else if ( _target.currentFrame < _endFrameNum ) {
				_target.nextFrame();
			}
			else {
				_target.prevFrame();
			}
		}
	}
}





