﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.commands {
	import caurina.transitions.Tweener;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import jp.nium.utils.ObjectUtil;
	import jp.progression.core.commands.Command;
	import jp.progression.events.CommandCatchEvent;
	import jp.progression.events.CommandEvent;
	
	/*======================================================================*//**
	* コマンドの実行中に例外エラーが発生した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CommandCatchEvent.EXCEPTION
	*/
	[Event( name="exception", type="jp.progression.events.CommandCatchEvent" )]
	
	/*======================================================================*//**
	* DoTweener クラスは caurina.transitions.Tweener パッケージのイージング機能を実行するコマンドクラスです。
	* <p/>
	* Tweener
	* Transition controller for movieclips, sounds, textfields and other objects
	* Zeh Fernando, Nate Chatellier, Arthur Debert
	* @see		http://code.google.com/p/tweener/
	* @see		http://code.google.com/p/tweener/wiki/License
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import caurina.transitions.Equations;
	* import jp.nium.graphics.Square;
	* import jp.progression.commands.DoTweener;
	* 
	* // Square を作成する
	* var square:Square = new Square();
	* addChild( square );
	* 
	* // DoTweener コマンドを実行する
	* new DoTweener( square, {
	* 	x			:100,
	* 	y			:100,
	* 	time		:3,
	* 	transition	:Equations.easeInOutBack
	* 	}, 1000 )
	* 	.before( null, trace, [ "トゥイーンを実行します。" ] )
	* 	.after( null, trace, [ "トゥイーンを実行しました。" ] )
	* 	.execute();
	* </listing>
	*/
	public class DoTweener extends Command {
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function get enabled()			:Boolean { return ( _target is Object && _tweeningParameters is Object ) ? super.enabled : false; }
		
		/*======================================================================*//**
		* Any object that will suffer a tweening. These objects are usually MovieClip, TextField, or Sound instances, or any other custom object with a numeric property that needs to be tweened.
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get target()					:Object { return _target; }
		public function set target( value				:Object ):void { _target = value; }
		private var _target								:Object;
		
		/*======================================================================*//**
		* An object containing various properties of the original object that you want to tween on the original objects, with their final values assigned (some special properties are also allowed), as well as some built-in Tweener properties used when defining tweening parameters. This is like the recipe for the tweening, declaring both what will be tweened, and how.
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get tweeningParameters()		:Object { return _tweeningParameters; }
		public function set tweeningParameters( value	:Object ):void { _tweeningParameters = value; }
		private var _tweeningParameters					:Object;
		
		/*======================================================================*//**
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		/*
		public function get useBitmap()					:Boolean { return _useBitmap; }
		public function set useBitmap( value			:Boolean ):void { _useBitmap = value; }
		private var _useBitmap							:Boolean = false;
		*/
		
		/*======================================================================*//**
		* Tweener に設定されていた onComplete() メソッドを取得します。
		*/
		private var _onCompleteFunction					:Function;
		
		/*======================================================================*//**
		* Bitmap インスタンスを取得します。 
		*/
		private var _bitmap								:Bitmap;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい DoTweener インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	target				Any object that will suffer a tweening. These objects are usually MovieClip, TextField, or Sound instances, or any other custom object with a numeric property that needs to be tweened.
		* @param	tweeningParameters	An object containing various properties of the original object that you want to tween on the original objects, with their final values assigned (some special properties are also allowed), as well as some built-in Tweener properties used when defining tweening parameters. This is like the recipe for the tweening, declaring both what will be tweened, and how.
		* @param	delay				処理開始まで遅延時間をミリ秒で指定します。
		*/
		public function DoTweener( target:Object = null, tweeningParameters:Object = null, delay:int = 0 ) {//, useBitmap:Boolean = false ) {
			super( "DoTweener", delay );
			
			// 引数を設定する
			this.target = target;
			this.tweeningParameters = tweeningParameters;
			
			/*
			this.useBitmap = useBitmap;
			*/
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _executeProgress():void {
			// 既存のイベントハンドラメソッドを保存する
			_onCompleteFunction = tweeningParameters.onComplete;
			
			// イベントハンドラメソッドを登録する
			tweeningParameters.onComplete = _onComplete;
			
			// アニメーションを開始する
			if ( !Tweener.addTween( _target, _tweeningParameters ) ) {
				// 処理を終了する
				_executeComplete();
			}
		}
		
		/*======================================================================*//**
		* 
		*/
		private function _onComplete():void {
			// Tweener に設定されていた onComplete() メソッドを実行する
			if ( _onCompleteFunction is Function ) {
				_onCompleteFunction.apply( null );
			}
			
			// 処理を終了する
			_executeComplete();
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _interruptProgress():void {
			// Tween を停止する
			_tweeningParameters.onComplete = null;
			Tweener.removeTweens( _target );
			
			// 停止処理を終了する
			_interruptComplete();
		}
		
		/*======================================================================*//**
		* DoTweener インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい DoTweener インスタンスです。
		*/
		public override function clone():Command {
			var command:Command = new DoTweener( _target, _tweeningParameters, delay );// , useBitmap );
			command.enabled = enabled;
			command.before( _beforeScorp, _beforeFunc, _beforeArgs );
			command.after( _afterScorp, _afterFunc, _afterArgs );
			return command;
		}
	}
}





