﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.commands {
	import jp.progression.core.commands.Command;
	import jp.progression.events.CommandEvent;
	
	/*======================================================================*//**
	* Empty クラスは、何も処理を行わないコマンドクラスです。
	* IfElse クラスの条件分岐で、何も処理を行いたくない場合などで使用します。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.commands.Empty;
	* import jp.progression.commands.IfElse;
	* import jp.progression.commands.Trace;
	* 
	* // IfElse コマンドを実行する
	* new IfElse( function():Boolean { return Math.random() > 0.5; }
	* 	new Trace( "条件が true です。" ),
	* 	new Empty() )
	* 	.before( null, trace, [ "条件によってコマンド処理を振り分けます。" ] )
	* 	.after( null, trace, [ "条件によってコマンド処理を振り分けました。" ] )
	* 	.execute();
	* </listing>
	*/
	public class Empty extends Command {
		
		/*======================================================================*//**
		* 新しい Empty インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function Empty() {
			super( "Empty", 0 );
		}
		
		
		
		
		
		/*======================================================================*//**
		* Empty インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい Empty インスタンスです。
		*/
		public override function clone():Command {
			var command:Command = new Empty();
			command.enabled = enabled;
			command.before( _beforeScorp, _beforeFunc, _beforeArgs );
			command.after( _afterScorp, _afterFunc, _afterArgs );
			return command;
		}
	}
}





