﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.commands {
	import flash.events.Event;
	import flash.events.IEventDispatcher;
	import jp.progression.core.commands.Command;
	import jp.progression.events.CommandCatchEvent;
	import jp.progression.events.CommandEvent;
	
	/*======================================================================*//**
	* コマンドの実行中に例外エラーが発生した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CommandCatchEvent.EXCEPTION
	*/
	[Event( name="exception", type="jp.progression.events.CommandCatchEvent" )]
	
	/*======================================================================*//**
	* Func クラスは指定された関数を実行するコマンドクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.commands.Func;
	* 
	* // Func コマンドを実行する
	* new Func( null, trace, [ "関数を実行中です。" ] )
	* 	.before( null, trace, [ "関数を実行します。" ] )
	* 	.after( null, trace, [ "関数を実行しました。" ] )
	* 	.execute();
	* 
	* // 遅延処理を行う Func コマンドを実行する
	* new Func( null, function():void {
	* 	// イベントを送出します。
	* 	dispatchEvent( new Event( Event.COMPLETE ) );
	* }, null, 0, this, Event.COMPLETE )
	* 	.before( null, trace, [ "関数を実行します。" ] )
	* 	.after( null, trace, [ "関数を実行しました。" ] )
	* 	.execute();
	* </listing>
	*/
	public class Func extends Command {
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function get enabled()		:Boolean { return ( _func is Function ) ? super.enabled : false; }
		
		/*======================================================================*//**
		* 関数の適用先のオブジェクトを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get scope()					:Object { return _scope; }
		public function set scope( value			:Object ):void { _scope = value || this; }
		private var _scope							:Object;
		
		/*======================================================================*//**
		* 実行したい関数を取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get func()					:Function { return _func; }
		public function set func( value				:Function ):void { _func = value; }
		private var _func							:Function;
		
		/*======================================================================*//**
		* 関数を実行する際に引数として使用する配列を取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get args()					:Array { return _args; }
		public function set args( value				:Array ):void { _args = value; }
		private var _args							:Array;
		
		/*======================================================================*//**
		* 処理の終了イベントを発行する EventDispatcher インスタンスを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get dispatcher()			:IEventDispatcher { return _dispatcher; }
		public function set dispatcher( value		:IEventDispatcher ):void { _dispatcher = value; }
		private var _dispatcher						:IEventDispatcher;
		
		/*======================================================================*//**
		* 発行される終了イベントの種類を取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get eventType()				:String { return _eventType; }
		public function set eventType( value		:String ):void { _eventType = value; }
		private var _eventType						:String;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい Func インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	scope		関数の適用先のオブジェクトです。
		* @param	func		実行したい関数です。
		* @param	args		関数を実行する際に引数として使用する配列です。
		* @param	delay		処理開始まで遅延時間をミリ秒で指定します。
		* @param	dispatcher	処理の終了イベントを発行する EventDispatcher インスタンスです。
		* @param	eventType	発行される終了イベントの種類です。
		*/
		public function Func( scope:Object = null, func:Function = null, args:Array = null, delay:int = 0, dispatcher:IEventDispatcher = null, eventType:String = null ) {
			super( "Func", delay );
			
			// 引数を設定する
			this.scope = scope;
			this.func = func;
			this.args = args;
			this.dispatcher = dispatcher;
			this.eventType = eventType;
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _executeProgress():void {
			// イベントが存在するかどうか確認する
			var hasEvent:Boolean = _dispatcher && _eventType;
			
			// イベントが存在すれば登録する
			if ( hasEvent ) {
				_dispatcher.addEventListener( _eventType, _listener, false, int.MAX_VALUE, true );
			}
			
			// 関数を実行する
			_func.apply( _scope, _args );
			
			// イベントが存在すれば終了する
			if ( hasEvent ) { return; }
			
			// 処理を終了する
			_executeComplete();
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _interruptProgress():void {
			// イベントが存在すれば
			if ( _dispatcher && _eventType ) {
				// イベントリスナーを解除する
				_dispatcher.removeEventListener( _eventType, _listener );
			}
			
			// 停止処理を終了する
			_interruptComplete();
		}
		
		/*======================================================================*//**
		* Func インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい Func インスタンスです。
		*/
		public override function clone():Command {
			var command:Command = new Func( _scope, _func, _args ? _args.concat() : null, delay, _dispatcher, _eventType );
			command.enabled = enabled;
			command.before( _beforeScorp, _beforeFunc, _beforeArgs );
			command.after( _afterScorp, _afterFunc, _afterArgs );
			return command;
		}
		
		
		
		
		
		/*======================================================================*//**
		* 
		*/
		private function _listener( e:Event ):void {
			IEventDispatcher( e.target ).removeEventListener( eventType, _listener );
			
			// 処理を終了する
			_executeComplete();
		}
	}
}





