﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.commands {
	import jp.progression.core.commands.Command;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.events.CommandCatchEvent;
	import jp.progression.events.CommandEvent;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* コマンドの実行中に例外エラーが発生した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CommandCatchEvent.EXCEPTION
	*/
	[Event( name="exception", type="jp.progression.events.CommandCatchEvent" )]
	
	/*======================================================================*//**
	* IfElse クラスは指定された条件に応じて実行するコマンドを振り分けるコマンドクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.commands.IfElse;
	* import jp.progression.commands.Trace;
	* 
	* // IfElse コマンドを実行する
	* new IfElse( function():Boolean { return Math.random() > 0.5; }
	* 	new Trace( "条件が true です。" ),
	* 	new Trace( "条件が false です。" ) )
	* 	.before( null, trace, [ "条件によってコマンド処理を振り分けます。" ] )
	* 	.after( null, trace, [ "条件によってコマンド処理を振り分けました。" ] )
	* 	.execute();
	* </listing>
	*/
	public class IfElse extends Command {
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function get enabled()	:Boolean { return ( _condition is Function ) ? super.enabled : false; }
		
		/*======================================================================*//**
		* 条件の設定されている関数を取得または設定します。
		* この関数は Boolean 型の戻り値が必要です。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get condition()			:Function { return _condition; }
		public function set condition( value	:Function ):void { _condition = value; }
		private var _condition					:Function;
		
		/*======================================================================*//**
		* 条件関数の戻り値が true の場合に実行したいコマンドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get success()			:Command { return _success; }
		public function set success( value		:Command ):void {
			if ( _success ) {
				// 関連性を削除する
				_success.progression_internal::parent = null;
				_success.progression_internal::depth = 0;
			}
			
			// 設定する
			_success = value;
			
			if ( _success ) {
				// 関連性を設定する
				_success.progression_internal::parent = this;
				_success.progression_internal::depth = progression_internal::depth + 1;
			}
		}
		private var _success					:Command;
		
		/*======================================================================*//**
		* 条件関数の戻り値が false の場合に実行したいコマンドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get failure()			:Command { return _failure; }
		public function set failure( value		:Command ):void {
			if ( _failure ) {
				// 関連性を削除する
				_failure.progression_internal::parent = null;
				_failure.progression_internal::depth = 0;
			}
			
			// 設定する
			_failure = value;
			
			if ( _failure ) {
				// 関連性を設定する
				_failure.progression_internal::parent = this;
				_failure.progression_internal::depth = progression_internal::depth + 1;
			}
		}
		private var _failure					:Command;
		
		/*======================================================================*//**
		* Command インスタンスを取得します。 
		*/
		private var _currentCommand				:Command;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい IfElse インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	condition	条件の設定されている関数です。
		* @param	success		条件関数の戻り値が true の場合に実行したいコマンドです。
		* @param	failure		条件関数の戻り値が false の場合に実行したいコマンド
		* @param	delay		処理開始まで遅延時間をミリ秒で指定します。
		*/
		public function IfElse( condition:Function = null, success:Command = null, failure:Command = null, delay:int = 0 ) {
			super( "IfElse", delay );
			
			// 引数を設定する
			this.condition = condition;
			this.success = success;
			this.failure = failure;
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _executeProgress():void {
			// 条件関数を実行する
			_currentCommand = _condition() ? _success : _failure;
			
			// コマンドが存在すれば実行する
			if ( _currentCommand ) {
				_currentCommand.addExclusivelyEventListener( CommandEvent.COMMAND_COMPLETE, _commandComplete, false, int.MAX_VALUE, true );
				_currentCommand.execute( extra );
				return;
			}
			
			// 処理を終了する
			_executeComplete();
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _interruptProgress():void {
			// 実行中のコマンドがあれば
			if ( _currentCommand ) {
				// イベントリスナーを解除する
				_currentCommand.removeAllListeners( true );
				
				// 停止する
				_currentCommand.addExclusivelyEventListener( CommandEvent.COMMAND_INTERRUPT, _commandInterrupt, false, int.MAX_VALUE, true );
				_currentCommand.interrupt( extra );
				return;
			}
			
			// 停止処理を終了する
			_interruptComplete();
		}
		
		/*======================================================================*//**
		* IfElse インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい IfElse インスタンスです。
		*/
		public override function clone():Command {
			var command:Command = new IfElse( _condition, _success ? _success.clone() : null, _failure ? _failure.clone() : null, delay );
			command.enabled = enabled;
			command.before( _beforeScorp, _beforeFunc, _beforeArgs );
			command.after( _afterScorp, _afterFunc, _afterArgs );
			return command;
		}
		
		
		
		
		
		/*======================================================================*//**
		* コマンドの処理が完了した場合に送出されます。
		*/
		private function _commandComplete( e:CommandEvent ):void {
			Command( e.target ).removeAllListeners( true );
			
			// 処理を終了する
			_executeComplete();
		}
		
		/*======================================================================*//**
		* コマンドの処理を停止した場合に送出されます。
		*/
		private function _commandInterrupt( e:CommandEvent ):void {
			Command( e.target ).removeAllListeners( true );
			
			// 停止処理を終了する
			_interruptComplete();
		}
	}
}





