﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.commands {
	import flash.events.Event;
	import flash.events.HTTPStatusEvent;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	import flash.events.SecurityErrorEvent;
	import flash.net.URLRequest;
	import jp.nium.net.ExURLLoader;
	import jp.progression.core.commands.Command;
	import jp.progression.events.CommandCatchEvent;
	import jp.progression.events.CommandEvent;
	
	/*======================================================================*//**
	* コマンドの実行中にイベントが発生した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CommandCatchEvent.ERROR
	*/
	[Event( name="error", type="jp.progression.events.CommandCatchEvent" )]
	
	/*======================================================================*//**
	* コマンドの実行中に例外エラーが発生した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CommandCatchEvent.EXCEPTION
	*/
	[Event( name="exception", type="jp.progression.events.CommandCatchEvent" )]
	
	/*======================================================================*//**
	* LoadURL クラスは指定されたファイルを読み込ませるコマンドクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import flash.net.URLRequest;
	* import jp.progression.commands.LoadURL;
	* 
	* // LoadURL コマンドを実行する
	* new LoadURL( new URLRequest( "http://progression.jp/" ) )
	* 	.before( null, trace, [ "テキストデータを読み込みます。" ] )
	* 	.after( null, trace, [ "読み込んだデータの内容 :", this.data ] )
	* 	.execute();
	* </listing>
	*/
	public class LoadURL extends Command {
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function get enabled()		:Boolean { return ( _request is URLRequest ) ? super.enabled : false; }
		
		/*======================================================================*//**
		* 読み込むファイルの絶対 URL または相対 URL を表す URLRequest インスタンスを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get request()				:URLRequest { return _request; }
		public function set request( value			:URLRequest ):void { _request = value; }
		private var _request						:URLRequest;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get bytesLoaded()			:int { return _loader.bytesLoaded; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get bytesTotal()			:int { return _loader.bytesTotal; }
		
		/*======================================================================*//**
		* ロード操作によって受信したデータを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get data()					:Object { return _loader.data; }
		
		/*======================================================================*//**
		* URLLoader インスタンスを取得します。 
		*/
		private var _loader							:ExURLLoader;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい LoadURL インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	request		読み込むファイルの絶対 URL または相対 URL を表す URLRequest インスタンスです。
		* @param	delay		処理開始まで遅延時間をミリ秒で指定します。
		*/
		public function LoadURL( request:URLRequest = null, delay:int = 0 ) {
			super( "LoadURL", delay );
			
			// 初期化する
			_loader = new ExURLLoader();
			
			// 引数を設定する
			this.request = request;
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _executeProgress():void {
			// コンテンツを読み込む
			_loader = new ExURLLoader();
            _loader.addEventListener( Event.OPEN, _open, false, int.MAX_VALUE, true );
			_loader.addEventListener( Event.COMPLETE, _complete, false, int.MAX_VALUE, true );
            _loader.addEventListener( ProgressEvent.PROGRESS, _progress, false, int.MAX_VALUE, true );
            _loader.addEventListener( SecurityErrorEvent.SECURITY_ERROR, _securityError, false, int.MAX_VALUE, true );
            _loader.addEventListener( HTTPStatusEvent.HTTP_STATUS, _httpStatus, false, int.MAX_VALUE, true );
            _loader.addEventListener( IOErrorEvent.IO_ERROR, _ioError, false, int.MAX_VALUE, true );
			_loader.load( _request );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _interruptProgress():void {
			// イベントリスナーを解除する
			_loader.removeAllListeners( true );
			
			// 読み込みを中断する
			_loader.close();
			
			// 停止処理を終了する
			_interruptComplete();
		}
		
		/*======================================================================*//**
		* LoadURL インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい LoadURL インスタンスです。
		*/
		public override function clone():Command {
			var command:Command = new LoadURL( _request, delay );
			command.enabled = enabled;
			command.before( _beforeScorp, _beforeFunc, _beforeArgs );
			command.after( _afterScorp, _afterFunc, _afterArgs );
			return command;
		}
		
		
		
		
		
		/*======================================================================*//**
		* URLLoader.load() メソッドの呼び出しによりダウンロード処理が開始されると送出されます。
		*/
		private function _open( e:Event ):void {
		}
		
		/*======================================================================*//**
		* 受信したすべてのデータがデコードされて URLLoader インスタンスの data プロパティへの格納が完了したときに送出されます。
		*/
		private function _complete( e:Event ):void {
			// イベントリスナーを解除する
			ExURLLoader( e.target ).removeAllListeners( true );
			
			// 処理を終了する
			_executeComplete();
		}
		
		/*======================================================================*//**
		* ダウンロード処理を実行中にデータを受信したときに送出されます。
		*/
		private function _progress( e:ProgressEvent ):void {
		}
		
		/*======================================================================*//**
		* URLLoader.load() の呼び出しによってセキュリティサンドボックスの外部にあるサーバーからデータをロードしようとすると送出されます。
		*/
		private function _securityError( e:SecurityErrorEvent ):void {
			// イベントリスナーを解除する
			ExURLLoader( e.target ).removeAllListeners( true );
			
			// イベントを送出する
			dispatchEvent( new CommandCatchEvent( CommandCatchEvent.ERROR, false, false, this, e ) );
		}
		
		/*======================================================================*//**
		* URLLoader.load() を呼び出して HTTP 経由でデータへのアクセスを試みたときに Flash Player がその要求のステータスコードを検出して返すことが可能な環境にある場合に送出されます。
		*/
		private function _httpStatus( e:HTTPStatusEvent ):void {
		}
		
		/*======================================================================*//**
		* URLLoader.load() の呼び出し時に致命的なエラーが発生してダウンロードが終了した場合に送出されます。
		*/
		private function _ioError( e:IOErrorEvent ):void {
			// イベントリスナーを解除する
			ExURLLoader( e.target ).removeAllListeners( true );
			
			// イベントを送出する
			dispatchEvent( new CommandCatchEvent( CommandCatchEvent.ERROR, false, false, this, e ) );
		}
	}
}





