﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.commands {
	import flash.display.DisplayObjectContainer;
	import jp.progression.casts.ICastObject;
	import jp.progression.commands.ParallelList;
	import jp.progression.commands.RemoveChild;
	import jp.progression.core.commands.Command;
	import jp.progression.events.CommandCatchEvent;
	import jp.progression.events.CommandEvent;
	
	/*======================================================================*//**
	* コマンドの実行中に例外エラーが発生した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.CommandCatchEvent.EXCEPTION
	*/
	[Event( name="exception", type="jp.progression.events.CommandCatchEvent" )]
	
	/*======================================================================*//**
	* RemoveAllChild クラスは対象の DisplayObjectContainer インスタンスのディスプレイリストから ICastObject インスタンスを削除するコマンドクラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.casts.CastSprite;
	* import jp.progression.commands.AddChild;
	* import jp.progression.commands.RemoveAllChild;
	* 
	* // CastSprite を作成する
	* var cast:CastSprite = new CastSprite( "cast" );
	* 
	* // AddChild コマンドを実行する
	* new AddChild( this, cast )
	* 	.before( null, trace, [ "キャストをディスプレイリストに追加します。" ] )
	* 	.after( null, trace, [ "キャストをディスプレイリストに追加しました。" ] )
	* 	.execute();
	* 
	* // RemoveAllChild コマンドを実行する
	* new RemoveAllChild( this, cast )
	* 	.before( null, trace, [ "全てのキャストをディスプレイリストから削除します。" ] )
	* 	.after( null, trace, [ "全てのキャストをディスプレイリストから削除しました。" ] )
	* 	.execute();
	* </listing>
	*/
	public class RemoveAllChild extends Command {
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function get enabled()		:Boolean { return ( container is DisplayObjectContainer ) ? super.enabled : false; }
		
		/*======================================================================*//**
		* ICastObject インスタンスを削除する対象のディスプレイリストを含む DisplayObjectContainer インスタンスを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get container()				:DisplayObjectContainer { return _container; }
		public function set container( value		:DisplayObjectContainer ):void { _container = value; }
		private var _container						:DisplayObjectContainer;
		
		/*======================================================================*//**
		* キャストを削除する際に、alpha プロパティを使用したフェード効果を使用するかどうかを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get autoAlpha()				:Boolean { return _autoAlpha; }
		public function set autoAlpha( value		:Boolean ):void { _autoAlpha = value; }
		private var _autoAlpha						:Boolean = false;
		
		/*======================================================================*//**
		* ParallelList インスタンスを取得します。 
		*/
		private var _commandList					:ParallelList;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい RemoveAllChild インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	container	ICastObject インスタンスを削除する対象のディスプレイリストを含む DisplayObjectContainer インスタンスです。
		* @param	autoAlpha	キャストを削除する際に、alpha プロパティを使用したフェード効果を使用するかどうかです。
		* @param	delay		処理開始まで遅延時間をミリ秒で指定します。
		*/
		public function RemoveAllChild( container:DisplayObjectContainer = null, autoAlpha:Boolean = false, delay:int = 0 ) {
			super( "RemoveAllChild", delay );
			
			// 引数を設定する
			this.container = container;
			this.autoAlpha = autoAlpha;
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _executeProgress():void {
			// 初期化する
			_commandList = new ParallelList();
			_commandList.addExclusivelyEventListener( CommandEvent.COMMAND_COMPLETE, _commandComplete, false, int.MAX_VALUE, true );
			_commandList.addExclusivelyEventListener( CommandCatchEvent.EXCEPTION, _catch, false, int.MAX_VALUE, true );
			_commandList.addExclusivelyEventListener( CommandCatchEvent.ERROR, _catch, false, int.MAX_VALUE, true );
			
			// container インスタンスが ICastObject インターフェイスを実装していれば
			if ( container is ICastObject ) {
				var list:Array = ICastObject( container ).casts;
				var l:int = list.length;
				for ( var i:int = 0; i < l; i++ ) {
					_commandList.addCommand( new RemoveChild( container, list[i], autoAlpha ) );
				}
				_commandList.execute( extra );
				return;
			}
			
			// それ以外の場合
			l = container.numChildren;
			for ( i = 0; i < l; i++ ) {
				if ( !( container.getChildAt[i] is ICastObject ) ) { continue; }
				_commandList.addCommand( new RemoveChild( container, container.getChildAt[i], autoAlpha ) );
			}
			_commandList.execute( extra );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		protected override function _interruptProgress():void {
			// 実行中のコマンドがあれば
			if ( _commandList ) {
				// イベントリスナーを解除する
				_commandList.removeAllListeners( true );
				
				// 停止する
				_commandList.addExclusivelyEventListener( CommandEvent.COMMAND_INTERRUPT, _commandInterrupt, false, int.MAX_VALUE, true );
				_commandList.interrupt( extra );
				return;
			}
			
			// 停止処理を終了する
			_interruptComplete();
		}
		
		/*======================================================================*//**
		* RemoveAllChild インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい RemoveAllChild インスタンスです。
		*/
		public override function clone():Command {
			var command:Command = new RemoveAllChild( container, autoAlpha, delay );
			command.enabled = enabled;
			command.before( _beforeScorp, _beforeFunc, _beforeArgs );
			command.after( _afterScorp, _afterFunc, _afterArgs );
			return command;
		}
		
		
		
		
		
		/*======================================================================*//**
		* コマンドの処理が完了した場合に送出されます。
		*/
		private function _commandComplete( e:CommandEvent ):void {
			Command( e.target ).removeAllListeners( true );
			
			// 処理を終了する
			_executeComplete();
		}
		
		/*======================================================================*//**
		* コマンドの処理を停止した場合に送出されます。
		*/
		private function _commandInterrupt( e:CommandEvent ):void {
			Command( e.target ).removeAllListeners( true );
			
			// 停止処理を終了する
			_interruptComplete();
		}
		
		/*======================================================================*//**
		* 
		*/
		private function _catch( e:CommandCatchEvent ):void {
			Command( e.target ).removeAllListeners( true );
			
			// 処理を終了する
			_executeComplete();
		}
	}
}





