﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.core.collections {
	import flash.utils.Dictionary;
	import flash.utils.flash_proxy;
	import flash.utils.Proxy;
	import jp.nium.utils.MathUtil;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.scenes.SceneObject;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* @private
	*/
	dynamic public class SceneList extends Proxy {
		
		/*======================================================================*//**
		* @private
		*/
		private static var _internallyCalled				:Boolean = false;
		
		
		
		
		
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function get numChildren()		:int { return progression_internal::children.length; }
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function get children()		:Array { return _children; }
		private var _children								:Array;
		
		/*======================================================================*//**
		* 
		*/
		private var _parent									:SceneObject;
		
		/*======================================================================*//**
		* インデックスをキーとして SceneObject インスタンスを格納した Dictionary インスタンスを取得します。 
		*/
		private var _indexToScenes							:Dictionary;
		
		/*======================================================================*//**
		* シーン名をキーとして SceneObject インスタンスを格納した Dictionary インスタンスを取得します。 
		*/
		private var _nameToScenes							:Dictionary;
		
		/*======================================================================*//**
		* SceneObject インスタンスをキーとしてインデックスを格納した Dictionary インスタンスを取得します。 
		*/
		private var _sceneToIndexs							:Dictionary;
		
		
		
		
		
		/*======================================================================*//**
		* @private
		*/
		public function SceneList( parent:SceneObject ) {
			// パッケージ外から呼び出されたらエラーを送出する
			if ( !_internallyCalled ) { throw new ArgumentError( "Error #2012: SceneList クラスを直接インスタンス化することはできません。" ); }
			
			// 引数を設定する
			_parent = parent;
			
			// 初期化する
			_children = [];
			_indexToScenes = new Dictionary( true );
			_nameToScenes = new Dictionary( true );
			_sceneToIndexs = new Dictionary( true );
			_internallyCalled = false;
		}
		
		
		
		
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal static function createInstance( parent:SceneObject ):SceneList {
			_internallyCalled = true;
			return new SceneList( parent );
		}
		
		
		
		
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function sort():void {
			// 全てのインデックス値を取得する
			var list:Array = [];
			var l:int = _children.length;
			for ( var i:int = 0; i < l; i++ ) {
				list.push( {
					child	:_children[i],
					index	:_sceneToIndexs[_children[i]]
					} );
			}
			
			// インデックス値でソートする
			list.sortOn( "index", Array.NUMERIC );
			
			// 配列を初期化する
			_children = [];
			
			// ソート済みの配列をコピーする
			l = list.length;
			for ( i = 0; i < l; i++ ) {
				_children.push( list[i].child );
			}
		}
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function addChild( child:SceneObject ):SceneObject {
			// 自分自身を追加しようとしたらエラーを送出する
			if ( child == _parent ) { throw new ArgumentError( "Error #2024: インスタンスをそれ自体の子として追加することはできません。" ); }
			
			// 対象が自身の親であればエラーを送出する。
			var target:SceneObject = _parent;
			while ( target && target != target.public::parent ) {
				if ( target == child ) { throw new ArgumentError( "Error #2150: インスタンスをそれ自体の子 (またはその子の子など) として追加することはできません。" ); }
				
				// 親に遡る
				target = target.public::parent;
			}
			
			// すでに登録されていれば削除する
			if ( child.public::parent && child.public::parent.contains( child ) ) {
				child.public::parent.removeChild( child );
			}
			
			// すでに同じシーン名の SceneObject が存在すれば削除する
			if ( _nameToScenes[child.name] ) {
				_parent.removeChild( _nameToScenes[child.name] );
			}
			
			// 最高値のインデックス位置を取得する
			var index:int = _children.length ? _sceneToIndexs[_children[_children.length - 1]] + 1 : 0;
			
			// 登録する
			_indexToScenes[index] = child;
			_nameToScenes[child.name] = child;
			_sceneToIndexs[child] = index;
			_children.push( child );
			
			return child;
		}
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function addChildAt( child:SceneObject, index:int, addAbove:Boolean = false ):SceneObject {
			// 自分自身を追加しようとしたらエラーを送出する
			if ( child == _parent ) { throw new ArgumentError( "Error #2024: インスタンスをそれ自体の子として追加することはできません。" ); }
			
			// 対象が自身の親であればエラーを送出する。
			var target:SceneObject = _parent;
			while ( target && target != target.public::parent ) {
				if ( target == child ) { throw new ArgumentError( "Error #2150: インスタンスをそれ自体の子 (またはその子の子など) として追加することはできません。" ); }
				
				// 親に遡る
				target = target.public::parent;
			}
			
			// すでに登録されていれば削除する
			if ( child.public::parent && child.public::parent.contains( child ) ) {
				child.public::parent.removeChild( child );
			}
			
			// すでに同じシーン名の SceneObject が存在すれば削除する
			if ( _nameToScenes[child.name] ) {
				_parent.removeChild( _nameToScenes[child.name] );
			}
			
			// 0 以上に数値を丸める
			index = Math.max( 0, index );
			
			// 指定インデックス位置にすでに登録済みであれば以降のインデックスを上にずらす
			if ( _indexToScenes[index] is SceneObject ) {
				// 仮想インデックス値を取得する
				var virtualIndex:int = _sceneToIndexs[_indexToScenes[index]];
				
				// 対象の上に配置する
				if ( addAbove ) {
					virtualIndex++;
					index++;
				}
				
				// インデックス位置を走査する
				var l:int = _children.length;
				for ( var i:int = virtualIndex; i < l; i++ ) {
					var chd1:SceneObject = _indexToScenes[Math.max( virtualIndex, i - 1 )];
					var chd2:SceneObject = _indexToScenes[i];
					
					// ひとつ前のインデックス位置と重複していなければ終了する
					if ( _sceneToIndexs[chd1] != _sceneToIndexs[chd2] ) { break; }
					
					// 値を 1 つずらす
					var idx:int = _sceneToIndexs[chd2] + 1;
					_sceneToIndexs[chd2] = idx;
					_indexToScenes[idx] = chd2;
				}
			}
			
			// データを格納する
			_indexToScenes[index] = child;
			_nameToScenes[child.name] = child;
			_sceneToIndexs[child] = index;
			_children.push( child );
			
			// インデックス値でソートする
			progression_internal::sort();
			
			return child;
		}
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function removeChild( child:SceneObject ):SceneObject {
			// 子に登録されていない SceneObject インスタンスであればエラーを送出する
			if ( !( _sceneToIndexs[child] is Number ) ) { throw new ArgumentError( "Error #2025: 指定した DisplayObject は呼び出し元の子でなければなりません。" ); }
			
			// 削除する
			delete _indexToScenes[_sceneToIndexs[child]];
			delete _nameToScenes[child.name];
			delete _sceneToIndexs[child];
			
			var l:int = _children.length;
			for ( var i:int = 0; i < l; i++ ) {
				if ( _children[i] != child ) { continue; }
				
				// 削除する
				_children.splice( i, 1 );
				break;
			}
			
			return child;
		}
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function removeChildAt( index:int ):SceneObject {
			return progression_internal::removeChild( _indexToScenes[index] );
		}
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function contains( child:SceneObject ):Boolean {
			// 自身であれば true を返す
			if ( _parent == child ) { return true; }
			
			// 存在すれば true を返す
			if ( _sceneToIndexs[child] is int ) { return true; }
			
			// 子に存在すれば true を返す
			for each ( var scene:SceneObject in _indexToScenes ) {
				if ( scene.contains( child ) ) { return true; }
			}
			
			return false;
		}
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function getChildAt( index:int ):SceneObject {
			return _indexToScenes[index];
		}
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function getChildByName( name:String ):SceneObject {
			return _nameToScenes[name];
		}
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function getNextChild( child:SceneObject ):SceneObject {
			var l:int = _children.length;
			for ( var i:int = 0; i < l; i++ ) {
				if ( _children[i] == child ) { return _children[MathUtil.cycle( i + 1, l )]; }
			}
			
			return null;
		}
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function getPreviousChild( child:SceneObject ):SceneObject {
			var l:int = _children.length;
			for ( var i:int = 0; i < l; i++ ) {
				if ( _children[i] == child ) { return _children[MathUtil.cycle( i - 1, l )]; }
			}
			
			return null;
		}
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function getChildIndex( child:SceneObject ):int {
			return _sceneToIndexs[child];
		}
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function setChildIndex( child:SceneObject, index:int, addAbove:Boolean = false ):void {
			// 位置を再設定する
			progression_internal::addChildAt( progression_internal::removeChild( child ), index, addAbove )
		}
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function swapChildren( child1:SceneObject, child2:SceneObject ):void {
			// インデックス位置を格納する
			var index1:int = _sceneToIndexs[child1];
			var index2:int = _sceneToIndexs[child2];
			
			// シーンリストから削除する
			progression_internal::removeChild( child1 );
			progression_internal::removeChild( child2 );
			
			// シーンリストに追加する
			progression_internal::addChildAt( child1, index2 );
			progression_internal::addChildAt( child2, index1 );
		}
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function swapChildrenAt( index1:int, index2:int ):void {
			// ディスプレイオブジェクトを格納する
			var child1:SceneObject = _indexToScenes[index1];
			var child2:SceneObject = _indexToScenes[index2];
			
			// ディスプレイリストから削除する
			progression_internal::removeChild( child1 );
			progression_internal::removeChild( child2 );
			
			// ディスプレイリストに追加する
			progression_internal::addChildAt( child1, index2 );
			progression_internal::addChildAt( child2, index1 );
		}
		
		/*======================================================================*//**
		* @private
		*/
		flash_proxy override function callProperty( methodName:*, ... args:Array ):* {
		}
		
		/*======================================================================*//**
		* @private
		*/
		flash_proxy override function getProperty( name:* ):* {
			return _nameToScenes[name];
		}
		
		/*======================================================================*//**
		* @private
		*/
		flash_proxy override function setProperty( name:*, value:* ):void {
		}
		
		/*======================================================================*//**
		* @private
		*/
		flash_proxy override function deleteProperty( name:* ):Boolean {
			return false;
		}
		
		/*======================================================================*//**
		* @private
		*/
		flash_proxy override function nextName( index:int ):String {
			return SceneObject( _children[index - 1] ).name;
		}
		
		/*======================================================================*//**
		* @private
		*/
		flash_proxy override function nextNameIndex( index:int ):int {
			if ( index < _children.length ) { return index + 1; }
			
			return 0;
		}
		
		/*======================================================================*//**
		* @private
		*/
		flash_proxy override function nextValue( index:int ):* {
			return _children[index - 1];
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public function toString():String {
			return "[object SceneList]";
		}
	}
}





