﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.core.commands {
	import flash.events.Event;
	import flash.utils.Dictionary;
	import flash.utils.getQualifiedClassName;
	import flash.utils.getQualifiedSuperclassName;
	import jp.progression.core.commands.Command;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.events.CommandCatchEvent;
	import jp.progression.events.CommandEvent;
	import jp.progression.Progression;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* CommandList クラスを直接インスタンス化することはできません。
	* new CommandList() コンストラクタを呼び出すと、ArgumentError 例外がスローされます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class CommandList extends Command {
		
		/*======================================================================*//**
		* 登録されているコマンド配列を取得します。
		* この配列を操作することで元の配列を変更することはできません。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get commands()							:Array { return _commands.concat(); }
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function get commands()			:Array { return _commands; }
		progression_internal function set commands( value		:Array ):void { _commands = value; }
		private var _commands									:Array;
		
		/*======================================================================*//**
		* 登録されているコマンドの数を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get length()							:int { return _commands.length; }
		
		/*======================================================================*//**
		* Dictionary インスタンスを取得します。 
		*/
		private var _exceptionList								:Dictionary;
		
		/*======================================================================*//**
		* Dictionary インスタンスを取得します。 
		*/
		private var _eventList									:Dictionary;
		
		/*======================================================================*//**
		* CommandEvent インスタンスを取得します。 
		*/
		private var _commandEvent								:CommandEvent;
		
		/*======================================================================*//**
		* Command インスタンスを取得します。 
		*/
		private var _currentCommand								:Command;
		
		/*======================================================================*//**
		* Command インスタンスを取得します。 
		*/
		private var _finallyCommand								:Command;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい CommandList インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	type			コマンドの種類です。
		* @param	delay			コマンド実行時の遅延時間です。
		* @param	commands		登録したいコマンドを格納した配列です。
		* @throws	ArgumentError	CommandList クラスを直接インスタンス化することはできません。
		*/
		public function CommandList( type:String, delay:int = 0, commands:Array = null ) {
			super( type, delay );
			
			// 継承せずにオブジェクト化されたらエラーを送出する
			if ( getQualifiedSuperclassName( this ) == getQualifiedClassName( Command ) ) { throw new ArgumentError( "Error #2012: CommandList クラスを直接インスタンス化することはできません。" ); }
			
			// 初期化する
			_commands = [];
			_exceptionList = new Dictionary( true );
			_eventList = new Dictionary( true );
			
			// コマンドを追加する
			var l:int = commands.length;
			for ( var i:int = 0; i < l; i++ ) {
				addCommand( commands[i] );
			}
		}
		
		
		
		
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	e	説明文
		*/
		protected function _executeCatch( e:CommandCatchEvent ):void {
			Progression.verbose.log( this, null, 1, "<" + type + " : " + name + "> コマンドが " + e.type + " イベントをキャッチ" );
			
			// コマンドを取得する
			switch ( e.type ) {
				case CommandCatchEvent.EXCEPTION	: { _currentCommand = Command( _exceptionList[e.relatedObject != null ? e.relatedObject.constructor : null] ); break; }
				case CommandCatchEvent.ERROR		: { _currentCommand = Command( _eventList[e.relatedObject != null ? Event(e.relatedObject).type : null] ); break; }
			}
			
			// コマンドが存在すれば実行する
			if ( _currentCommand ) {
				_commandEvent = e;
				_currentCommand.addExclusivelyEventListener( CommandEvent.COMMAND_COMPLETE, _complete, false, int.MAX_VALUE, true );
				_currentCommand.addExclusivelyEventListener( CommandCatchEvent.EXCEPTION, _catch, false, int.MAX_VALUE, true );
				_currentCommand.addExclusivelyEventListener( CommandCatchEvent.ERROR, _catch, false, int.MAX_VALUE, true );
				_currentCommand.execute( extra );
				return;
			}
			
			// イベントを送出する
			_executeFinally( e );
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	e	説明文
		*/
		protected function _executeFinally( e:CommandEvent ):void {
			// コマンドが存在すれば実行する
			if ( _finallyCommand ) {
				Progression.verbose.log( this, null, 1, "<" + type + " : " + name + "> コマンドの finally 処理を実行" );
				
				// イベントオブジェクトを保存する
				_commandEvent = e;
				
				// コマンドを実行する
				_finallyCommand.addExclusivelyEventListener( CommandEvent.COMMAND_COMPLETE, _finallyComplete, false, int.MAX_VALUE, true );
				_finallyCommand.addExclusivelyEventListener( CommandCatchEvent.EXCEPTION, _catch, false, int.MAX_VALUE, true );
				_finallyCommand.addExclusivelyEventListener( CommandCatchEvent.ERROR, _catch, false, int.MAX_VALUE, true );
				_finallyCommand.execute( extra );
				return;
			}
			
			// イベントを送出する
			_executeFinallyComplete( e );
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	e	説明文
		*/
		protected function _executeFinallyComplete( e:CommandEvent ):void {
			// 事後処理を実行する
			if ( _afterFunc is Function ) {
				_afterFunc.apply( _afterScorp, _afterArgs );
			}
			
			// イベントを送出する
			dispatchEvent( e );
		}
		
		/*======================================================================*//**
		* コマンドを追加します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	commands	追加したいコマンド配列です。
		* @return				自身の参照です。
		*/
		public function addCommand( ...commands:Array ):CommandList {
			var l:int = commands.length;
			for ( var i:int = 0; i < l; i++ ) {
				var command:Command = Command( commands[i] );
				
				// 存在しなければ次へ
				if ( !command ) { break; }
				
				// 関連性を設定する
				command.progression_internal::parent = this;
				command.progression_internal::depth = progression_internal::depth + 1;
				
				// リストに追加する
				_commands.push( command );
			}
			
			return this;
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	exception	説明文
		* @param	command		説明文
		* @return				自身の参照です。
		*/
		public function catchException( exception:Class, command:Command ):CommandList {
			// データを設定する
			_exceptionList[exception] = command;
			_exceptionList[command] = exception;
			return this;
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	eventType	説明文
		* @param	command		説明文
		* @return				自身の参照です。
		*/
		public function catchEvent( eventType:String, command:Command ):CommandList {
			// データを設定する
			_eventList[eventType] = command;
			_eventList[command] = eventType;
			return this;
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	command		説明文
		* @return				自身の参照です。
		*/
		public function setFinally( command:Command ):CommandList {
			_finallyCommand = command;
			return this;
		}
		
		/*======================================================================*//**
		* CommandList インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい CommandList インスタンスです。
		*/
		public override function clone():Command {
			var command:CommandList = new CommandList( name, delay );
			command.enabled = enabled;
			command.before( _beforeScorp, _beforeFunc, _beforeArgs );
			command.after( _afterScorp, _afterFunc, _afterArgs );
			
			// コマンドリストを複製する
			var l:int = commands.length;
			for ( var i:int = 0; i < l; i++ ) {
				command.addCommand( commands[i] );
			}
			
			return Command( command );
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object CommandList : " + name + "]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* コマンドの処理が完了した場合に送出されます。
		*/
		private function _complete( e:CommandEvent ):void {
			Command( e.target ).removeAllListeners( true );
			
			// 処理を終了する
			_executeFinally( _commandEvent );
		}
		
		/*======================================================================*//**
		* コマンドの処理が完了した場合に送出されます。
		*/
		private function _finallyComplete( e:CommandEvent ):void {
			Command( e.target ).removeAllListeners( true );
			
			// 処理を終了する
			_executeFinallyComplete( _commandEvent );
		}
		
		/*======================================================================*//**
		* コマンド処理中にエラーが発生した場合に送出されます。
		*/
		private function _catch( e:CommandCatchEvent ):void {
			Command( e.target ).removeAllListeners( true );
			
			// イベントを送出する
			dispatchEvent( e );
		}
	}
}





