﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.core.components.presets {
	import flash.display.MovieClip;
	import flash.events.Event;
	import flash.ui.ContextMenu;
	import jp.nium.utils.ClassUtil;
	import jp.progression.casts.ICastObject;
	import jp.progression.casts.presets.FadeMovie;
	import jp.progression.core.commands.CommandList;
	import jp.progression.core.display.ToolTip;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.events.CastEvent;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* 
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class FadeMovieComponent extends FadeMovie {
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="ツールチップの内容", defaultValue="" )]
		public function get toolTipText()						:String { return super.toolTip.text; }
		public function set toolTipText( value					:String ):void { super.toolTip.text = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="ツールチップ文字色", type="Color", defaultValue="#000000", verbose="1" )]
		public function get toolTipTextColor()					:uint { return super.toolTip.textColor; }
		public function set toolTipTextColor( value				:uint ):void { super.toolTip.textColor = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="ツールチップフォント", type="Font Name", defaultValue="_ゴシック", verbose="1" )]
		public function get toolTipFont()						:String { return super.toolTip.textFont; }
		public function set toolTipFont( value					:String ):void { super.toolTip.textFont = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="ツールチップ背景色", defaultValue="#FFFFFF", type="Color", verbose="1" )]
		public function get toolTipBackgroundColor()			:uint { return super.toolTip.backgroundColor; }
		public function set toolTipBackgroundColor( value		:uint ):void { super.toolTip.backgroundColor = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="ツールチップ線色", type="Color", defaultValue="#CCCCCC", verbose="1" )]
		public function get toolTipBorderColor()				:uint { return super.toolTip.borderColor; }
		public function set toolTipBorderColor( value			:uint ):void { super.toolTip.borderColor = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="ツールチップの遅延", type="Number", defaultValue="750", verbose="1" )]
		public function get toolTipDelay()						:int { return super.toolTip.delay; }
		public function set toolTipDelay( value					:int ):void { super.toolTip.delay = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="ツールチップの追従", type="Boolean", defaultValue="true", verbose="1" )]
		public function get toolTipFollowing()					:Boolean { return super.toolTip.following; }
		public function set toolTipFollowing( value				:Boolean ):void { super.toolTip.following = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="コンテクストメニュー", type="Boolean", defaultValue="true", verbose="1" )]
		public function get useContextMenu()					:Boolean { return _useContextMenu; }
		public function set useContextMenu( value				:Boolean ):void {
			_useContextMenu = value;
			
			// 対象が自分であれば
			if ( _target == this ) {
				uiContextMenu.enabled = value;
				return;
			}
			
			// 親が対象であれば
			if ( value ) {
				_target.contextMenu = progression_internal::contextMenu;
			}
			else {
				_target.contextMenu = new ContextMenu();
				_target.contextMenu.hideBuiltInItems();
			}
		}
		private var _useContextMenu								:Boolean = true;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="種類", type="List", enumeration="in,out,inOut", defaultValue="inOut" )]
		public override function get direction()				:String { return super.direction; }
		public override function set direction( value			:String ):void { super.direction = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="長さ（ミリ秒）", defaultValue="1000" )]
		public override function get duration()					:int { return super.duration; }
		public override function set duration( value			:int ):void { super.duration = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="透明マルチ", defaultValue="0", verbose="1" )]
		public override function get alphaMultiplier()			:int { return super.alphaMultiplier; }
		public override function set alphaMultiplier( value		:int ):void { super.alphaMultiplier = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="透明オフセット", defaultValue="0", verbose="1" )]
		public override function get alphaOffset()				:int { return super.alphaOffset; }
		public override function set alphaOffset( value			:int ):void { super.alphaOffset = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="青マルチ", defaultValue="1", verbose="1" )]
		public override function get blueMultiplier()			:int { return super.blueMultiplier; }
		public override function set blueMultiplier( value		:int ):void { super.blueMultiplier = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="青オフセット", defaultValue="0", verbose="1" )]
		public override function get blueOffset()				:int { return super.blueOffset; }
		public override function set blueOffset( value			:int ):void { super.blueOffset = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="緑マルチ", defaultValue="1", verbose="1" )]
		public override function get greenMultiplier()			:int { return super.greenMultiplier; }
		public override function set greenMultiplier( value		:int ):void { super.greenMultiplier = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="緑オフセット", defaultValue="0", verbose="1" )]
		public override function get greenOffset()				:int { return super.greenOffset; }
		public override function set greenOffset( value			:int ):void { super.greenOffset = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="赤マルチ", defaultValue="1", verbose="1" )]
		public override function get redMultiplier()			:int { return super.redMultiplier; }
		public override function set redMultiplier( value		:int ):void { super.redMultiplier = value; }
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		[Inspectable( name="赤オフセット", defaultValue="0", verbose="1" )]
		public override function get redOffset()				:int { return super.redOffset; }
		public override function set redOffset( value			:int ):void { super.redOffset = value; }
		
		
		
		
		
		/*======================================================================*//**
		* 新しい FadeMovieComponent インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function FadeMovieComponent() {
			// 表示を削除する
			setProperties( {
				x			:0,
				y			:0,
				width		:0,
				height		:0
			} );
			
			// イベントリスナーを登録する
			addExclusivelyEventListener( Event.ADDED_TO_STAGE, _addedToStage, false, int.MAX_VALUE, true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object FadeMovieComponent : " + ( parent ? parent.name : "null" ) + "]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* 
		*/
		private function _addedToStage( e:Event ):void {
			// ICastObject インターフェイスを実装していれば終了する
			if ( parent is ICastObject ) {
				trace( parent, "は、すでに ICastObject インターフェイスを実装している為、コンポーネントは無効化されます" );
				return;
			}
			
			// MovieClip クラスでなければ終了する
			if ( !( parent is MovieClip ) ) {
				trace( parent, "は、MovieClip クラスを継承していない為、コンポーネントは無効化されます" );
				return;
			}
			
			// dynamic クラスでなければ終了する
			if ( !ClassUtil.isDynamic( parent ) ) {
				trace( parent, "は、dynamic クラスではない為、コンポーネントは無効化されます" );
				return;
			}
			
			// ターゲットを親に設定する
			_target = MovieClip( parent );
			
			// ターゲットを初期化する
			_target.progression_internal_clearCommand = clearCommand;
			_target.progression_internal_getCommand = function():CommandList { return progression_internal::commandList; }
			_target.contextMenu = progression_internal::contextMenu;
			
			// イベントリスナーを接続する
			_target.addEventListener( CastEvent.CAST_ADDED, dispatchEvent, false, int.MAX_VALUE, true );
			_target.addEventListener( CastEvent.CAST_REMOVED, dispatchEvent, false, int.MAX_VALUE, true );
		}
	}
}





