﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.core.debug {
	import flash.errors.IllegalOperationError;
	import jp.nium.events.EventIntegrator;
	import jp.nium.utils.StringUtil;
	import jp.progression.casts.CastButton;
	import jp.progression.casts.ICastObject;
	import jp.progression.core.commands.Command;
	import jp.progression.core.commands.CommandList;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.Progression;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* Verbose クラスは、開発者用の出力機能を制御するデバッグクラスです。
	* <p />
	* Verbose クラスを直接インスタンス化することはできません。
	* new Verbose() コンストラクタを呼び出すと、ArgumentError 例外がスローされます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class Verbose {
		
		/*======================================================================*//**
		* @private
		*/
		private static var _internallyCalled			:Boolean = false;
		
		/*======================================================================*//**
		* 開発者用の出力機能が使用可能かどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get enabled()					:Boolean { return _enabled; }
		public function set enabled( value				:Boolean ):void { _enabled = value; }
		private var _enabled							:Boolean = false;
		
		/*======================================================================*//**
		* @private
		*/
		public function get enable()					:Boolean { throw new IllegalOperationError( "Verbose クラスの enable プロパティは、他の仕様と統一するため enabled プロパティに変更されました。\n同様の操作をする場合、今後は enabled プロパティを使用してください。" ); }
		public function set enable( value				:Boolean ):void { throw new IllegalOperationError( "Verbose クラスの enable プロパティは、他の仕様と統一するため enabled プロパティに変更されました。\n同様の操作をする場合、今後は enabled プロパティを使用してください。" ); }
		
		/*======================================================================*//**
		* Progression クラスを継承したサブクラスをフィルタリング対象とするかどうかを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get progression()				:Boolean { return hasFilter( Progression ); }
		public function set progression( value			:Boolean ):void {
			if ( value ) {
				addFilter( Progression );
			}
			else {
				removeFilter( Progression );
			}
		}
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get loggingFunction()			:Function { return _loggingFunction || trace; }
		public function set loggingFunction( value		:Function ):void { _loggingFunction = value; }
		private var _loggingFunction					:Function;
		
		/*======================================================================*//**
		* Command クラスを継承したサブクラスをフィルタリング対象とするかどうかを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get filterCommand()				:Boolean { return hasFilter( Command ); }
		public function set filterCommand( value		:Boolean ):void {
			if ( value ) {
				addFilter( Command );
			}
			else {
				removeFilter( Command );
			}
		}
		
		/*======================================================================*//**
		* CommandList クラスを継承したサブクラスをフィルタリング対象とするかどうかを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get filterCommandList()			:Boolean { return hasFilter( CommandList ); }
		public function set filterCommandList( value	:Boolean ):void {
			if ( value ) {
				addFilter( CommandList );
			}
			else {
				removeFilter( CommandList );
			}
		}
		
		/*======================================================================*//**
		* ICastObject インターフェイスを実装したクラスをフィルタリング対象とするかどうかを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get filterCastObject()			:Boolean { return hasFilter( ICastObject ); }
		public function set filterCastObject( value		:Boolean ):void {
			if ( value ) {
				addFilter( ICastObject );
			}
			else {
				removeFilter( ICastObject );
			}
		}
		
		/*======================================================================*//**
		* CastButton クラスを継承したサブクラスをフィルタリング対象とするかどうかを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get filterCastButton()			:Boolean { return hasFilter( CastButton ); }
		public function set filterCastButton( value		:Boolean ):void {
			if ( value ) {
				addFilter( CastButton );
			}
			else {
				removeFilter( CastButton );
			}
		}
		
		/*======================================================================*//**
		*/
		private var _filters							:Array;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい Verbose インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @throws	ArgumentError	Verbose クラスを直接インスタンス化することはできません。
		*/
		public function Verbose() {
			// パッケージ外から呼び出されたらエラーを送出する
			if ( !_internallyCalled ) { throw new ArgumentError( "Error #2012: Verbose クラスを直接インスタンス化することはできません。" ); }
			
			// 初期化する
			_filters = [];
			_internallyCalled = false;
		}
		
		
		
		
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal static function createInstance():Verbose {
			_internallyCalled = true;
			return new Verbose();
		}
		
		
		
		
		
		/*======================================================================*//**
		* 開発者用の出力を行います。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	target				出力を実行する対象オブジェクトです。
		* @param	status				説明文
		* @param	indent				説明文
		* @param	message				出力したい String インスタンスです。
		* @param	separateBefore		出力直前にセパレータを表示するかどうかです。
		*/
		public function log( target:* = null, status:String = null, indent:int = 0, message:* = null, separateBefore:Boolean = false ):void {
			if ( !enabled ) { return; }
			
			// 初期値を設定する
			status = status ? status : "　";
			
			// フィルタリング対象が設定されていれば終了する
			var l:int = _filters.length;
			for ( var i:int = 0; i < l; i++ ) {
				if ( target is _filters[i] ) { return; }
			}
			
			// セパレータを表示する
			if ( separateBefore ) {
				loggingFunction( "\n----------------------------------------------------------------------" );
			}
			
			// 出力する
			loggingFunction( "PROG | " + status + " | " + StringUtil.repeat( "  ", indent ) + String( message ) );
		}
		
		/*======================================================================*//**
		* 対象クラスオブジェクトを出力時のフィルタリング対象として登録します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	target	フィルタリング対象として登録するクラスオブジェクトです。
		*/
		public function addFilter( target:Class ):void {
			// すでに登録されていれば終了する
			if ( hasFilter( target ) ) { return; }
			
			// リストに登録する
			_filters.push( target );
		}
		
		/*======================================================================*//**
		* 対象クラスオブジェクトを出力時のフィルタリング対象から除外します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	target	フィルタリング対象から除外したいクラスオブジェクトです。
		*/
		public function removeFilter( target:Class ):void {
			var l:int = _filters.length;
			for ( var i:int = 0; i < l; i++ ) {
				if ( _filters[i] == target ) {
					// リストから削除する
					_filters.splice( i, 1 );
					return;
				}
			}
		}
		
		/*======================================================================*//**
		* 対象クラスオブジェクトを出力時のフィルタリング対象として登録されているかどうかを返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	target	フィルタリング対象として登録されているかテストしたいクラスオブジェクトです。
		* @return			フィルタリング対象に登録されていれば true に、それ以外の場合は false になります。
		*/
		public function hasFilter( target:Class ):Boolean {
			var l:int = _filters.length;
			for ( var i:int = 0; i < l; i++ ) {
				if ( _filters[i] == target ) { return true; }
			}
			return false;
		}
		
		
		
		
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public function toString():String {
			return "[object Verbose]";
		}
	}
}





