﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.core.display {
	import flash.display.DisplayObject;
	import flash.display.Sprite;
	import flash.errors.IllegalOperationError;
	import flash.events.Event;
	import jp.nium.utils.StageUtil;
	import jp.progression.casts.CastSprite;
	import jp.progression.core.namespaces.progression_internal;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* 
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class Background extends CastSprite {
		
		/*======================================================================*//**
		* @private
		*/
		private static var _internallyCalled		:Boolean = false;
		
		
		
		
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get enabled()				:Boolean { return _enabled; }
		public function set enabled( value			:Boolean ):void {
			_enabled = value;
			
			if ( _enabled ) {
				// 矩形を描画する
				graphics.beginFill( 0x000000, 0 );
				graphics.drawRect( 0, 0, 100, 100 );
				graphics.endFill();
				
				if ( stage && stage.loaderInfo ) {
					// イベントリスナーを登録する
					stage.loaderInfo.addEventListener( Event.COMPLETE, _complete, false, int.MAX_VALUE, true );
				}
				
				if ( stage ) {
					// イベントリスナーを登録する
					stage.addEventListener( Event.RESIZE, _resize, false, int.MAX_VALUE, true );
				}
				
				// イベントリスナーを登録する
				addExclusivelyEventListener( Event.ADDED_TO_STAGE, _addedToStage, false, int.MAX_VALUE, true );
				addExclusivelyEventListener( Event.REMOVED_FROM_STAGE, _removedFromStage, false, int.MAX_VALUE, true );
				
				// 画面を更新する
				_update();
			}
			else {
				// グラフィックを削除する
				graphics.clear();
				
				if ( stage && stage.loaderInfo ) {
					// イベントリスナーを解除する
					stage.loaderInfo.removeEventListener( Event.COMPLETE, _complete );
				}
				
				if ( stage ) {
					// イベントリスナーを解除する
					stage.removeEventListener( Event.RESIZE, _resize );
				}
				
				// イベントリスナーを解除する
				completelyRemoveEventListener( Event.ADDED_TO_STAGE, _addedToStage );
				completelyRemoveEventListener( Event.REMOVED_FROM_STAGE, _removedFromStage );
			}
		}
		private var _enabled						:Boolean = true;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい Background インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @throws	ArgumentError	Background クラスを直接インスタンス化することはできません。
		*/
		public function Background() {
			// パッケージ外から呼び出されたらエラーを送出する
			if ( !_internallyCalled ) { throw new ArgumentError( "Error #2012: Background クラスを直接インスタンス化することはできません。" ); }
			
			// 初期化する
			enabled = true;
			_internallyCalled = false;
		}
		
		
		
		
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal static function createInstance():Background {
			_internallyCalled = true;
			return new Background();
		}
		
		
		
		
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object Background]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* 
		*/
		private function _update():void {
			if ( !stage ) { return; }
			
			// ドキュメントクラスの参照を取得する
			var root:Sprite = StageUtil.getDocument( stage );
			
			if ( !root || !root.loaderInfo ) { return; }
			
			x = -StageUtil.getMarginLeft( stage );
			y = -StageUtil.getMarginTop( stage );
			
			width = stage.stageWidth;
			height = stage.stageHeight;
		}
		
		
		
		
		
		/*======================================================================*//**
		* 
		*/
		private function _addedToStage( e:Event ):void {
			enabled = enabled;
		}
		
		/*======================================================================*//**
		* 
		*/
		private function _removedFromStage( e:Event ):void {
			throw new IllegalOperationError( "このインスタンスを stage のディスプレイリストから削除することはできません。" );
		}
		
		/*======================================================================*//**
		* 
		*/
		private function _resize( e:Event ):void {
			_update();
		}
		
		/*======================================================================*//**
		* 
		*/
		private function _complete( e:Event ):void {
			enabled = enabled;
		}
	}
}





