﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.core.managers {
	import com.asual.swfaddress.SWFAddress;
	import jp.nium.events.EventIntegrator;
	import jp.nium.external.Browser;
	import jp.progression.core.managers.HistoryManagerType;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.Progression;
	import jp.progression.scenes.SceneId;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* 
	* HistoryManager クラスを直接インスタンス化することはできません。
	* new HistoryManager() コンストラクタを呼び出すと、ArgumentError 例外がスローされます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class HistoryManager extends EventIntegrator {
		
		/*======================================================================*//**
		* ヒストリー機能が使用可能かどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get enabled()		:Boolean { return true; }
		
		/*======================================================================*//**
		* 履歴を管理している対象タイプを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function get type()			:String { return Browser.enabled ? HistoryManagerType.BROWSER : HistoryManagerType.FLASH; }
		
		/*======================================================================*//**
		* 履歴配列を取得します。
		*/
		/*
		private static var _histories				:Array = [];
		*/
		
		/*======================================================================*//**
		* 現在の履歴位置を取得します。
		*/
		/*
		private static var _currentCount			:int = 0;
		*/
		
		
		
		
		
		/*======================================================================*//**
		* @private
		*/
		public function HistoryManager() {
			throw new ArgumentError( "Error #2012: HistoryManager クラスを直接インスタンス化することはできません。" );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 履歴を追加します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	sceneId		履歴に追加したいシーン識別子です。
		*/
		/*
		public static function addHistory( sceneId:SceneId ):void {
			// 現在位置から後の履歴を削除する
			_histories.splice( _currentCount + 1 );
			
			// 履歴に追加する
			_histories.push( sceneId );
			
			// 現在位置を移動する
			_currentCount = _histories.length - 1;
		}
		*/
		
		/*======================================================================*//**
		* 全ての履歴を削除します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		/*
		public static function clearHistory():void {
			_histories = [];
			_currentCount = 0;
		}
		*/
		
		/*======================================================================*//**
		* 履歴が存在するかどうかを返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	指定した位置に履歴が存在する場合は true に、それ以外の場合は false になります。
		*/
		public static function hasHistory( position:int ):Boolean {
			// ブラウザ通信が有効であれば、ブラウザの履歴を使用する
			if ( Browser.enabled ) { return true; }
			
			/*
			return Boolean( _histories[_currentCount + position] );
			*/
			
			return false;
		}
		
		/*======================================================================*//**
		* 履歴を 1 つ先に進めます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function forward():void {
			// ブラウザ通信が有効であれば、ブラウザの履歴を使用する
			if ( Browser.enabled ) {
				SWFAddress.forward();
				return;
			}
			
			/*
			_currentCount = Math.min( _currentCount + 1, _histories.length - 1 );
			Progression.goto( _histories[_currentCount] );
			*/
		}
		
		/*======================================================================*//**
		* 履歴を 1 つ前に戻します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static function back():void {
			// ブラウザ通信が有効であれば、ブラウザの履歴を使用する
			if ( Browser.enabled ) {
				SWFAddress.back();
				return;
			}
			
			/*
			_currentCount = Math.max( 0, _currentCount - 1 );
			Progression.goto( _histories[_currentCount] );
			*/
		}
		
		
		
		
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object HistoryManager]";
		}
	}
}





