﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.core.managers {
	import flash.utils.getDefinitionByName;
	import jp.nium.events.EventIntegrator;
	import jp.progression.core.commands.Command;
	import jp.progression.core.commands.CommandList;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.events.CommandCatchEvent;
	import jp.progression.events.CommandEvent;
	import jp.progression.events.ProcessEvent;
	import jp.progression.events.SceneEvent;
	import jp.progression.Progression;
	import jp.progression.scenes.SceneId;
	import jp.progression.scenes.SceneObject;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* シーン移動処理が開始された場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.ProcessEvent.PROCESS_START
	*/
	[Event( name="processStart", type="jp.progression.events.ProcessEvent" )]
	
	/*======================================================================*//**
	* シーン移動処理が完了した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.ProcessEvent.PROCESS_COMPLETE
	*/
	[Event( name="processComplete", type="jp.progression.events.ProcessEvent" )]
	
	/*======================================================================*//**
	* シーン移動処理が停止された場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.ProcessEvent.PROCESS_INTERRUPT
	*/
	[Event( name="processInterrupt", type="jp.progression.events.ProcessEvent" )]
	
	/*======================================================================*//**
	* シーン移動処理中に対象シーンが変更された場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.ProcessEvent.PROCESS_SCENE
	*/
	[Event( name="processScene", type="jp.progression.events.ProcessScene" )]
	
	/*======================================================================*//**
	* シーン移動処理中に対象シーンでイベントが発生した場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.ProcessEvent.PROCESS_EVENT
	*/
	[Event( name="processEvent", type="jp.progression.events.ProcessEvent" )]
	
	/*======================================================================*//**
	* シーン移動処理中に移動先のシーンが存在しなかった場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.ProcessEvent.PROCESS_ERROR
	*/
	[Event( name="processError", type="jp.progression.events.ProcessEvent" )]
	
	/*======================================================================*//**
	* SceneManager クラスは、シーンの移動とコマンドの実行を制御する為のマネージャクラスです。
	* <p />
	* SceneManager クラスを直接インスタンス化することはできません。
	* new SceneManager() コンストラクタを呼び出すと、ArgumentError 例外がスローされます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class SceneManager extends EventIntegrator {
		
		/*======================================================================*//**
		* @private
		*/
		private static var _internallyCalled					:Boolean = false;
		
		
		
		
		
		/*======================================================================*//**
		* 関連付けられている Progression インスタンスを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get progression()						:Progression { return _progression; }
		private var _progression								:Progression;
		
		/*======================================================================*//**
		* シーンオブジェクトのツリー構造部分の一番上にある SceneObject インスタンスを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get root()								:SceneObject { return _root; }
		public function set root( value							:SceneObject ):void {
			_root = value;
			_current = value;
			_previous = value;
			
			// root を設定する
			_root.progression_internal::progression = _progression;
			_root.progression_internal::root = _root;
			_root.progression_internal::parent = _root;
			
			// sceneId を設定する
			_currentSceneId = _root.sceneId.clone();
			_departedSceneId = _currentSceneId.clone();
			_destinedSceneId = _currentSceneId.clone();
			_previousSceneId = _currentSceneId.clone();
		}
		private var _root										:SceneObject;
		
		/*======================================================================*//**
		* 現在、カレント状態となっている　SceneObject インスタンスを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get current()							:SceneObject { return _current; }
		private var _current									:SceneObject;
		
		/*======================================================================*//**
		* ひとつ前に表示したシーンオブジェクトを取得します。
		*/
		private var _previous									:SceneObject;
		
		/*======================================================================*//**
		* 現在、カレント状態となっているシーンオブジェクトのシーン識別子を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get currentSceneId()					:SceneId { return _currentSceneId.clone(); }
		private var _currentSceneId								:SceneId;
		
		/*======================================================================*//**
		* シーン移動時の、出発地となるシーン識別子を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get departedSceneId()					:SceneId { return _departedSceneId.clone(); }
		private var _departedSceneId							:SceneId;
		
		/*======================================================================*//**
		* シーン移動時の、目的地となるシーン識別子を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get destinedSceneId()					:SceneId { return _destinedSceneId.clone(); }
		private var _destinedSceneId							:SceneId;
		
		/*======================================================================*//**
		* ひとつ前に表示したシーン識別子を取得します。
		*/
		private var _previousSceneId							:SceneId;
		
		/*======================================================================*//**
		* 現在表示中のシーンで発生しているイベントタイプを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get eventType()							:String { return _eventType; }
		private var _eventType									:String = SceneEvent.UNLOAD;
		
		/*======================================================================*//**
		* コマンド処理を実行中に、外部からの goto() メソッドの呼び出しを有効にするかどうかを設定または取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get lock()								:Boolean { return _lock; }
		public function set lock( value							:Boolean ):void {
			// autoLock が有効化されていれば無効化する
			if ( _autoLock ) {
				_autoLock = false;
			}
			
			_lock = value;
		}
		private var _lock										:Boolean = false;
		
		/*======================================================================*//**
		* コマンド処理を実行中に lock プロパティの値を自動的に有効化するかどうかを設定または取得します。
		* この設定が有効である場合には、コマンド処理完了後に lock プロパティの値が false となります。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get autoLock()							:Boolean { return _autoLock; }
		public function set autoLock( value						:Boolean ):void { _autoLock = value; }
		private var _autoLock									:Boolean = true;
		
		/*======================================================================*//**
		* 現在の処理状態を取得します。
		* 実行中であれば true、それ以外は false となります。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		// TODO: !! を外すと、謎の強制型変換の警告が出る・・・
		public function get running()							:Boolean { return !!progression_internal::running; }
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function get running()				:Boolean { return _running; }
		progression_internal function set running( value		:Boolean ):void {
			_running = value;
			
			// オートロックが有効であればロックする
			if ( autoLock ) {
				_lock = value;
			}
		}
		private var _running									:Boolean = false;
		
		/*======================================================================*//**
		* 中断処理中かどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		// TODO: !! を外すと、謎の強制型変換の警告が出る・・・
		public function get interrupting()						:Boolean { return !!progression_internal::interrupting; }
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function get interrupting()		:Boolean { return _interrupting; }
		progression_internal function set interrupting( value	:Boolean ):void {
			_interrupting = value;
			
			// オートロックが有効であればロックする
			if ( autoLock ) {
				_lock = value;
			}
		}
		private var _interrupting								:Boolean = false;
		
		/*======================================================================*//**
		* CommandList インスタンスを取得します。 
		*/
		private var _commandList								:CommandList;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい SceneManager インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	progression		関連付けたい Progression インスタンスです。
		* @throws	ArgumentError	SceneManager クラスを直接インスタンス化することはできません。
		*/
		public function SceneManager( progression:Progression ) {
			// パッケージ外から呼び出されたらエラーを送出する
			if ( !_internallyCalled ) { throw new ArgumentError( "Error #2012: SceneManager クラスを直接インスタンス化することはできません。" ); }
			
			// 初期化する
			_progression = progression;
			_internallyCalled = false;
		}
		
		
		
		
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal static function createInstance( progression:Progression ):SceneManager {
			_internallyCalled = true;
			return new SceneManager( progression );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 指定されたシーン識別子の指すシーンに移動します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	sceneId		移動先となるシーン識別子です。
		*/
		public function goto( sceneId:SceneId ):void {
			// SceneManager が管理しているルートシーンではないルートが指定された場合
			if ( sceneId.getNameByIndex( 0 ) != _root.name ) {
				// エラーを送出して終了する
				dispatchEvent( new ProcessEvent( ProcessEvent.PROCESS_ERROR, false, false, _current, _eventType ) );
				return;
			}
			
			// すでに目的のシーンを表示していれば終了する
			if ( _currentSceneId.equals( sceneId ) && _eventType == SceneEvent.INIT ) { return; }
			
			// ロックされていたら終了する
			if ( _lock ) { return; }
			
			// 目的地を設定する
			_departedSceneId = _currentSceneId.clone();
			_destinedSceneId = sceneId.clone();
			
			// すでに中断処理中であれば終了する
			if ( _interrupting ) { return; }
			
			// すでに処理を開始していれば停止させる
			if ( _running ) {
				// イベントリスナーを登録する
				addExclusivelyEventListener( ProcessEvent.PROCESS_INTERRUPT, _processInterrupt, false, int.MAX_VALUE, true );
				
				// 停止処理を実行する
				interrupt();
				return;
			}
			
			// 処理を開始する
			progression_internal::running = true;
			
			// イベントを送出する
			dispatchEvent( new ProcessEvent( ProcessEvent.PROCESS_START, false, false, _current, _eventType ) );
			
			// 処理を開始する
			_executeStart();
		}
		
		/*======================================================================*//**
		* 移動処理の開始処理です。
		*/
		private function _executeStart():void {
			// 現在のシーン状態を保存する
			_previous = _current;
			_previousSceneId = _currentSceneId.clone();
			
			// 目的のシーンと現在のシーンが同じであれば
			if ( _currentSceneId.equals( _destinedSceneId ) ) {
				switch ( _eventType ) {
					case SceneEvent.LOAD		: { _eventType = SceneEvent.INIT; break; }
					case SceneEvent.DESCEND		: { _eventType = SceneEvent.INIT; break; }
					case SceneEvent.INIT		: { break; }
					case SceneEvent.GOTO		: { _eventType = SceneEvent.INIT; break; }
					case SceneEvent.ASCEND		: { _eventType = SceneEvent.INIT; break; }
					case SceneEvent.UNLOAD		: { _eventType = SceneEvent.LOAD; break; }
				}
			}
			
			// 目的のシーンが現在のシーンの子であれば
			else if ( _currentSceneId.contains( _destinedSceneId ) ) {
				switch ( _eventType ) {
					case SceneEvent.LOAD		: { _eventType = SceneEvent.DESCEND; break; }
					case SceneEvent.DESCEND		: {
						// シーンを移動する
						_current = _current.scenes[_destinedSceneId.getNameByIndex( _currentSceneId.length )];
						_eventType = SceneEvent.LOAD;
						break;
					}
					case SceneEvent.INIT		: { _eventType = SceneEvent.GOTO; break;}
					case SceneEvent.GOTO		: {
						// シーンを移動する
						_current = _current.scenes[_destinedSceneId.getNameByIndex( _currentSceneId.length )];
						_eventType = SceneEvent.LOAD;
						break;
					}
					case SceneEvent.ASCEND		: { _eventType = SceneEvent.DESCEND; break; }
					case SceneEvent.UNLOAD		: { _eventType = SceneEvent.LOAD; break; }
				}
			}
			
			// 目的のシーンが現在のシーンの親もしくは親戚であれば
			else {
				switch ( _eventType ) {
					case SceneEvent.LOAD		: { _eventType = SceneEvent.UNLOAD; break; }
					case SceneEvent.DESCEND		: { _eventType = SceneEvent.ASCEND; break; }
					case SceneEvent.INIT		: { _eventType = SceneEvent.GOTO; break; }
					case SceneEvent.GOTO		: { _eventType = SceneEvent.UNLOAD; break; }
					case SceneEvent.ASCEND		: { _eventType = SceneEvent.UNLOAD; break; }
					case SceneEvent.UNLOAD		: {
						// 親シーンに移動する
						_current = _current.public::parent;
						
						// 移動先と目的地のシーンがルートシーンであれば
						if ( _current.sceneId.equals( _root.sceneId ) && _destinedSceneId.equals( _root.sceneId ) ) {
							// ルートシーンに移動する
							_current = _root;
							_eventType = SceneEvent.INIT;
							break;
						}
						
						// 目的のシーンと移動先のシーンが同一であれば
						if ( _current.sceneId.equals( _destinedSceneId ) ) {
							_eventType = SceneEvent.INIT;
							break;
						}
						
						// 目的のシーンが移動先のシーンの子であれば
						if ( _current.sceneId.contains( _destinedSceneId ) ) {
							_current = _current.scenes[_destinedSceneId.getNameByIndex( _current.sceneId.length )];
							_eventType = SceneEvent.LOAD;
							break;
						}
						
						// 目的のシーンが移動先のシーンの親シーンであれば
						_eventType = SceneEvent.ASCEND;
						break;
					}
				}
			}
			
			// 処理を実行する
			_executeProgress();
		}
		
		/*======================================================================*//**
		* 移動処理の実行処理です。
		*/
		private function _executeProgress():void {
			// シーンが存在していれば
			if ( _current is SceneObject ) {
				_currentSceneId = _current.sceneId.clone();
			}
			
			// シーンが存在していなければ
			else {
				// ひとつ前のシーンで停止する
				_current = _previous;
				_currentSceneId = _previousSceneId;
				
				// 処理を終了する
				progression_internal::running = false;
				
				// シーンが存在しなければ、エラーを送出して終了する
				dispatchEvent( new ProcessEvent( ProcessEvent.PROCESS_ERROR, false, false, _current, _eventType ) );
				return;
			}
			
			// シーンが変更されていればイベントを送出する
			if ( !_currentSceneId.equals( _previousSceneId ) ) {
				dispatchEvent( new ProcessEvent( ProcessEvent.PROCESS_SCENE, false, false, _current, _eventType ) );
			}
			
			// イベントを送出する
			dispatchEvent( new ProcessEvent( ProcessEvent.PROCESS_EVENT, false, false, _current, _eventType ) );
			
			// シーンのコマンドリストを消去する
			_current.clearCommand();
			
			// イベントを送出する
			_current.dispatchEvent( new SceneEvent( eventType, false, false, _current, _eventType ) );
			
			// コマンドを取得する
			_commandList = CommandList( _current.progression_internal::commandList );
			
			// コマンドが存在すれば実行する
			if ( _commandList && _commandList.length > 0 ) {
				_commandList.addExclusivelyEventListener( CommandEvent.COMMAND_COMPLETE, _commandComplete, false, int.MAX_VALUE, true );
				_commandList.addExclusivelyEventListener( CommandCatchEvent.EXCEPTION, _catch, false, int.MAX_VALUE, true );
				_commandList.addExclusivelyEventListener( CommandCatchEvent.ERROR, _catch, false, int.MAX_VALUE, true );
				_commandList.execute( {} );
				return;
			}
			
			// 処理を終了する
			_executeComplete();
		}
		
		/*======================================================================*//**
		* 移動処理の終了処理です。
		*/
		private function _executeComplete():void {
			// 目的のシーンに到達していない、もしくはイベントタイプ SceneEvent.INIT でなければ
			if ( !_currentSceneId.equals( _destinedSceneId ) || _eventType != SceneEvent.INIT ) {
				// 処理を開始する
				_executeStart();
				return;
			}
			
			// 処理を終了する
			progression_internal::running = false;
			
			// イベントを送出する
			dispatchEvent( new ProcessEvent( ProcessEvent.PROCESS_COMPLETE, false, false, _current, _eventType ) );
		}
		
		/*======================================================================*//**
		* シーン移動処理を中断します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function interrupt():void {
			// 実行中でなければ終了する
			if ( !_running ) { return; }
			
			// すでに処理していれば終了する
			if ( _interrupting ) { return; }
			
			// 中断処理を開始する
			progression_internal::interrupting = true;
			
			// 実行中のコマンドがあれば
			if ( _commandList ) {
				// イベントリスナーを解除する
				_commandList.removeAllListeners( true );
				
				// イベントリスナーを登録する
				_commandList.addExclusivelyEventListener( CommandEvent.COMMAND_INTERRUPT, _commandInterrupt, false, int.MAX_VALUE, true );
				
				// 停止処理を実行する
				_commandList.interrupt( _current );
				return;
			}
			
			// 処理を終了する
			progression_internal::running = false;
			progression_internal::interrupting = false;
			
			// イベントを送出する
			dispatchEvent( new ProcessEvent( ProcessEvent.PROCESS_INTERRUPT, false, false, _current, _eventType ) );
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object SceneManager]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* コマンド処理が完了した場合に送出されます。
		*/
		private function _commandComplete( e:CommandEvent ):void {
			// 全てのイベントリスナーを解除する
			Command( e.target ).removeAllListeners( true );
			
			// 処理を終了する
			_executeComplete();
		}
		
		/*======================================================================*//**
		* コマンドの処理を停止した場合に送出されます。
		*/
		private function _commandInterrupt( e:CommandEvent ):void {
			// イベントリスナーを解除する
			Command( e.target ).completelyRemoveEventListener( CommandEvent.COMMAND_INTERRUPT, _commandInterrupt );
			
			// 処理を終了する
			progression_internal::running = false;
			progression_internal::interrupting = false;
			
			// イベントを送出する
			dispatchEvent( new ProcessEvent( ProcessEvent.PROCESS_INTERRUPT, false, false, _current, _eventType ) );
		}
		
		/*======================================================================*//**
		* コマンド処理中にエラーが発生した場合に送出されます。
		*/
		private function _catch( e:CommandCatchEvent ):void {
			// 全てのイベントリスナーを解除する
			Command( e.target ).removeAllListeners( true );
			
			// 処理を終了する
			_executeComplete();
		}
		
		/*======================================================================*//**
		* イベント処理が停止された場合に送出されます。
		*/
		private function _processInterrupt( e:ProcessEvent ):void {
			// イベントリスナーを解除する
			completelyRemoveEventListener( ProcessEvent.PROCESS_INTERRUPT, _processInterrupt );
			
			// 移動する
			goto( _destinedSceneId );
		}
	}
}





