﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.core.managers {
	import com.asual.swfaddress.SWFAddress;
	import com.asual.swfaddress.SWFAddressEvent;
	import flash.events.Event;
	import flash.utils.Dictionary;
	import jp.nium.events.EventIntegrator;
	import jp.nium.external.Browser;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.events.ProcessEvent;
	import jp.progression.events.SceneEvent;
	import jp.progression.Progression;
	import jp.progression.scenes.SceneId;
	import jp.progression.scenes.SceneObject;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* SyncManager クラスは、ブラウザとの同期を制御する為のマネージャクラスです。
	* <p />
	* SyncManager クラスを直接インスタンス化することはできません。
	* new SyncManager() コンストラクタを呼び出すと、ArgumentError 例外がスローされます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class SyncManager extends EventIntegrator {
		
		/*======================================================================*//**
		* @private
		*/
		private static var _internallyCalled		:Boolean = false;
		
		
		
		
		
		/*======================================================================*//**
		* ブラウザとの同期処理が可能かどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get enabled()				:Boolean { return _enabled; }
		public function set enabled( value			:Boolean ):void {
			_enabled = false;
			
			// ブラウザとの通信が有効ではなければ終了する
			if ( !Browser.enabled ) { return; }
			
			// swf ファイルを実行中の html 上に swfobject.js と swfaddress.js が読み込まれていなければ終了する
			var code:String = 'function() { switch ( undefined ) {'
				+ 'case deconcept.util				:'
				+ 'case deconcept.SWFObjectUtil		:'
				+ 'case com.asual.util.Browser		:'
				+ 'case com.asual.util.Events		:'
				+ 'case SWFAddress					:'
				+ 'case SWFAddressEvent				:'
				+ '		return "false";'
				+ 'default						:'
				+ '		return "true";'
				+ '} }';
			if ( Browser.call( code ) != "true" ) {
				Progression.verbose.log( this, null, 0, "swfobject.js または swfaddress.js が存在しないか、対応していないバージョンを使用している可能性があります。" );
				return;
			}
			
			// 非ストリクトモードにする
			Browser.call( "SWFAddress.setStrict", false );
			SWFAddress.setStrict( false );
			
			// ストリクトモードのままであれば終了する
			if ( SWFAddress.getStrict() ) {
				Progression.verbose.log( this, null, 0, "使用している swfaddress.js は getStrict() メソッドが使用できないバージョンです。" );
				return;
			}
			
			_enabled = value;
			
			if ( _enabled ) {
				// 他の SyncManager を無効化する
				for each ( var manager:SyncManager in _instances ) {
					if ( manager == this ) { continue; }
					
					manager.enabled = false;
				}
				
				// イベントリスナーを登録する
				SWFAddress.addEventListener( SWFAddressEvent.CHANGE, _change );
				_sceneManager.addExclusivelyEventListener( ProcessEvent.PROCESS_SCENE, _processScene, false, int.MAX_VALUE, true );
				_sceneManager.addExclusivelyEventListener( ProcessEvent.PROCESS_COMPLETE, _processComplete, false, int.MAX_VALUE, true );
			}
			else {
				// イベントリスナーを解除する
				SWFAddress.removeEventListener( SWFAddressEvent.CHANGE, _change );
				_sceneManager.completelyRemoveEventListener( ProcessEvent.PROCESS_SCENE, _processScene );
				_sceneManager.completelyRemoveEventListener( ProcessEvent.PROCESS_COMPLETE, _processComplete );
			}
		}
		private var _enabled						:Boolean = false;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get useLongPath()			:Boolean { return _useLongPath; }
		public function set useLongPath( value		:Boolean ):void { _useLongPath = value; }
		private var _useLongPath					:Boolean = true;
		
		/*======================================================================*//**
		* 関連付けられている SceneManager インスタンスを取得します。
		*/
		private var _sceneManager					:SceneManager;
		
		/*======================================================================*//**
		* SceneObject インスタンスを取得します。
		*/
		private var _scene							:SceneObject;
		
		/*======================================================================*//**
		* SyncManager インスタンスリストを取得します。
		*/
		private static var _instances				:Dictionary = new Dictionary( true );
		
		
		
		
		
		/*======================================================================*//**
		* 新しい SyncManager インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	sceneManager	関連付けたい sceneManager インスタンスです。
		* @throws	ArgumentError	SyncManager クラスを直接インスタンス化することはできません。
		*/
		public function SyncManager( sceneManager:SceneManager ) {
			// パッケージ外から呼び出されたらエラーを送出する
			if ( !_internallyCalled ) { throw new ArgumentError( "Error #2012: SyncManager クラスを直接インスタンス化することはできません。" ); }
			
			// 引数を設定する
			_sceneManager = sceneManager;
			
			// 初期化する
			_instances[this] = this;
			_internallyCalled = false;
		}
		
		
		
		
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal static function createInstance( sceneManager:SceneManager ):SyncManager {
			_internallyCalled = true;
			return new SyncManager( sceneManager );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 
		*/
		private function _update():void {
			// 目的のシーンではなければ終了する
			if ( !_sceneManager.currentSceneId.equals( _sceneManager.destinedSceneId ) ) { return; }
			
			// 更新する
			SWFAddress.setValue( _sceneManager.progression.syncedScenePath );
			SWFAddress.setTitle( _sceneManager.current.title );
			
			// 存在すれば、リスナーを解除する
			if ( _scene ) {
				_scene.completelyRemoveEventListener( SceneEvent.SCENE_STATE_CHANGE, _sceneStateChange );
			}
			
			// 現在のシーンを設定する
			_scene = _sceneManager.current;
			
			// イベントリスナーを登録する
			if ( _scene ) {
				_scene.addEventListener( SceneEvent.SCENE_STATE_CHANGE, _sceneStateChange, false, int.MAX_VALUE, true );
			}
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object SyncManager]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* シーン移動処理中に対象シーンが変更された場合に送出されます。
		*/
		private function _processScene( e:ProcessEvent ):void {
			_update();
		}
		
		/*======================================================================*//**
		* シーン移動処理が完了した場合に送出されます。
		*/
		private function _processComplete( e:ProcessEvent ):void {
			_update();
		}
		
		/*======================================================================*//**
		* シーンオブジェクトの状態が変更された場合に送出されます。
		*/
		private function _sceneStateChange( e:SceneEvent ):void {
			// タイトルを設定する
			SWFAddress.setTitle( e.scene.title );
		}
		
		/*======================================================================*//**
		* 
		*/
		private function _change( e:SWFAddressEvent ):void {
			var value:String = SWFAddress.getValue();
			
			// ロングパスを使用しないのであれば
			if ( !useLongPath ) {
				value = "/" + _sceneManager.progression.id + value;
			}
			
			// 空 String であればルートシーン識別子を設定する
			if ( value == "" ) {
				value = String( _sceneManager.root.sceneId );
			}
			
			// 書式が間違っていれば終了する
			if ( !SceneId.validate( value ) ) { return; }
			
			// シーン識別子を取得する
			var sceneId:SceneId = new SceneId( value );
			
			// 目的地シーンと同一であれば終了する
			if ( _sceneManager.destinedSceneId.equals( sceneId ) ) { return; }
			
			// シーンを移動する
			_sceneManager.goto( sceneId );
		}
	}
}





