﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.core.ui {
	import flash.events.ContextMenuEvent;
	import flash.net.navigateToURL;
	import flash.system.System;
	import flash.ui.ContextMenu;
	import flash.ui.ContextMenuItem;
	import jp.nium.external.Browser;
	import jp.nium.lang.Locale;
	import jp.progression.casts.CastButton;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.scenes.SceneId;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* 
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class CastButtonContextMenu {
		
		/*======================================================================*//**
		* @private
		*/
		private static var _internallyCalled					:Boolean = false;
		
		/*======================================================================*//**
		* @private
		*/
		private static var _locate								:Boolean = function():Boolean {
			Locale.setString( "開く","ja", "開く" );
			Locale.setString( "メールアドレスをコピーする","ja", "メールアドレスをコピーする" );
			
			Locale.setString( "開く","en", "Open" );
			Locale.setString( "メールアドレスをコピーする","en", "Copy Mail Address" );
			
			return true;
		}.apply();
		
		
		
		
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get enabled()						:Boolean { return _enabled; }
		public function set enabled( value					:Boolean ):void { _enabled = value; }
		private var _enabled								:Boolean = true;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get customItems()						:Array { return _customItems; }
		public function set customItems( value					:Array ):void { _customItems = value; }
		private var _customItems								:Array;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get hideOpen()							:Boolean { return _hideOpen; }
		public function set hideOpen( value						:Boolean ):void { _hideOpen = value; }
		private var _hideOpen									:Boolean = false;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get hideOpenNewWindow()					:Boolean { return _hideOpenNewWindow; }
		public function set hideOpenNewWindow( value			:Boolean ):void { _hideOpenNewWindow = value; }
		private var _hideOpenNewWindow							:Boolean = false;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get hideCopyURL()						:Boolean { return _hideCopyURL; }
		public function set hideCopyURL( value					:Boolean ):void { _hideCopyURL = value; }
		private var _hideCopyURL								:Boolean = false;
		
		/*======================================================================*//**
		* 
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get hideBuiltInItems()					:Boolean { return _hideBuiltInItems; }
		public function set hideBuiltInItems( value				:Boolean ):void { _hideBuiltInItems = value; }
		private var _hideBuiltInItems							:Boolean = true;
		
		/*======================================================================*//**
		* 
		*/
		progression_internal function get mouseTarget()			:CastButton { return _mouseTarget; }
		progression_internal function set mouseTarget( value	:CastButton ):void { _mouseTarget = value; }
		private var _mouseTarget								:CastButton;
		
		/*======================================================================*//**
		* 
		*/
		private var _target										:ContextMenu;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい CastButtonContextMenu インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	target			関連付けたい ContextMenu インスタンスです。
		* @throws	ArgumentError	Container クラスを直接インスタンス化することはできません。
		*/
		public function CastButtonContextMenu( target:ContextMenu ) {
			// パッケージ外から呼び出されたらエラーを送出する
			if ( !_internallyCalled ) { throw new ArgumentError( "Error #2012: CastButtonContextMenu クラスを直接インスタンス化することはできません。" ); }
			
			// 初期化する
			_target = target;
			_customItems = new Array();
			_internallyCalled = false;
			
			// イベントリスナーを登録する
			_target.addEventListener( ContextMenuEvent.MENU_SELECT, _menuSelect, false, int.MAX_VALUE, true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal static function createInstance( target:ContextMenu ):CastButtonContextMenu {
			_internallyCalled = true;
			return new CastButtonContextMenu( target );
		}
		
		
		
		
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public function toString():String {
			return "[object CastButtonContextMenu]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* ユーザーが最初にコンテキストメニューを生成したときに、コンテキストメニューの内容が表示される前に送出されます。
		*/
		private function _menuSelect( e:ContextMenuEvent ):void {
			var items:Array = _target.customItems;
			var item:ContextMenuItem;
			
			// イベントを発生させた対象が CastButton インスタンスであれば、対象に設定する
			if ( e.contextMenuOwner is CastButton ) {
				_mouseTarget = CastButton( e.contextMenuOwner );
			}
			
			// 対象が存在しなければ終了する
			if ( !_mouseTarget ) { return; }
			
			// ビルドインアイテムを非表示にする
			if ( hideBuiltInItems ) {
				_target.hideBuiltInItems();
			}
			
			// 無効化されていれば終了する
			if ( !enabled ) { return;  }
			
			// 移動先を取得する
			var sceneId:SceneId = _mouseTarget.sceneId;
			var href:String = _mouseTarget.href;
			
			// 開くを有効化するかどうか
			if ( !hideOpen ) {
				item = new ContextMenuItem( Locale.getString( "開く" ) + "　" );
				item.enabled = ( sceneId || href );
				item.addEventListener( ContextMenuEvent.MENU_ITEM_SELECT, function( e:ContextMenuEvent ):void {
					_mouseTarget.navigateTo();
				} );
				items.push( item );
			}
			
			// 新規ウィンドウで開くを有効化するかどうか
			if ( !hideOpenNewWindow ) {
				item = new ContextMenuItem( Locale.getString( "新規ウィンドウで開く" ) + "　" );
				item.enabled = ( ( Browser.enabled && sceneId ) || href );
				item.addEventListener( ContextMenuEvent.MENU_ITEM_SELECT, function( e:ContextMenuEvent ):void {
					_mouseTarget.navigateTo( null, "_blank" );
				} );
				items.push( item );
			}
			
			// URL のコピーを有効化するかどうか
			if ( !hideCopyURL ) {
				item = new ContextMenuItem( Locale.getString( "URL をコピーする" ) + "　", true );
				item.enabled = ( ( Browser.enabled && sceneId ) || href );
				item.addEventListener( ContextMenuEvent.MENU_ITEM_SELECT, function( e:ContextMenuEvent ):void {
					System.setClipboard( _mouseTarget.progression_internal::currentNavigateURL );
				} );
				items.push( item );
				
				// href がメールアドレスであれば
				if ( new RegExp( "^mailto:", "g" ).test( href ) ) {
					item = new ContextMenuItem( Locale.getString( "メールアドレスをコピーする" ) + "　" );
					item.addEventListener( ContextMenuEvent.MENU_ITEM_SELECT, function( e:ContextMenuEvent ):void {
						System.setClipboard( href.replace( new RegExp( "^mailto:", "g" ), "" ) );
					} );
					items.push( item );
				}
			}
			
			// 先頭の要素のセパレータを有効化する
			ContextMenuItem( items[0] ).separatorBefore = true;
			
			// カスタムメニューを追加する
			var l:int = customItems.length;
			for ( var i:int = 0; i < l; i++ ) {
				items.unshift( customItems[l - i - 1] );
			}
			
			/*
			if ( Progression.usage ) {
				item = new ContextMenuItem( Locale.getString( "操作方法について by Progression" ) + "　", true );
				item.addEventListener( ContextMenuEvent.MENU_ITEM_SELECT, function( e:ContextMenuEvent ):void {
					navigateToURL( new URLRequest( Progression.USAGE_WEBSITE ), "_blank" );
				} );
				items.push( item );
			}
			*/
		}
	}
}





