﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.events {
	import flash.events.Event;
	
	/*======================================================================*//**
	* キャストオブジェクトが AddChild コマンドや RemoveChild コマンドなどで表示リストに追加された場合や、キャストの状態が変更された場合などに、キャストによって CastEvent インスタンスが送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.casts.CastSprite;
	* import jp.progression.commands.AddChild;
	* import jp.progression.commands.RemoveChild;
	* import jp.progression.events.CastEvent;
	* 
	* // CastSprite を作成する
	* var cast:CastSprite = new CastSprite();
	* 
	* // イベントリスナーを登録する
	* cast.addEventListener( CastEvent.CAST_ADDED, function():void {
	* 	trace( "表示リストに追加しました。" );
	* } );
	* cast.addEventListener( CastEvent.CAST_REMOVED, function():void {
	* 	trace( "表示リストから削除しました。" );
	* } );
	* 
	* // コマンドを経由して表示リストに追加する
	* new AddChild( root, cast ).execute();
	* 
	* // コマンドを経由して表示リストから削除する
	* new RemoveChild( root, cast ).execute();
	* </listing>
	*/
	public class CastEvent extends Event {
		
		/*======================================================================*//**
		* castAdded イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const CAST_ADDED			:String = "castAdded";
		
		/*======================================================================*//**
		* castRemoved イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const CAST_REMOVED		:String = "castRemoved";
		
		/*======================================================================*//**
		* castStateChange イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const CAST_STATE_CHANGE	:String = "castStateChange";
		
		
		
		
		
		/*======================================================================*//**
		* 新しい CastEvent インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	type		CastEvent.type としてアクセス可能なイベントタイプです。
		* @param	bubbles		CastEvent インスタンスがイベントフローのバブリング段階で処理されるかどうかを判断します。デフォルト値は false です。
		* @param	cancelable	CastEvent インスタンスがキャンセル可能かどうかを判断します。デフォルト値は false です。
		*/
		public function CastEvent( type:String, bubbles:Boolean = false, cancelable:Boolean = false ) {
			super( type, bubbles, cancelable );
		}
		
		
		
		
		
		/*======================================================================*//**
		* CastEvent インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい CastEvent インスタンスです。
		*/
		public override function clone():Event {
			return new CastEvent( type, bubbles, cancelable );
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return formatToString( "CastEvent", "type", "bubbles", "cancelable" );
		}
	}
}





