﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.events {
	import flash.display.InteractiveObject;
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	/*======================================================================*//**
	* CastButton インスタンスで MouseEvent イベントが発生した場合に、CastButton インスタンスによって CastMouseEvent インスタンスが送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.casts.CastButton;
	* import jp.progression.events.CastMouseEvent;
	* 
	* // CastButton を作成する
	* var cast:CastButton = CastButton( root.addChild( new CastButton() ) );
	* 
	* // イベントリスナーを登録する
	* cast.addEventListener( CastMouseEvent.CAST_CLICK, function():void {
	* 	trace( "クリックしました。" );
	* } );
	* cast.addEventListener( CastMouseEvent.CAST_ROLL_OVER, function():void {
	* 	trace( "ロールオーバーしました。" );
	* } );
	* cast.addEventListener( CastMouseEvent.CAST_ROLL_OUT, function():void {
	* 	trace( "ロールアウトしました。" );
	* } );
	* </listing>
	*/
	public class CastMouseEvent extends MouseEvent {
		
		/*======================================================================*//**
		* castClick イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const CAST_CLICK			:String = "castClick";
		
		/*======================================================================*//**
		* castDoubleClick イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const CAST_DOUBLE_CLICK	:String = "castDoubleClick";
		
		/*======================================================================*//**
		* castMouseDown イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const CAST_MOUSE_DOWN		:String = "castMouseDown";
		
		/*======================================================================*//**
		* castMouseUp イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const CAST_MOUSE_UP		:String = "castMouseUp";
		
		/*======================================================================*//**
		* castMouseOver イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const CAST_MOUSE_OVER		:String = "castMouseOver";
		
		/*======================================================================*//**
		* castMouseOut イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const CAST_MOUSE_OUT		:String = "castMouseOut";
		
		/*======================================================================*//**
		* castMouseMove イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const CAST_MOUSE_MOVE		:String = "castMouseMove";
		
		/*======================================================================*//**
		* castMouseWheel イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const CAST_MOUSE_WHEEL	:String = "castMouseWheel";
		
		/*======================================================================*//**
		* castRollOver イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const CAST_ROLL_OVER		:String = "castRollOver";
		
		/*======================================================================*//**
		* castRollOut イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const CAST_ROLL_OUT		:String = "castRollOut";
		
		
		
		
		
		/*======================================================================*//**
		* 新しい CastMouseEvent インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	type			CastMouseEvent.type としてアクセス可能なイベントタイプです。
		* @param	bubbles			CastMouseEvent インスタンスがイベントフローのバブリング段階で処理されるかどうかを判断します。デフォルト値は false です。
		* @param	cancelable		CastMouseEvent インスタンスがキャンセル可能かどうかを判断します。デフォルト値は false です。
		* @param	localX			スプライトを基準とするイベント発生位置の水平座標です。
		* @param	localY			スプライトを基準とするイベント発生位置の垂直座標です。
		* @param	relatedObject	イベントの影響を受ける補完的な InteractiveObject インスタンスです。たとえば、mouseOut イベントが発生した場合、relatedObject はポインティングデバイスが現在指している表示リストオブジェクトを表します。
		* @param	ctrlKey			Ctrl キーがアクティブになっているかどうかを示します。
		* @param	altKey			将来の使用のために予約されています。
		* @param	shiftKey		Shift キーがアクティブになっているかどうかを示します。
		* @param	buttonDown		マウスの主ボタンが押されているか押されていないかを示します。
		* @param	delta			ユーザーがマウスホイールを 1 目盛り回すごとにスクロールする行数を示します。正の delta 値は上方向へのスクロールを表します。負の値は下方向へのスクロールを表します。一般的な値は 1 ～ 3 の範囲ですが、ホイールの回転が速くなると、delta の値は大きくなります。このパラメータは、CastMouseEvent.mouseWheel イベントのみで使用されます。
		*/
		public function CastMouseEvent( type:String, bubbles:Boolean = false, cancelable:Boolean = false, localX:Number = 0, localY:Number = 0, relatedObject:InteractiveObject = null, ctrlKey:Boolean = false, altKey:Boolean = false, shiftKey:Boolean = false, buttonDown:Boolean = false, delta:int = 0 ) {
			super( type, bubbles, cancelable, localX, localY, relatedObject, ctrlKey, altKey, shiftKey, buttonDown, delta );
		}
		
		
		
		
		
		/*======================================================================*//**
		* CastMouseEvent インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい CastMouseEvent インスタンスです。
		*/
		public override function clone():Event {
			return new CastMouseEvent( type, bubbles, cancelable, localX, localY, relatedObject, ctrlKey, altKey, shiftKey, buttonDown, delta );
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return formatToString( "CastMouseEvent", "type", "bubbles", "cancelable", "localX", "localY", "relatedObject", "ctrlKey", "altKey", "shiftKey", "buttonDown", "delta" );
		}
	}
}





