﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.events {
	import flash.events.Event;
	import jp.progression.core.commands.Command;
	import jp.progression.events.CommandEvent;
	
	/*======================================================================*//**
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class CommandCatchEvent extends CommandEvent {
		
		/*======================================================================*//**
		* error イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const ERROR					:String = "error";
		
		/*======================================================================*//**
		* exception イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const EXCEPTION				:String = "exception";
		
		
		
		
		
		/*======================================================================*//**
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get commandTarget()			:Command { return _commandTarget; }
		private var _commandTarget					:Command;
		
		/*======================================================================*//**
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get relatedObject()			:* { return _relatedObject; }
		private var _relatedObject					:*;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい CommandCatchEvent インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	type			CommandCatchEvent.type としてアクセス可能なイベントタイプです。
		* @param	bubbles			CommandCatchEvent インスタンスがイベントフローのバブリング段階で処理されるかどうかを判断します。デフォルト値は false です。
		* @param	cancelable		CommandCatchEvent インスタンスがキャンセル可能かどうかを判断します。デフォルト値は false です。
		* @param	commandTarget	説明文
		* @param	relatedObject	説明文
		*/
		public function CommandCatchEvent( type:String, bubbles:Boolean = false, cancelable:Boolean = false, commandTarget:Command = null, relatedObject:* = null ) {
			super( type, bubbles, cancelable );
			
			// 引数を設定する
			_commandTarget = commandTarget;
			_relatedObject = relatedObject;
		}
		
		
		
		
		
		/*======================================================================*//**
		* CommandCatchEvent インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい CommandCatchEvent インスタンスです。
		*/
		public override function clone():Event {
			return new CommandCatchEvent( type, bubbles, cancelable, commandTarget.clone(), relatedObject );
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return formatToString( "CommandCatchEvent", "type", "bubbles", "cancelable", "commandTarget", "relatedObject" );
		}
	}
}





