﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.events {
	import flash.events.Event;
	
	/*======================================================================*//**
	* コマンドオブジェクトが処理を開始・終了した場合などに、コマンドオブジェクトによって CommandEvent インスタンスが送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.commands.Wait;
	* import jp.progression.events.CommandEvent;
	* 
	* var com:Wait = new Wait( 1000 );
	* com.addEventListener( CommandEvent.COMMAND_START, function( e:CommandEvent ):void {
	* 	trace( "コマンド処理を開始しました。" );
	* } );
	* com.addEventListener( CommandEvent.COMMAND_COMPLETE, function( e:CommandEvent ):void {
	* 	trace( "コマンド処理を完了しました。" );
	* } );
	* com.addEventListener( CommandEvent.COMMAND_INTERRUPT, function( e:CommandEvent ):void {
	* 	trace( "コマンド処理を中断しました。" );
	* } );
	* com.execute();
	* </listing>
	*/
	public class CommandEvent extends Event {
		
		/*======================================================================*//**
		* commandStart イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const COMMAND_START			:String = "commandStart";
		
		/*======================================================================*//**
		* commandComplete イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const COMMAND_COMPLETE		:String = "commandComplete";
		
		/*======================================================================*//**
		* commandInterrupt イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const COMMAND_INTERRUPT		:String = "commandInterrupt";
		
		
		
		
		
		/*======================================================================*//**
		* 新しい CommandEvent インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	type		CommandEvent.type としてアクセス可能なイベントタイプです。
		* @param	bubbles		CommandEvent インスタンスがイベントフローのバブリング段階で処理されるかどうかを判断します。デフォルト値は false です。
		* @param	cancelable	CommandEvent インスタンスがキャンセル可能かどうかを判断します。デフォルト値は false です。
		*/
		public function CommandEvent( type:String, bubbles:Boolean = false, cancelable:Boolean = false ) {
			super( type, bubbles, cancelable );
		}
		
		
		
		
		
		/*======================================================================*//**
		* CommandEvent インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい CommandEvent インスタンスです。
		*/
		public override function clone():Event {
			return new CommandEvent( type, bubbles, cancelable );
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return formatToString( "CommandEvent", "type", "bubbles", "cancelable" );
		}
	}
}





