﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.events {
	import flash.events.Event;
	import jp.progression.scenes.SceneId;
	import jp.progression.scenes.SceneObject;
	
	/*======================================================================*//**
	* SceneManager インスタンスによって SceneObject インスタンスの移動状態が変化した場合などに、 SceneObject インスタンスによって SceneEvent インスタンスが送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* </listing>
	*/
	public class SceneEvent extends Event {
		
		/*======================================================================*//**
		* load イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const LOAD				:String = "load";
		
		/*======================================================================*//**
		* unload イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const UNLOAD				:String = "unload";
		
		/*======================================================================*//**
		* init イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const INIT				:String = "init";
		
		/*======================================================================*//**
		* goto イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const GOTO				:String = "goto";
		
		/*======================================================================*//**
		* descend イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const DESCEND				:String = "descend";
		
		/*======================================================================*//**
		* ascend イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const ASCEND				:String = "ascend";
		
		/*======================================================================*//**
		* sceneAdded イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const SCENE_ADDED			:String = "sceneAdded";
		
		/*======================================================================*//**
		* sceneRemoved イベントオブジェクトの type プロパティ値を定義します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const SCENE_REMOVED		:String = "sceneRemoved";
		
		/*======================================================================*//**
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const SCENE_STATE_CHANGE	:String = "sceneStateChange";
		
		
		
		
		
		/*======================================================================*//**
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get scene()					:SceneObject { return _scene; }
		private var _scene							:SceneObject;
		
		/*======================================================================*//**
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get eventType()				:String { return _eventType; }
		private var _eventType						:String;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい SceneEvent インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	type		SceneEvent.type としてアクセス可能なイベントタイプです。
		* @param	bubbles		SceneEvent インスタンスがイベントフローのバブリング段階で処理されるかどうかを判断します。デフォルト値は false です。
		* @param	cancelable	SceneEvent インスタンスがキャンセル可能かどうかを判断します。デフォルト値は false です。
		* @param	scene		説明文
		* @param	eventType	説明文
		*/
		public function SceneEvent( type:String, bubbles:Boolean = false, cancelable:Boolean = false, scene:SceneObject = null, eventType:String = null ) {
			super( type, bubbles, cancelable );
			
			// 引数を設定する
			_scene = scene;
			_eventType = eventType;
		}
		
		
		
		
		
		/*======================================================================*//**
		* SceneEvent インスタンスのコピーを作成して、各プロパティの値を元のプロパティの値と一致するように設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	元のオブジェクトと同じプロパティ値を含む新しい SceneEvent インスタンスです。
		*/
		public override function clone():Event {
			return new SceneEvent( type, bubbles, cancelable, scene, eventType );
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return formatToString( "SceneEvent", "type", "bubbles", "cancelable", "scene", "eventType" );
		}
	}
}





