﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.loader {
	import jp.progression.loader.PRMLLoader;
	import jp.progression.Progression;
	import jp.progression.scenes.EasyCastingScene;
	
	/*======================================================================*//**
	* EasyCastingLoader クラスは、拡張された PRML 形式の外部 XML ファイルを読み込んで ActionScript を使用しないスタイルの開発方法を提供します。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import flash.net.URLRequest;
	* import jp.progression.loader.EasyCastingLoader;
	* 
	* // EasyCastingLoader インスタンスを作成する
	* var loader:EasyCastingLoader = new EasyCastingLoader();
	* addChild( loader );
	* 
	* // イベントリスナーを登録する
	* loader.addEventListener( Event.COMPLETE, function():void {
	* 	trace( "読み込みが完了しました。" );
	* } );
	* 
	* // XML ファイルを読み込む
	* loader.load( new URLRequest( "easycasting.xml" ) );
	* </listing>
	*/
	public class EasyCastingLoader extends PRMLLoader {
		
		/*======================================================================*//**
		* 新しい EasyCastingLoader インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function EasyCastingLoader() {
			// コンパイル時に swf に含まれるように参照しておく
			EasyCastingScene;
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function parse( xml:XML ):Progression {
			// <scene> タグの cls プロパティに EasyCastingScene クラスを設定する
			for each ( var scene:XML in xml..scene ) {
				// 設定する
				scene.@cls = "jp.progression.scenes.EasyCastingScene";
			}
			
			// パースする
			return super.parse( xml );
		}
	}
}





