﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.loader {
	import flash.events.Event;
	import flash.events.HTTPStatusEvent;
	import flash.events.IOErrorEvent;
	import flash.events.ProgressEvent;
	import flash.events.SecurityErrorEvent;
	import flash.net.URLRequest;
	import flash.utils.getQualifiedClassName;
	import jp.nium.net.ExURLLoader;
	import jp.nium.utils.XMLUtil;
	import jp.progression.casts.CastSprite;
	import jp.progression.core.display.ToolTip;
	import jp.progression.Progression;
	import jp.progression.scenes.SceneObject;
	
	/*======================================================================*//**
	* データが正常にロードされたときに送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType flash.events.Event.COMPLETE
	*/
	[Event( name="complete", type="flash.events.Event" )]
	
	/*======================================================================*//**
	* 入出力エラーが発生してロード処理が失敗したときに送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType flash.events.IOErrorEvent.IO_ERROR
	*/
	[Event( name="ioError", type="flash.events.IOErrorEvent" )]
	
	/*======================================================================*//**
	* PRMLLoader クラスは、PRML 形式の外部 XML ファイルを使用して Progression Framework を実行する基本クラスです。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import flash.net.URLRequest;
	* import jp.progression.loader.PRMLLoader;
	* 
	* // PRMLLoader インスタンスを作成する
	* var loader:PRMLLoader = new PRMLLoader();
	* addChild( loader );
	* 
	* // イベントリスナーを登録する
	* loader.addEventListener( Event.COMPLETE, function():void {
	* 	trace( "読み込みが完了しました。" );
	* } );
	* 
	* // XML ファイルを読み込む
	* loader.load( new URLRequest( "progression.xml" ) );
	* </listing>
	*/
	public class PRMLLoader extends CastSprite {
		
		/*======================================================================*//**
		* 関連付けられている Progression インスタンスを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get progression()				:Progression { return _progression; }
		private var _progression						:Progression;
		
		/*======================================================================*//**
		* シーン構造の作成に使用する PRML フォーマットの XML ファイルの URL を取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.28.0
		*/
		public function get url()						:String { return _url; }
		public function set url( value					:String ):void { _url = value; }
		private var _url								:String;
		
		/*======================================================================*//**
		* @copy jp.progression.Progression#lock
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get lock()						:Boolean { return _lock; }
		public function set lock( value					:Boolean ):void {
			if ( _progression ) { 
				_progression.lock = value;
			}
			
			_lock = value;
		}
		private var _lock								:Boolean = false;
		
		/*======================================================================*//**
		* @copy jp.progression.Progression#autoLock
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get autoLock()					:Boolean { return _autoLock; }
		public function set autoLock( value				:Boolean ):void {
			if ( _progression ) { 
				_progression.autoLock = value;
			}
			
			_autoLock = value;
		}
		private var _autoLock							:Boolean = true;
		
		/*======================================================================*//**
		* @copy jp.progression.Progression#useLongPath
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get useLongPath()				:Boolean { return _useLongPath; }
		public function set useLongPath( value			:Boolean ):void {
			if ( _progression ) { 
				_progression.useLongPath = value;
			}
			
			_useLongPath = value;
		}
		private var _useLongPath						:Boolean = true;
		
		/*======================================================================*//**
		* @copy jp.progression.Progression#sync
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get sync()						:Boolean { return _sync; }
		public function set sync( value					:Boolean ):void {
			if ( _progression ) { 
				_progression.sync = value;
			}
			
			_sync = value;
		}
		private var _sync								:Boolean = false;
		
		/*======================================================================*//**
		* @copy jp.progression.Progression#keyboard
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get keyboardEmulate()			:Boolean { return _progression.keyboard.enabled; }
		public function set keyboardEmulate( value		:Boolean ):void {
			if ( _progression ) { 
				_progression.keyboard.enabled = value;
			}
			
			_keyboardEmulate = value;
		}
		private var _keyboardEmulate					:Boolean = true;
		
		/*======================================================================*//**
		* ExURLLoader インスタンスを取得します。 
		*/
		private var _loader								:ExURLLoader;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい PRMLLoader インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function PRMLLoader() {
			// コンパイル時に swf に含まれるように参照しておく
			SceneObject;
			
			// 初期化する
			_loader = new ExURLLoader();
			
			// イベントリスナーを登録する
            _loader.addEventListener( Event.OPEN, _open, false, int.MAX_VALUE, true );
			_loader.addEventListener( Event.COMPLETE, _complete, false, int.MAX_VALUE, true );
            _loader.addEventListener( ProgressEvent.PROGRESS, _progress, false, int.MAX_VALUE, true );
            _loader.addEventListener( SecurityErrorEvent.SECURITY_ERROR, _securityError, false, int.MAX_VALUE, true );
            _loader.addEventListener( HTTPStatusEvent.HTTP_STATUS, _httpStatus, false, int.MAX_VALUE, true );
            _loader.addEventListener( IOErrorEvent.IO_ERROR, _ioError, false, int.MAX_VALUE, true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* PRML 形式の XML ファイルを読み込みます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	request		読み込む XML ファイルの絶対 URL または相対 URL です。
		*/
		public function load( request:URLRequest ):void {
			_loader.load( request );
		}
		
		/*======================================================================*//**
		* XML データをパースして Progression インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	Progression インスタンスです。
		*/
		public function parse( xml:XML ):Progression {
			// シーンを取得する
			var scene:XMLList = xml.child( "scene" );
			
			// 存在しなければ終了する
			if ( !scene ) { return null; }
			
			// ルートシーン名を取得する
			var sceneName:String = scene.attribute( "name" );
			
			// ルートシーン名を取得する
			var rootClass:String = scene.attribute( "cls" );
			
			// name が設定されていなければ終了する
			if ( !sceneName ) { return null; }
			
			// Progression インスタンスを作成する
			var prog:Progression = new Progression( sceneName, stage, rootClass );
			prog.keyboard.enabled = _keyboardEmulate;
			prog.lock = _lock;
			prog.sync = _sync;
			prog.useLongPath = _useLongPath;
			prog.autoLock = _autoLock;
			
			// ルートシーンを作成する
			prog.root.setProperties( XMLUtil.xmlToObject( scene.attributes() ) );
			prog.root.extra = scene.children().( name() != "scene" );
			
			// ルートを削除した XML インスタンスを作成する
			xml = <prml version="2.0.0" />;
			xml.prependChild( scene.children() );
			
			// ルートシーン以下を作成する
			prog.root.addChildFromXML( xml );
			
			return prog;
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object " + getQualifiedClassName( this ).split( "::" ).pop() + "]";
		}
		
		
		
		
		
		/*======================================================================*//**
		* ロード操作が開始したときに送出されます。
		*/
		private function _open( e:Event ):void {
			// イベントを送出する
			dispatchEvent( e );
		}
		
		/*======================================================================*//**
		* データが正常にロードされたときに送出されます。
		*/
		private function _complete( e:Event ):void {
			// Progression インスタンスを作成する
			_progression = parse( XML( e.target.data ) );
			
			// 作成に失敗すれば
			if ( !_progression ) {
				// イベントを送出する
				dispatchEvent( new IOErrorEvent( IOErrorEvent.IO_ERROR ) );
				return;
			}
			
			// イベントを送出する
			dispatchEvent( e );
			
			// 最初のシーンに移動する
			_progression.goto( _progression.firstSceneId );
		}
		
		/*======================================================================*//**
		* ダウンロード処理を実行中にデータを受信したときに送出されます。
		*/
		private function _progress( e:ProgressEvent ):void {
			// イベントを送出する
			dispatchEvent( e );
		}
		
		/*======================================================================*//**
		* URLLoader.load() の呼び出しによってセキュリティサンドボックスの外部にあるサーバーからデータをロードしようとすると送出されます。
		*/
		private function _securityError( e:SecurityErrorEvent ):void {
			// イベントを送出する
			dispatchEvent( e );
		}
		
		/*======================================================================*//**
		* ネットワーク要求が HTTP を介して行われ、Flash Player が HTTP 状況コードを検出できる場合に送出されます。
		*/
		private function _httpStatus( e:HTTPStatusEvent ):void {
			// イベントを送出する
			dispatchEvent( e );
		}
		
		/*======================================================================*//**
		* 入出力エラーが発生してロード処理が失敗したときに送出されます。
		*/
		private function _ioError( e:IOErrorEvent ):void {
			// イベントを送出する
			dispatchEvent( e );
		}
	}
}





