﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.scenes {
	import flash.errors.IllegalOperationError;
	import flash.net.URLRequest;
	import flash.utils.Dictionary;
	import flash.utils.getDefinitionByName;
	import jp.nium.utils.ObjectUtil;
	import jp.progression.commands.Prop;
	import jp.nium.utils.StringUtil;
	import jp.progression.commands.AddChild;
	import jp.progression.commands.AddChildAt;
	import jp.progression.commands.ParallelList;
	import jp.progression.commands.RemoveChild;
	import jp.progression.commands.SerialList;
	import jp.progression.events.SceneEvent;
	import jp.progression.scenes.SceneObject;
	
	/*======================================================================*//**
	* EasyCastingScene クラスは、拡張された PRML 形式の XML データを使用して ActionScript を使用しないスタイルの開発方法を提供します。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.Progression;
	* import jp.progression.scenes.EasyCastingScene;
	* 
	* // 開発者用の出力を有効化する
	* Progression.verbose.enabled = true;
	* 
	* // Progression インスタンスを作成する
	* var prog:Progression = new Progression( "index", stage );
	* 
	* // シーンを作成する
	* var page1:EasyCastingScene = EasyCastingScene( prog.root.addChild( new EasyCastingScene( "page1" ) ) );
	* var page2:EasyCastingScene = EasyCastingScene( page1.addChild( new EasyCastingScene( "page2" ) ) );
	* var page3:EasyCastingScene = EasyCastingScene( page2.addChild( new EasyCastingScene( "page3" ) ) );
	* page3.extra = new XMLList(
	* 	<cast cls="CastSprite1" />
	* 	<cast cls="CastSprite2" />
	* );
	* 
	* // シーンを移動する
	* prog.goto( page3.sceneId );
	* </listing>
	*/
	public class EasyCastingScene extends SceneObject {
		
		/*======================================================================*//**
		* EasyCastingScene が管理しているキャストオブジェクトを格納した Dictionary インスタンスを取得します。
		*/
		private static var _casts				:Dictionary = new Dictionary( true );
		
		
		
		
		
		/*======================================================================*//**
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		/*
		public function get inherit() 			:Boolean { return _inherit; }
		public function set inherit( value		:Boolean ):void { _inherit = value; }
		private var _inherit					:Boolean = false;
		*/
		
		
		
		
		
		/*======================================================================*//**
		* 新しい EasyCastingScene インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	name	シーンの名前です。
		*/
		public function EasyCastingScene( name:String ) {
			super( name );
			
			// イベントリスナーを登録する
			addExclusivelyEventListener( SceneEvent.INIT, _init, false, int.MAX_VALUE, true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function addChild( child:SceneObject ):SceneObject {
			if ( !( child is EasyCastingScene ) ) { throw new IllegalOperationError( "EasyCastingScene　インスタンスの子シーンには、EasyCastingScene インスタンス以外設定することはできません。" ); }
			return super.addChild( child );
		}
		
		/*======================================================================*//**
		* @inheritDoc
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public override function addChildAt( child:SceneObject, index:int, addAbove:Boolean = false ):SceneObject {
			if ( !( child is EasyCastingScene ) ) { throw new IllegalOperationError( "EasyCastingScene　インスタンスの子シーンには、EasyCastingScene インスタンス以外設定することはできません。" ); }
			return super.addChildAt( child, index, addAbove );
		}
		
		
		
		
		
		/*======================================================================*//**
		* シーンオブジェクトが目的地だった場合に、到達した瞬間に送出されます。
		*/
		private function _init( e:SceneEvent ):void {
			// ルート以下の XML 構造を取得する
			var xml:XML = new XML( root.toXMLString() );
			
			// casts を更新する
			for each ( var cast:XML in xml..cast ) {
				var classPath:String = String( cast.attribute( "cls" ) );
				
				// クラスパスが設定されていなければ次へ
				if ( !classPath ) { continue; }
				
				// すでにキャストが存在していれば次へ
				if ( _casts[classPath] ) { continue; }
				
				// cls をクラスに変換する
				var classRef:Class = Class( getDefinitionByName( classPath ) );
				var instance:* = new classRef();
				
				// キャストを設定する
				_casts[classPath] = instance;
			}
			
			// リストを作成する
			var dic:Dictionary = new Dictionary( true );
			for each ( cast in extra ) {
				classPath = String( cast.attribute( "cls" ) );
				
				dic[classPath] = cast;
			}
			
			// コマンドリストを作成する
			var addCommandList:ParallelList = new ParallelList();
			var removeCommandList:ParallelList = new ParallelList();
			
			// コマンドを作成する
			for ( classPath in _casts ) {
				// プロパティを取得する
				var o:Object = {};
				for each ( var attr:XML in XML( dic[classPath] ).attributes() ) {
					o[String( attr.name() )] = StringUtil.toProperType( attr.toString() );
				}
				
				// プロパティを設定する
				ObjectUtil.setProperties( _casts[classPath], o );
				
				// 存在すれば
				if ( dic[classPath] ) {
					// インデックスを取得する
					var index:int = parseInt( XML( dic[classPath] ).attribute( "index" ) );
					
					// コマンドを追加する
					if ( index is int ) {
						addCommandList.addCommand( new AddChildAt( progression.container, _casts[classPath], index, true ) );
					}
					else {
						addCommandList.addCommand( new AddChild( progression.container, _casts[classPath] ) );
					}
				}
				else {
					// コマンドを追加する
					removeCommandList.addCommand( new RemoveChild( progression.container, _casts[classPath] ) );
				}
			}
			
			// コマンドを追加する
			addCommand(
				new SerialList( 0, removeCommandList, addCommandList )
			);
		}
	}
}





