﻿/*======================================================================*//**
* 
* Progression Framework
* 
* @author		Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* @version		2.0.22
* @see			http://progression.jp/
* @see			http://progression.libspark.org/
* 
* Project Manager
* taka:nium		http://nium.jp/
* 
* Hosted by
* Spark project	http://www.libspark.org/
* 
* Supported by
* yossy			http://www.be-interactive.org/
* cellfusion	http://cellfusion.jp/
* rch850		http://850mb.net/
* nobu			http://humming.via-kitchen.com/
* trick7		http://www.trick7.com/
* minaco		http://minaco.net/
* ksakai		http://www.onflow.jp/blog/
* 
* Licensed under the MIT License
* 
* Copyright (c) 2007-2008 taka:nium.jp, supported by Spark project.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
* 
*/
package jp.progression.scenes {
	import flash.display.DisplayObject;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.events.SecurityErrorEvent;
	import flash.net.URLRequest;
	import flash.utils.getDefinitionByName;
	import flash.utils.getQualifiedClassName;
	import jp.nium.events.EventIntegrator;
	import jp.nium.net.ExURLLoader;
	import jp.nium.utils.ObjectUtil;
	import jp.nium.utils.XMLUtil;
	import jp.progression.casts.ICastObject;
	import jp.progression.commands.ParallelList;
	import jp.progression.commands.SerialList;
	import jp.progression.core.collections.SceneList;
	import jp.progression.core.commands.Command;
	import jp.progression.core.commands.CommandList;
	import jp.progression.core.namespaces.progression_internal;
	import jp.progression.events.ProcessEvent;
	import jp.progression.events.SceneEvent;
	import jp.progression.Progression;
	import jp.progression.scenes.SceneId;
	
	use namespace progression_internal;
	
	/*======================================================================*//**
	* シーン移動時に目的地がシーンオブジェクト自身もしくは子階層だった場合に、階層が変更された瞬間に送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.SceneEvent.LOAD
	*/
	[Event( name="load", type="jp.progression.events.SceneEvent" )]
	
	/*======================================================================*//**
	* シーン移動時に目的地がシーンオブジェクト自身もしくは親階層だった場合に、階層が変更された瞬間に送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.SceneEvent.UNLOAD
	*/
	[Event( name="unload", type="jp.progression.events.SceneEvent" )]
	
	/*======================================================================*//**
	* シーンオブジェクトが目的地だった場合に、到達した瞬間に送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.SceneEvent.INIT
	*/
	[Event( name="init", type="jp.progression.events.SceneEvent" )]
	
	/*======================================================================*//**
	* シーンオブジェクトが出発地だった場合に、移動を開始した瞬間に送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.SceneEvent.GOTO
	*/
	[Event( name="goto", type="jp.progression.events.SceneEvent" )]
	
	/*======================================================================*//**
	* シーン移動時に目的地がシーンオブジェクトの子階層であり、かつ出発地ではない場合に、移動を中継した瞬間に送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.SceneEvent.DESCEND
	*/
	[Event( name="descend", type="jp.progression.events.SceneEvent" )]
	
	/*======================================================================*//**
	* シーン移動時に目的地がシーンオブジェクトの親階層であり、かつ出発地ではない場合に、移動を中継した瞬間に送出されます。
	* このイベント処理の実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.SceneEvent.ASCEND
	*/
	[Event( name="ascend", type="jp.progression.events.SceneEvent" )]
	
	/*======================================================================*//**
	* シーンオブジェクトがシーンリストに追加された場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.SceneEvent.SCENE_ADDED
	*/
	[Event( name="sceneAdded", type="jp.progression.events.SceneEvent" )]
	
	/*======================================================================*//**
	* シーンオブジェクトがシーンリストから削除された場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.SceneEvent.SCENE_REMOVED
	*/
	[Event( name="sceneRemoved", type="jp.progression.events.SceneEvent" )]
	
	/*======================================================================*//**
	* シーンオブジェクトの状態が変更された場合に送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType jp.progression.events.SceneEvent.SCENE_STATE_CHANGE
	*/
	[Event( name="sceneStateChange", type="jp.progression.events.SceneEvent" )]
	
	/*======================================================================*//**
	* データが正常にロードされたときに送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType flash.events.Event.COMPLETE
	*/
	[Event( name="complete", type="flash.events.Event" )]
	
	/*======================================================================*//**
	* 入出力エラーが発生してロード処理が失敗したときに送出されます。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @eventType flash.events.IOErrorEvent.IO_ERROR
	*/
	[Event( name="ioError", type="flash.events.IOErrorEvent" )]
	
	/*======================================================================*//**
	* SceneObject クラスは、シーンコンテナとして機能する全てのオブジェクトの基本クラスです。
	* シーンコンテナは、Progression で使用するシーン構造の全てを管理します。
	* @langversion 3.0
	* @playerversion Flash 9.0.45.0
	* @example <listing version="3.0" >
	* import jp.progression.Progression;
	* import jp.progression.scenes.SceneObject;
	* 
	* // 開発者用の出力を有効化する
	* Progression.verbose.enabled = true;
	* 
	* // Progression インスタンスを作成する
	* var prog:Progression = new Progression( "index", stage );
	* 
	* // シーンを作成する
	* var page1:SceneObject = prog.root.addChild( new SceneObject( "page1" ) );
	* var page2:SceneObject = page1.addChild( new SceneObject( "page2" ) );
	* var page3:SceneObject = page2.addChild( new SceneObject( "page3" ) );
	* page3.onInit = function():void {
	* 	trace( "page3 を表示しました。" )
	* }
	* 
	* // シーンを移動する
	* prog.goto( page3.sceneId );
	* </listing>
	*/
	public class SceneObject extends EventIntegrator {
		
		/*======================================================================*//**
		* 対応している PRML 形式のバージョンを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public static const PRML_VERSION						:String = "2.0.0";
		
		
		
		
		
		/*======================================================================*//**
		* シーンの名前を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get name()								:String { return _name; }
		private var _name										:String;
		
		/*======================================================================*//**
		* シーン識別子を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get sceneId()							:SceneId {
			// 親が存在しなければ null を返す
			if ( !parent ) { return null; }
			
			// 親が自分であれば
			if ( parent == this ) { return new SceneId( "/" + name ); }
			
			return new SceneId( String( parent.sceneId ) + "/" + name );
		}
		
		/*======================================================================*//**
		* 子シーンオブジェクトのツリーリストを取得します。
		* 子シーンを指定するにはスラッシュ・シンタックスで、scenes."子シーンオブジェクト名" のように指定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get scenes()							:SceneList { return _scenes; }
		private var _scenes										:SceneList;
		
		/*======================================================================*//**
		* 子シーンオブジェクトのツリーリストを配列で取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get children()							:Array { return _scenes.progression_internal::children.concat(); }
		
		/*======================================================================*//**
		* このシーンオブジェクトの子の数を取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get numChildren()						:int { return _scenes.progression_internal::numChildren; }
		
		/*======================================================================*//**
		* 関連付けられている Progression インスタンスを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get progression()						:Progression { return progression_internal::progression; }
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function get progression()			:Progression { return _progression; }
		progression_internal function set progression( value	:Progression ):void {
			// 設定が存在していれば
			if ( _progression ) {
				// イベントリスナーを解除する
				_progression.completelyRemoveEventListener( ProcessEvent.PROCESS_SCENE, _processScene );
			}
			
			// 値を設定する
			_progression = value;
			
			// 設定が存在していれば
			if ( _progression ) {
				// イベントリスナーを登録する
				_progression.addExclusivelyEventListener( ProcessEvent.PROCESS_SCENE, _processScene, false, int.MAX_VALUE, true );
			}
			
			// 子の関連性を再設定する
			for each ( var scene:SceneObject in scenes ) {
				scene.progression_internal::progression = value;
			}
		}
		private var _progression								:Progression;
		
		/*======================================================================*//**
		* シーンオブジェクトのツリー構造部分の一番上にある SceneObject インスタンスを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get root()								:SceneObject { return progression_internal::root; }
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function get root()				:SceneObject { return _root; }
		progression_internal function set root( value			:SceneObject ):void {
			// 値を設定する
			_root = value;
			
			// 子の関連性を再設定する
			for each ( var scene:SceneObject in scenes ) {
				scene.progression_internal::root = value;
			}
		}
		private var _root										:SceneObject;
		
		/*======================================================================*//**
		* このシーンオブジェクトを子に含んでいる親シーンオブジェクトを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get parent()							:SceneObject { return progression_internal::parent; }
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function get parent()				:SceneObject { return _parent; }
		progression_internal function set parent( value			:SceneObject ):void {
			// 値を設定する
			_parent = value;
			
			// 子の関連性を再設定する
			for each ( var scene:SceneObject in scenes ) {
				scene.progression_internal::parent = this;
			}
		}
		private var _parent										:SceneObject;
		
		/*======================================================================*//**
		* このシーンオブジェクトの次に位置するシーンオブジェクトを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get next()								:SceneObject {
			// 親が存在しなければ null を返す
			if ( !parent ) { return null; }
			
			// 親が自分であれば自分を返す
			if ( parent == this ) { return this; }
			
			return parent.scenes.progression_internal::getNextChild( this );
		}
		
		/*======================================================================*//**
		* このシーンオブジェクトの前に位置するシーンオブジェクトを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get previous()							:SceneObject {
			// 親が存在しなければ null を返す
			if ( !parent ) { return null; }
			
			// 親が自分であれば自分を返す
			if ( parent == this ) { return this; }
			
			return parent.scenes.progression_internal::getPreviousChild( this );
		}
		
		/*======================================================================*//**
		* このシーンオブジェクトがカレントシーンであるかどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get isCurrent()							:Boolean { return _isCurrent; }
		private var _isCurrent									:Boolean = false;
		
		/*======================================================================*//**
		* このシーンオブジェクトがカレントシーンの親シーンに当たるかどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get isParent()							:Boolean { return _isParent; }
		private var _isParent									:Boolean = false;
		
		/*======================================================================*//**
		* このシーンオブジェクトがカレントシーンの子シーンに当たるかどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get isChild()							:Boolean { return _isChild; }
		private var _isChild									:Boolean = false;
		
		/*======================================================================*//**
		* addCommand() メソッドで追加した Command インスタンスの実行方法を並列処理にするかどうかを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get parallelMode()						:Boolean { return _commandList is ParallelList; }
		public function set parallelMode( value					:Boolean ):void {
			// 既存のコマンドを取得する
			var commands:Array = _commandList.progression_internal::commands.concat();
			
			// 初期化する
			_commandList = ( value ) ? new ParallelList() : new SerialList();
			
			// コマンドを追加する
			_commandList.addCommand.apply( null, commands );
		}
		
		/*======================================================================*//**
		* このシーンオブジェクトが既読であるかどうかを取得します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get readed()							:Boolean { return _readed; }
		private var _readed										:Boolean = false;
		
		/*======================================================================*//**
		* このシーンが属している Progression インスタンスの sync プロパティの値が true であり、
		* かつ自身がカレントシーンである場合に、ブラウザのタイトルとして使用される文字列を取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get title()								:String { return _title || name; }
		public function set title( value						:String ):void {
			// 値を設定する
			_title = value;
			
			// イベントを送出する
			dispatchEvent( new SceneEvent( SceneEvent.SCENE_STATE_CHANGE, false, false, this ) );
		}
		private var _title										:String;
		
		/*======================================================================*//**
		* 任意のデータを取得または設定します。
		* addChildFromXML() を経由して作成された場合には、XML から自動的に値が設定されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get extra()								:Object { return _extra; }
		public function set extra( value						:Object ):void { _extra = value; }
		private var _extra										:Object;
		
		/*======================================================================*//**
		* @private
		*/
		progression_internal function get commandList()			:CommandList { return _commandList; }
		progression_internal function set commandList( value	:CommandList ):void { _commandList = value; }
		private var _commandList								:CommandList;
		
		/*======================================================================*//**
		* シーンオブジェクトが SceneEvent.LOAD イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* このイベントハンドラメソッド実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onLoad()							:Function { return __onLoad || _onLoad; }
		public function set onLoad( value						:Function ):void { __onLoad = value; }
		private var __onLoad									:Function;
		
		/*======================================================================*//**
		* サブクラスで onLoad イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onLoad プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onLoad
		*/
		protected function _onLoad()							:void {}
		
		/*======================================================================*//**
		* シーンオブジェクトが SceneEvent.UNLOAD イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* このイベントハンドラメソッド実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onUnload()							:Function { return __onUnload || _onUnload; }
		public function set onUnload( value						:Function ):void { __onUnload = value; }
		private var __onUnload									:Function;
		
		/*======================================================================*//**
		* サブクラスで onUnload イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onUnload プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onUnload
		*/
		protected function _onUnload()							:void {}
		
		/*======================================================================*//**
		* シーンオブジェクトが SceneEvent.INIT イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* このイベントハンドラメソッド実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onInit()							:Function { return __onInit || _onInit; }
		public function set onInit( value						:Function ):void { __onInit = value; }
		private var __onInit									:Function;
		
		/*======================================================================*//**
		* サブクラスで onInit イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onInit プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onInit
		*/
		protected function _onInit()							:void {}
		
		/*======================================================================*//**
		* シーンオブジェクトが SceneEvent.GOTO イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* このイベントハンドラメソッド実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onGoto()							:Function { return __onGoto || _onGoto; }
		public function set onGoto( value						:Function ):void { __onGoto = value; }
		private var __onGoto									:Function;
		
		/*======================================================================*//**
		* サブクラスで onGoto イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onGoto プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onGoto
		*/
		protected function _onGoto()							:void {}
		
		/*======================================================================*//**
		* シーンオブジェクトが SceneEvent.DESCEND イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* このイベントハンドラメソッド実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onDescend()							:Function { return __onDescend || _onDescend; }
		public function set onDescend( value					:Function ):void { __onDescend = value; }
		private var __onDescend									:Function;
		
		/*======================================================================*//**
		* サブクラスで onDescend イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onDescend プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onDescend
		*/
		protected function _onDescend()							:void {}
		
		/*======================================================================*//**
		* シーンオブジェクトが SceneEvent.ASCEND イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* このイベントハンドラメソッド実行中には、addCommand() メソッドによるコマンドの同期処理が行えます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onAscend()							:Function { return __onAscend || _onAscend; }
		public function set onAscend( value						:Function ):void { __onAscend = value; }
		private var __onAscend									:Function;
		
		/*======================================================================*//**
		* サブクラスで onAscend イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onAscend プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onAscend
		*/
		protected function _onAscend()							:void {}
		
		/*======================================================================*//**
		* シーンオブジェクトが SceneEvent.SCENE_ADDED イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onSceneAdded()						:Function { return __onSceneAdded || _onSceneAdded; }
		public function set onSceneAdded( value					:Function ):void { __onSceneAdded = value; }
		private var __onSceneAdded								:Function;
		
		/*======================================================================*//**
		* サブクラスで onSceneAdded イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onSceneAdded プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onSceneAdded
		*/
		protected function _onSceneAdded()						:void {}
		
		/*======================================================================*//**
		* シーンオブジェクトが SceneEvent.SCENE_REMOVED イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onSceneRemoved()					:Function { return __onSceneRemoved || _onSceneRemoved; }
		public function set onSceneRemoved( value				:Function ):void { __onSceneRemoved = value; }
		private var __onSceneRemoved							:Function;
		
		/*======================================================================*//**
		* サブクラスで onSceneRemoved イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* このメソッドは、onSceneRemoved プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onSceneRemoved
		*/
		protected function _onSceneRemoved()					:void {}
		
		/*======================================================================*//**
		* シーンオブジェクトが SceneEvent.SCENE_STATE_CHANGE イベントを受け取った場合に呼び出されるイベントハンドラメソッドを取得または設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function get onSceneStateChange()				:Function { return __onSceneStateChange || _onSceneStateChange; }
		public function set onSceneStateChange( value			:Function ):void { __onSceneStateChange = value; }
		private var __onSceneStateChange						:Function;
		
		/*======================================================================*//**
		* サブクラスで onSceneStateChange イベントハンドラメソッドの処理を override で指定したい場合に上書きします。
		* onSceneStateChange プロパティに別のメソッドを指定された場合、無効化されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @see	#onSceneStateChange
		*/
		protected function _onSceneStateChange()				:void {}
		
		/*======================================================================*//**
		* ExURLLoader インスタンスを取得します。 
		*/
		private var _loader										:ExURLLoader;
		
		/*======================================================================*//**
		* クラスネームを String で取得します。
		*/
		private var _className									:String;
		
		
		
		
		
		/*======================================================================*//**
		* 新しい SceneObject インスタンスを作成します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	name			シーンの名前です。
		* @throws	ArgumentError	指定されたシーン名の書式が正しくありません。
		*/
		public function SceneObject( name:String ) {
			// 書式が正しくない場合エラーを送出する
			if ( !SceneId.validate( "/" + name ) ) { throw new ArgumentError( "シーン名には半角英数（_ および - を含む）のみ使用可能です。" ); }
			
			// 引数を設定する
			_name = name;
			
			// 初期化する
			_scenes = SceneList.progression_internal::createInstance( this );
			_commandList = new SerialList();
			_extra = {};
			_className = getQualifiedClassName( this ).split( "::" ).pop();
			
			// ExURLLoader インスタンスを作成する
			_loader = new ExURLLoader();
			_loader.addEventListener( Event.COMPLETE, _complete, false, int.MAX_VALUE, true );
            _loader.addEventListener( SecurityErrorEvent.SECURITY_ERROR, _securityError, false, int.MAX_VALUE, true );
			_loader.addEventListener( IOErrorEvent.IO_ERROR, _ioError, false, int.MAX_VALUE, true );
			
			// イベントリスナーを登録する
			addExclusivelyEventListener( SceneEvent.LOAD, _load, false, 0, true );
			addExclusivelyEventListener( SceneEvent.DESCEND, _descend, false, 0, true );
			addExclusivelyEventListener( SceneEvent.INIT, _init, false, 0, true );
			addExclusivelyEventListener( SceneEvent.GOTO, _goto, false, 0, true );
			addExclusivelyEventListener( SceneEvent.ASCEND, _ascend, false, 0, true );
			addExclusivelyEventListener( SceneEvent.UNLOAD, _unload, false, 0, true );
			addExclusivelyEventListener( SceneEvent.SCENE_ADDED, _sceneAdded, false, 0, true );
			addExclusivelyEventListener( SceneEvent.SCENE_REMOVED, _sceneRemoved, false, 0, true );
			addExclusivelyEventListener( SceneEvent.SCENE_STATE_CHANGE, _sceneStateChange, false, 0, true );
		}
		
		
		
		
		
		/*======================================================================*//**
		* PRML 形式のバージョンが対応しているかどうかを返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	対応していれば true に、それ以外の場合は false になります。
		*/
		public static function checkPRMLVersion( version:String ):Boolean {
			// バージョン情報を取得する
			var ver1:Array = version.split( "." );
			var ver2:Array = PRML_VERSION.split( "." );
			
			// バージョンが同じであれば true を返す
			if ( version == PRML_VERSION ) { return true; }
			
			// バージョンを比較する
			var l:int = ver1.length;
			for ( var i:int = 0; i < l; i++ ) {
				var v1:int = parseInt( ver1[i] );
				var v2:int = parseInt( ver2[i] );
				
				// バージョンが大きければ true を返す
				if ( v1 > v2 ) { return true; }
			}
			
			return false;
		}
		
		/*======================================================================*//**
		* このシーンオブジェクトに子シーンオブジェクトを追加します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	child		このシーンオブジェクトの子として追加するシーンオブジェクトです。
		* @return				child パラメータで渡すシーンオブジェクトです。 
		*/
		public function addChild( child:SceneObject ):SceneObject {
			// 子シーンに登録する
			scenes.progression_internal::addChild( child );
			
			// 関係性を設定する
			child.progression_internal::progression = progression;
			child.progression_internal::root = root;
			child.progression_internal::parent = this;
			
			// イベントを送出する
			child.dispatchEvent( new SceneEvent( SceneEvent.SCENE_ADDED, false, false, child ) );
			
			return child;
		}
		
		/*======================================================================*//**
		* このシーンオブジェクトに子シーンオブジェクトを追加します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	child		このシーンオブジェクトの子として追加するシーンオブジェクトです。
		* @param	index		子を追加するインデックス位置です。既にオブジェクトが置かれているインデックス位置を指定すると、その位置にあるオブジェクトとその上に位置するすべてのオブジェクトが、子リスト内で 1 つ上の位置に移動します。
		* @param	addAbove	指定したインデックス位置にすでにシーンオブジェクトが存在した場合に、そのシーンオブジェクトの上に配置するかどうかです。
		* @return				child パラメータで渡すシーンオブジェクトです。 
		*/
		public function addChildAt( child:SceneObject, index:int, addAbove:Boolean = false ):SceneObject {
			// 子シーンに登録する
			scenes.progression_internal::addChildAt( child, index, addAbove );
			
			// 関係性を設定する
			child.progression_internal::progression = progression;
			child.progression_internal::root = root;
			child.progression_internal::parent = this;
			
			// イベントを送出する
			child.dispatchEvent( new SceneEvent( SceneEvent.SCENE_ADDED, false, false, child ) );
			
			return child;
		}
		
		/*======================================================================*//**
		* このシーンオブジェクトの子シーンオブジェクトを PRML 形式の XML データから追加します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	xml		PRML 形式の XML データです。
		*/
		public function addChildFromXML( xml:XML ):void {
			// <prml> タグが存在しなければ終了する
			if ( xml.name() != "prml" ) { return; }
			
			// バージョンを取得する
			var version:String = String( xml.attribute( "version" ) );
			
			// バージョンが対応していなければ終了する
			if ( !checkPRMLVersion( version ) ) { return; }
			
			// <scene> タグリストを取得する
			var scenes:XMLList = xml.children();
			
			for each ( var scene:XML in scenes ) {
				// <scene> タグではなければ次へ
				if ( scene.name() != "scene" ) { continue; }
				
				// name が設定されていなければ次へ
				if ( !scene.attribute( "name" ) ) { continue; }
				
				// cls が指定されていなければ SceneObject とする
				var classPath:String = String( scene.attribute( "cls" ) );
				classPath = classPath || "jp.progression.scenes.SceneObject";
				
				// cls をクラスに変換する
				var classRef:Class = Class( getDefinitionByName( classPath ) );
				var child:SceneObject = new classRef( scene.attribute( "name" ) );
				
				// child が SceneObject クラスを継承していなければエラーを送出する
				if ( !( child is SceneObject ) ) { throw new ArgumentError( "指定された " + classPath + " クラスは、SceneObject クラスを継承していません。" ); }
				
				// 子に渡す XML インスタンスを作成する
				var childXML:XML = <prml version={ version } />;
				childXML.prependChild( scene.children() );
				
				// 子シーンを作成する
				addChild( child );
				child.setProperties( XMLUtil.xmlToObject( scene.attributes() ) );
				child.extra = scene.children().( name() != "scene" );
				child.addChildFromXML( childXML );
			}
		}
		
		/*======================================================================*//**
		* このシーンオブジェクトの子シーンオブジェクトを PRML 形式の XML ファイルを、指定された URL から読み込んで追加します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	request		読み込む XML ファイルの絶対 URL または相対 URL です。
		*/
		public function loadChild( request:URLRequest ):void {
			_loader.load( request );
		}
		
		/*======================================================================*//**
		*シーンオブジェクトの子リストから指定のシーンオブジェクトを削除します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	child			削除するシーンオブジェクトです。
		* @return					child パラメータで渡すシーンオブジェクトです。
		*/
		public function removeChild( child:SceneObject ):SceneObject {
			// イベントを送出する
			child.dispatchEvent( new SceneEvent( SceneEvent.SCENE_REMOVED, false, false, child ) );
			
			// 関係性を削除する
			child.progression_internal::progression = null;
			child.progression_internal::root = null;
			child.progression_internal::parent = null;
			
			// 子シーンから削除する
			scenes.progression_internal::removeChild( child );
			
			return child;
		}
		
		/*======================================================================*//**
		* シーンオブジェクトの子リストの指定された index 位置から子シーンオブジェクトを削除します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	index			削除するシーンオブジェクトの子インデックスです。
		* @return					scene パラメータで渡すシーンオブジェクトです。
		*/
		public function removeChildAt( index:int ):SceneObject {
			// イベントを送出する
			child.dispatchEvent( new SceneEvent( SceneEvent.SCENE_REMOVED, false, false, child ) );
			
			// 関係性を削除する
			child.progression_internal::progression = null;
			child.progression_internal::root = null;
			child.progression_internal::parent = null;
			
			// 子シーンから削除する
			var child:SceneObject = scenes.progression_internal::removeChildAt( index );
			
			return child;
		}
		
		/*======================================================================*//**
		* シーンオブジェクトの子リストに登録されている子シーンオブジェクトを全て削除します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function removeAllChild():void {
			for each ( var child:SceneObject in scenes ) {
				removeChild( child );
			}
		}
		
		/*======================================================================*//**
		* 指定されたシーンオブジェクトが、SceneObject インスタンスの子であるか、インスタンス自体であるかを指定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	child	テストする子シーンオブジェクトです。
		* @return			child インスタンスが子シーンオブジェクトの場合は true に、それ以外の場合は false になります。
		*/
		public function contains( child:SceneObject ):Boolean {
			return scenes.progression_internal::contains( child );
		}
		
		/*======================================================================*//**
		* 指定のインデックス位置にある子 SceneObject インスタンスを返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	index	子シーンオブジェクトのインデックス位置です。
		* @return			指定されたインデックス位置にある子シーンオブジェクトです。
		*/
		public function getChildAt( index:int ):SceneObject {
			return scenes.progression_internal::getChildAt( index );
		}
		
		/*======================================================================*//**
		* 指定された名前に一致する子シーンオブジェクトを返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	name	条件となる子シーンオブジェクトの名前です。
		* @return			条件に一致した SceneObject インスタンスです。
		*/
		public function getChildByName( name:String ):SceneObject {
			return scenes.progression_internal::getChildByName( name );
		}
		
		/*======================================================================*//**
		* child SceneObject インスタンスのインデックス位置を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	child	特定する SceneObject インスタンスです。
		* @return			特定する子シーンオブジェクトのインデックス位置です。
		*/
		public function getChildIndex( child:SceneObject ):int {
			return scenes.progression_internal::getChildIndex( child );
		}
		
		/*======================================================================*//**
		* シーンオブジェクトコンテナの既存の子の位置を変更します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	child		インデックス番号を変更する子 SceneObject インスタンスです。
		* @param	index		child シーンオブジェクトの結果のインデックス番号です。
		* @param	addAbove	指定したインデックス位置にすでにシーンオブジェクトが存在した場合に、そのシーンオブジェクトの上に配置するかどうかです。
		*/
		public function setChildIndex( child:SceneObject, index:int, addAbove:Boolean = false ):void {
			scenes.progression_internal::setChildIndex( child, index, addAbove );
		}
		
		/*======================================================================*//**
		* 指定された 2 つの子シーンオブジェクトのインデックス順序（重ね順）を入れ替えます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	child1	最初の子シーンオブジェクトです。
		* @param	child2	2 番目の子シーンオブジェクトです。
		*/
		public function swapChildren( child1:SceneObject, child2:SceneObject ):void {
			scenes.progression_internal::swapChildren( child1, child2 );
		}
		
		/*======================================================================*//**
		* 指定されたインデックス位置に該当する 2 つの子シーンオブジェクトのインデックス順序（重ね順）を入れ替えます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	index1	最初の子シーンオブジェクトのインデックス位置です。
		* @param	index2	2 番目の子シーンオブジェクトのインデックス位置です。
		*/
		public function swapChildrenAt( index1:int, index2:int ):void {
			scenes.progression_internal::swapChildrenAt( index1, index2 );
		}
		
		/*======================================================================*//**
		* 特定のイベントが送出された際に実行したい Command インスタンスを追加します。
		* Command インスタンスは、イベントが送出される直前に自動的にクリアされ、送出後に自動的に処理されます。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	commands	追加したいコマンド配列です。
		*/
		public function addCommand( ... commands:Array ):void {
			_commandList.addCommand.apply( null, commands );
		}
		
		/*======================================================================*//**
		* addCommand() メソッドで追加された Command インスタンスをクリアします。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		*/
		public function clearCommand():void {
			_commandList = parallelMode ? new ParallelList() : new SerialList();
		}
		
		/*======================================================================*//**
		* SceneObject インスタンスに対して、複数のプロパティを一括設定します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @param	props	設定したいプロパティを含んだ Object です。
		* @return			設定対象の SceneObject インスタンスです。
		*/
		public function setProperties( props:Object ):SceneObject {
			ObjectUtil.setProperties( this, props );
			return this;
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public override function toString():String {
			return "[object " + _className + " : " + String( sceneId ) + "]";
		}
		
		/*======================================================================*//**
		* 指定されたオブジェクトのストリング表現を返します。
		* @langversion 3.0
		* @playerversion Flash 9.0.45.0
		* @return	オブジェクトのストリング表現です。
		*/
		public function toXMLString():String {
			// XML を作成する
			var cls:String = getQualifiedClassName( this ).replace( new RegExp( "::", "g" ), "." );
			
			// シーンを作成する
			var xml:XML = <scene name={ name } cls={ cls } title={ title } />;
			if ( extra is XML || extra is XMLList ) {
				xml.prependChild( extra );
			}
			
			// 子シーンを作成する
			for each ( var scene:SceneObject in scenes ) {
				cls = getQualifiedClassName( scene ).replace( new RegExp( "::", "g" ), "." );
				var node:XML = <scene name={ scene.name } cls={ cls } title={ scene.title } />;
				node.prependChild( new XML( scene.toXMLString() ).children() );
				xml.prependChild( node );
			}
			
			return xml.toXMLString();
		}
		
		
		
		
		
		/*======================================================================*//**
		* シーン移動時に目的地がシーンオブジェクト自身もしくは子階層だった場合に、階層が変更された瞬間に送出されます。
		*/
		private function _load( e:SceneEvent ):void {
			// イベントハンドラメソッドを実行する
			onLoad();
		}
		
		/*======================================================================*//**
		* シーン移動時に目的地がシーンオブジェクト自身もしくは親階層だった場合に、階層が変更された瞬間に送出されます。
		*/
		private function _unload( e:SceneEvent ):void {
			// イベントハンドラメソッドを実行する
			onUnload();
		}
		
		/*======================================================================*//**
		* シーンオブジェクトが目的地だった場合に、到達した瞬間に送出されます。
		*/
		private function _init( e:SceneEvent ):void {
			// 未読であれば
			if ( !_readed ) {
				// 既読にする
				_readed = true;
				
				// イベントを送出する
				dispatchEvent( new SceneEvent( SceneEvent.SCENE_STATE_CHANGE, false, false, this ) );
			}
			
			// イベントハンドラメソッドを実行する
			onInit();
		}
		
		/*======================================================================*//**
		* シーンオブジェクトが出発地だった場合に、移動を開始した瞬間に送出されます。
		*/
		private function _goto( e:SceneEvent ):void {
			// イベントハンドラメソッドを実行する
			onGoto();
		}
		
		/*======================================================================*//**
		* シーン移動時に目的地がシーンオブジェクトの子階層であり、かつ出発地ではない場合に、移動を中継した瞬間に送出されます。
		*/
		private function _descend( e:SceneEvent ):void {
			// イベントハンドラメソッドを実行する
			onDescend();
		}
		
		/*======================================================================*//**
		* シーン移動時に目的地がシーンオブジェクトの親階層であり、かつ出発地ではない場合に、移動を中継した瞬間に送出されます。
		*/
		private function _ascend( e:SceneEvent ):void {
			// イベントハンドラメソッドを実行する
			onAscend();
		}
		
		/*======================================================================*//**
		* シーンオブジェクトがシーンリストに追加された場合に送出されます。
		*/
		private function _sceneAdded( e:SceneEvent ):void {
			// イベントハンドラメソッドを実行する
			onSceneAdded();
		}
		
		/*======================================================================*//**
		* シーンオブジェクトがシーンリストから削除された場合に送出されます。
		*/
		private function _sceneRemoved( e:SceneEvent ):void {
			// イベントハンドラメソッドを実行する
			onSceneRemoved();
		}
		
		/*======================================================================*//**
		* シーンオブジェクトの状態が変更された場合に送出されます。
		*/
		private function _sceneStateChange( e:SceneEvent ):void {
			// イベントハンドラメソッドを実行する
			onSceneStateChange();
		}
		
		/*======================================================================*//**
		* シーンの移動処理中に対象シーンが変更された場合に送出されます。
		*/
		private function _processScene( e:ProcessEvent ):void {
			// 現在の設定を保存する
			var isCurrent:Boolean = _isCurrent;
			var isParent:Boolean = _isParent;
			var isChild:Boolean = _isChild;
			
			// カレントシーンに該当するかどうか
			_isCurrent = ( this == e.scene );
			
			// 親シーンに該当するかどうか
			_isParent = !( _isCurrent || e.scene.sceneId.contains( sceneId ) );
			
			// 子シーンに該当するかどうか
			_isChild = ( !_isCurrent && e.scene.sceneId.contains( sceneId ) );
			
			// 変化していなければ終了する
			if ( isCurrent == _isCurrent && isParent == _isParent && isChild == _isChild ) { return; }
			
			// イベントを送出する
			dispatchEvent( new SceneEvent( SceneEvent.SCENE_STATE_CHANGE, false, false, this ) );
		}
		
		/*======================================================================*//**
		* データが正常にロードされたときに送出されます。
		*/
		private function _complete( e:Event ):void {
			// XML から子シーンを作成する
			addChildFromXML( XML( ExURLLoader( e.target ).data ) );
			
			// イベントを送出する
			dispatchEvent( e );
		}
		
		/*======================================================================*//**
		* URLLoader.load() の呼び出しによってセキュリティサンドボックスの外部にあるサーバーからデータをロードしようとすると送出されます。
		*/
		private function _securityError( e:SecurityErrorEvent ):void {
			// イベントを送出する
			dispatchEvent( e );
		}
		
		/*======================================================================*//**
		* 入出力エラーが発生してロード処理が失敗したときに送出されます。
		*/
		private function _ioError( e:IOErrorEvent ):void {
			// イベントを送出する
			dispatchEvent( e );
		}
	}
}





